@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the examples of the Qt Toolkit.
 **
@@ -17,8 +17,8 @@
 **     notice, this list of conditions and the following disclaimer in
 **     the documentation and/or other materials provided with the
 **     distribution.
-**   * Neither the name of Digia Plc and its Subsidiary(-ies) nor the names
-**     of its contributors may be used to endorse or promote products derived
+**   * Neither the name of The Qt Company Ltd nor the names of its
+**     contributors may be used to endorse or promote products derived
 **     from this software without specific prior written permission.
 **
 **
@@ -50,6 +50,8 @@ public:
 private slots:
     void quickWidgetStatusChanged(QQuickWidget::Status);
     void sceneGraphError(QQuickWindow::SceneGraphError error, const QString &message);
+    void grabToFile();
+    void renderToFile();
 
 private:
     QQuickWidget *m_quickWidget;
@@ -88,7 +90,11 @@ MainWindow::MainWindow()
 
     setCentralWidget(centralWidget);
 
-    QMenu *fileMenu = menuBar()->addMenu(tr("File"));
+    QMenu *fileMenu = menuBar()->addMenu(tr("&File"));
+    QAction *grabAction = fileMenu->addAction(tr("Grab to image"));
+    connect(grabAction, SIGNAL(triggered()), this, SLOT(grabToFile()));
+    QAction *renderAction = fileMenu->addAction(tr("Render to pixmap"));
+    connect(renderAction, SIGNAL(triggered()), this, SLOT(renderToFile()));
     QAction *quitAction = fileMenu->addAction(tr("Quit"));
     connect(quitAction, SIGNAL(triggered()), qApp, SLOT(quit()));
 }
@@ -108,6 +114,30 @@ void MainWindow::sceneGraphError(QQuickWindow::SceneGraphError, const QString &m
      statusBar()->showMessage(message);
 }
 
+template<class T> void saveToFile(QWidget *parent, T *saveable)
+{
+    QString t;
+    QFileDialog fd(parent, t, QString());
+    fd.setAcceptMode(QFileDialog::AcceptSave);
+    fd.setDefaultSuffix("png");
+    fd.selectFile("test.png");
+    if (fd.exec() == QDialog::Accepted)
+        saveable->save(fd.selectedFiles().first());
+}
+
+void MainWindow::grabToFile()
+{
+    QImage image = m_quickWidget->grabFramebuffer();
+    saveToFile(this, &image);
+}
+
+void MainWindow::renderToFile()
+{
+    QPixmap pixmap(m_quickWidget->size());
+    m_quickWidget->render(&pixmap);
+    saveToFile(this, &pixmap);
+}
+
 int main(int argc, char **argv)
 {
     QApplication app(argc, argv);
