@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the QtQml module of the Qt Toolkit.
 **
@@ -17,8 +17,8 @@
 **     notice, this list of conditions and the following disclaimer in
 **     the documentation and/or other materials provided with the
 **     distribution.
-**   * Neither the name of Digia Plc and its Subsidiary(-ies) nor the names
-**     of its contributors may be used to endorse or promote products derived
+**   * Neither the name of The Qt Company Ltd nor the names of its
+**     contributors may be used to endorse or promote products derived
 **     from this software without specific prior written permission.
 **
 **
@@ -41,11 +41,12 @@
 #include "dialogPlugin.h"
 #include "directory.h"
 #include "file.h"
-#include <QtQml/qqml.h>
+#include <QtQml>
 
 void DialogPlugin::registerTypes(const char *uri)
 {
-    //register the class Directory into QML as a "Directory" element version 1.0
+    // Register the class Directory into QML as a "Directory" type version 1.0
+    // @uri FileDialog
     qmlRegisterType<Directory>(uri, 1, 0, "Directory");
-    qmlRegisterType<File>(uri,1,0,"File");
+    qmlRegisterType<File>(uri, 1, 0, "File");
 }
