@@ -1,40 +1,32 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the examples of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -446,7 +438,7 @@ void QQuickFolderListModel::setFolder(const QUrl &folder)
 
 
 /*!
-   \qmlproperty url QQuickFolderListModel::rootFolder
+   \qmlproperty url FolderListModel::rootFolder
 
    When the rootFolder is set, then this folder will
    be threated as the root in the file system, so that
@@ -479,7 +471,7 @@ void QQuickFolderListModel::setRootFolder(const QUrl &path)
 /*!
     \qmlproperty url FolderListModel::parentFolder
 
-    Returns the URL of the parent of of the current \l folder.
+    Returns the URL of the parent of the current \l folder.
 */
 QUrl QQuickFolderListModel::parentFolder() const
 {
@@ -536,8 +528,8 @@ void QQuickFolderListModel::classBegin()
 void QQuickFolderListModel::componentComplete()
 {
     Q_D(QQuickFolderListModel);
-
-    if (!d->currentDir.isValid() || !d->currentDir.isLocalFile() || !QDir().exists(d->currentDir.toLocalFile()))
+    QString localPath = QQmlFile::urlToLocalFileOrQrc(d->currentDir);
+    if (localPath.isEmpty() || !QDir(localPath).exists())
         setFolder(QUrl::fromLocalFile(QDir::currentPath()));
 }
 
@@ -549,7 +541,7 @@ void QQuickFolderListModel::componentComplete()
     \list
     \li Unsorted - no sorting is applied.
     \li Name - sort by filename
-    \li LastModified - sort by time modified
+    \li Time - sort by time modified
     \li Size - sort by file size
     \li Type - sort by file type (extension)
     \endlist
