@@ -1,40 +1,32 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the QtQml module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -87,7 +79,7 @@ struct PlainVertices {
 /*!
     \qmltype CustomParticle
     \instantiates QQuickCustomParticle
-    \inqmlmodule QtQuick.Particles 2
+    \inqmlmodule QtQuick.Particles
     \inherits ParticlePainter
     \brief For specifying shaders to paint particles
     \ingroup qtquick-particles
@@ -269,6 +261,8 @@ QQuickShaderEffectNode *QQuickCustomParticle::prepareNextFrame(QQuickShaderEffec
         return 0;
 
     if (m_dirtyProgram) {
+        const bool isES = QOpenGLContext::currentContext()->isOpenGLES();
+
         QQuickShaderEffectMaterial *material = static_cast<QQuickShaderEffectMaterial *>(rootNode->material());
         Q_ASSERT(material);
 
@@ -276,17 +270,16 @@ QQuickShaderEffectNode *QQuickCustomParticle::prepareNextFrame(QQuickShaderEffec
         QSGShaderSourceBuilder builder;
         if (s.sourceCode[Key::FragmentShader].isEmpty()) {
             builder.appendSourceFile(QStringLiteral(":/particles/shaders/customparticle.frag"));
-#if defined(QT_OPENGL_ES_2)
-            builder.removeVersion();
-#endif
+            if (isES)
+                builder.removeVersion();
             s.sourceCode[Key::FragmentShader] = builder.source();
             builder.clear();
         }
 
         builder.appendSourceFile(QStringLiteral(":/particles/shaders/customparticletemplate.vert"));
-#if defined(QT_OPENGL_ES_2)
-        builder.removeVersion();
-#endif
+        if (isES)
+            builder.removeVersion();
+
         if (s.sourceCode[Key::VertexShader].isEmpty())
             builder.appendSourceFile(QStringLiteral(":/particles/shaders/customparticle.vert"));
         s.sourceCode[Key::VertexShader] = builder.source() + s.sourceCode[Key::VertexShader];
@@ -310,12 +303,15 @@ QQuickShaderEffectNode *QQuickCustomParticle::prepareNextFrame(QQuickShaderEffec
 
 QQuickShaderEffectNode* QQuickCustomParticle::buildCustomNodes()
 {
-#ifdef QT_OPENGL_ES_2
-    if (m_count * 4 > 0xffff) {
+    typedef QHash<int, QQuickShaderEffectNode*>::const_iterator NodeHashConstIt;
+
+    if (!QOpenGLContext::currentContext())
+        return 0;
+
+    if (QOpenGLContext::currentContext()->isOpenGLES() && m_count * 4 > 0xffff) {
         printf("CustomParticle: Too many particles... \n");
         return 0;
     }
-#endif
 
     if (m_count <= 0) {
         printf("CustomParticle: Too few particles... \n");
@@ -374,10 +370,11 @@ QQuickShaderEffectNode* QQuickCustomParticle::buildCustomNodes()
         }
     }
 
-    QHash<int, QQuickShaderEffectNode*>::const_iterator it = m_nodes.begin();
+    NodeHashConstIt it = m_nodes.begin();
     rootNode = it.value();
     rootNode->setFlag(QSGNode::OwnsMaterial, true);
-    for (++it; it != m_nodes.end(); ++it)
+    const NodeHashConstIt cend = m_nodes.end();
+    for (++it; it != cend; ++it)
         rootNode->appendChildNode(it.value());
 
     return rootNode;
