@@ -1,40 +1,32 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the QtQuick module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -103,7 +95,7 @@ DEFINE_BOOL_CONFIG_OPTION(qmlParticlesDebug, QML_PARTICLES_DEBUG)
 /*!
     \qmltype ParticleSystem
     \instantiates QQuickParticleSystem
-    \inqmlmodule QtQuick.Particles 2
+    \inqmlmodule QtQuick.Particles
     \brief A system which includes particle painter, emitter, and affector types
     \ingroup qtquick-particles
 
@@ -435,11 +427,11 @@ void QQuickParticleGroupData::prepareRecycler(QQuickParticleData* d)
 }
 
 QQuickParticleData::QQuickParticleData(QQuickParticleSystem* sys)
-    : group(0)
-    , e(0)
+    : e(0)
     , system(sys)
     , index(0)
     , systemIndex(-1)
+    , group(0)
     , colorOwner(0)
     , rotationOwner(0)
     , deformationOwner(0)
@@ -486,6 +478,26 @@ QQuickParticleData::~QQuickParticleData()
     delete v8Datum;
 }
 
+QQuickParticleData::QQuickParticleData(const QQuickParticleData &other)
+{
+    *this = other;
+}
+
+QQuickParticleData &QQuickParticleData::operator=(const QQuickParticleData &other)
+{
+    clone(other);
+
+    group = other.group;
+    e = other.e;
+    system = other.system;
+    index = other.index;
+    systemIndex = other.systemIndex;
+    // Lazily initialized
+    v8Datum = 0;
+
+    return *this;
+}
+
 void QQuickParticleData::clone(const QQuickParticleData& other)
 {
     x = other.x;
@@ -919,19 +931,26 @@ void QQuickParticleSystem::emittersChanged()
     if (!m_componentComplete)
         return;
 
-    m_emitters.removeAll(0);
-
-
-    QList<int> previousSizes;
-    QList<int> newSizes;
+    QVector<int> previousSizes;
+    QVector<int> newSizes;
+    previousSizes.reserve(m_nextGroupId);
+    newSizes.reserve(m_nextGroupId);
     for (int i=0; i<m_nextGroupId; i++) {
         previousSizes << groupData[i]->size();
         newSizes << 0;
     }
 
-    foreach (QQuickParticleEmitter* e, m_emitters) {//Populate groups and set sizes.
-        if (!groupIds.contains(e->group())
-                || (!e->group().isEmpty() && !groupIds[e->group()])) {//or it was accidentally inserted by a failed lookup earlier
+    // Populate groups and set sizes.
+    for (int i = 0; i < m_emitters.count(); ++i) {
+        QQuickParticleEmitter *e = m_emitters.at(i);
+        if (!e) {
+            m_emitters.removeAt(i);
+            i--;
+            continue;
+        }
+
+        if (!e->group().isEmpty() &&
+            !groupIds.contains(e->group())) {
             int id = m_nextGroupId++;
             QQuickParticleGroupData* gd = new QQuickParticleGroupData(id, this);
             groupIds.insert(e->group(), id);
