@@ -1,40 +1,32 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the QtQml module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -45,11 +37,18 @@
 #include <QVector>
 #include <QStringList>
 #include <QHash>
+#include <QUrl>
+
 #include <private/qv4value_p.h>
 #include <private/qv4executableallocator_p.h>
+#include <private/qqmlrefcount_p.h>
+#include <private/qqmlnullablevalue_p.h>
 
 QT_BEGIN_NAMESPACE
 
+class QQmlPropertyCache;
+class QQmlPropertyData;
+
 namespace QmlIR {
 struct Document;
 }
@@ -60,7 +59,6 @@ struct Function;
 }
 
 struct Function;
-struct ExecutionContext;
 
 namespace CompiledData {
 
@@ -68,6 +66,11 @@ struct String;
 struct Function;
 struct Lookup;
 struct RegExp;
+struct Unit;
+
+#if defined(Q_CC_MSVC) || defined(Q_CC_GNU)
+#pragma pack(push, 1)
+#endif
 
 struct Location
 {
@@ -82,29 +85,6 @@ struct Location
     }
 };
 
-struct TypeReference
-{
-    TypeReference(const Location &loc)
-        : location(loc)
-        , needsCreation(false)
-        , errorWhenNotFound(false)
-    {}
-    Location location; // first use
-    bool needsCreation : 1; // whether the type needs to be creatable or not
-    bool errorWhenNotFound: 1;
-};
-
-// map from name index to location of first use
-struct TypeReferenceMap : QHash<int, TypeReference>
-{
-    TypeReference &add(int nameIndex, const Location &loc) {
-        Iterator it = find(nameIndex);
-        if (it != end())
-            return *it;
-        return *insert(nameIndex, loc);
-    }
-};
-
 struct RegExp
 {
     enum Flags {
@@ -155,89 +135,10 @@ struct String
     // uint16 strdata[]
 
     static int calculateSize(const QString &str) {
-        return (sizeof(String) + (str.length() + 1) * sizeof(quint16) + 7) & ~0x7;
+        return (sizeof(String) + str.length() * sizeof(quint16) + 7) & ~0x7;
     }
 };
 
-static const char magic_str[] = "qv4cdata";
-
-struct Unit
-{
-    char magic[8];
-    qint16 architecture;
-    qint16 version;
-    quint32 unitSize; // Size of the Unit and any depending data. Does _not_ include size of data needed by QmlUnit.
-
-    enum {
-        IsJavascript = 0x1,
-        IsQml = 0x2,
-        StaticData = 0x4, // Unit data persistent in memory?
-        IsSingleton = 0x8,
-        IsSharedLibrary = 0x10 // .pragma shared?
-    };
-    quint32 flags;
-    uint stringTableSize;
-    uint offsetToStringTable;
-    uint functionTableSize;
-    uint offsetToFunctionTable;
-    uint lookupTableSize;
-    uint offsetToLookupTable;
-    uint regexpTableSize;
-    uint offsetToRegexpTable;
-    uint constantTableSize;
-    uint offsetToConstantTable;
-    uint jsClassTableSize;
-    uint offsetToJSClassTable;
-    qint32 indexOfRootFunction;
-    quint32 sourceFileIndex;
-
-    QString stringAt(int idx) const {
-        const uint *offsetTable = reinterpret_cast<const uint*>((reinterpret_cast<const char *>(this)) + offsetToStringTable);
-        const uint offset = offsetTable[idx];
-        const String *str = reinterpret_cast<const String*>(reinterpret_cast<const char *>(this) + offset);
-        if (str->size == 0)
-            return QString();
-        const QChar *characters = reinterpret_cast<const QChar *>(str + 1);
-        if (flags & StaticData)
-            return QString::fromRawData(characters, str->size);
-        return QString(characters, str->size);
-    }
-
-    const uint *functionOffsetTable() const { return reinterpret_cast<const uint*>((reinterpret_cast<const char *>(this)) + offsetToFunctionTable); }
-
-    const Function *functionAt(int idx) const {
-        const uint *offsetTable = functionOffsetTable();
-        const uint offset = offsetTable[idx];
-        return reinterpret_cast<const Function*>(reinterpret_cast<const char *>(this) + offset);
-    }
-
-    const Lookup *lookupTable() const { return reinterpret_cast<const Lookup*>(reinterpret_cast<const char *>(this) + offsetToLookupTable); }
-    const RegExp *regexpAt(int index) const {
-        return reinterpret_cast<const RegExp*>(reinterpret_cast<const char *>(this) + offsetToRegexpTable + index * sizeof(RegExp));
-    }
-    const QV4::Value *constants() const {
-        return reinterpret_cast<const QV4::Value*>(reinterpret_cast<const char *>(this) + offsetToConstantTable);
-    }
-
-    const JSClassMember *jsClassAt(int idx, int *nMembers) const {
-        const uint *offsetTable = reinterpret_cast<const uint *>(reinterpret_cast<const char *>(this) + offsetToJSClassTable);
-        const uint offset = offsetTable[idx];
-        const char *ptr = reinterpret_cast<const char *>(this) + offset;
-        const JSClass *klass = reinterpret_cast<const JSClass *>(ptr);
-        *nMembers = klass->nMembers;
-        return reinterpret_cast<const JSClassMember*>(ptr + sizeof(JSClass));
-    }
-
-    static int calculateSize(uint headerSize, uint nFunctions, uint nRegExps, uint nConstants,
-                             uint nLookups, uint nClasses) {
-        return (headerSize
-                + (nFunctions + nClasses) * sizeof(uint)
-                + nRegExps * RegExp::calculateSize()
-                + nConstants * sizeof(QV4::ReturnedValue)
-                + nLookups * Lookup::calculateSize()
-                + 7) & ~7; }
-};
-
 struct Function
 {
     enum Flags {
@@ -381,6 +282,8 @@ struct Q_QML_PRIVATE_EXPORT Binding
         return false;
     }
 
+    static QString escapedString(const QString &string);
+
     bool evaluatesToString() const { return type == Type_String || type == Type_Translation || type == Type_TranslationById; }
 
     QString valueAsString(const Unit *unit) const;
@@ -528,10 +431,39 @@ struct Import
     Import(): type(0), uriIndex(0), qualifierIndex(0), majorVersion(0), minorVersion(0) {}
 };
 
-struct QmlUnit
+static const char magic_str[] = "qv4cdata";
+
+struct Unit
 {
-    Unit header;
-    quint32 qmlUnitSize; // size including header and all surrounding data.
+    char magic[8];
+    qint16 architecture;
+    qint16 version;
+    quint32 unitSize; // Size of the Unit and any depending data.
+
+    enum {
+        IsJavascript = 0x1,
+        IsQml = 0x2,
+        StaticData = 0x4, // Unit data persistent in memory?
+        IsSingleton = 0x8,
+        IsSharedLibrary = 0x10 // .pragma shared?
+    };
+    quint32 flags;
+    uint stringTableSize;
+    uint offsetToStringTable;
+    uint functionTableSize;
+    uint offsetToFunctionTable;
+    uint lookupTableSize;
+    uint offsetToLookupTable;
+    uint regexpTableSize;
+    uint offsetToRegexpTable;
+    uint constantTableSize;
+    uint offsetToConstantTable;
+    uint jsClassTableSize;
+    uint offsetToJSClassTable;
+    qint32 indexOfRootFunction;
+    quint32 sourceFileIndex;
+
+    /* QML specific fields */
     quint32 nImports;
     quint32 offsetToImports;
     quint32 nObjects;
@@ -549,41 +481,105 @@ struct QmlUnit
     }
 
     bool isSingleton() const {
-        return header.flags & Unit::IsSingleton;
+        return flags & Unit::IsSingleton;
+    }
+    /* end QML specific fields*/
+
+    QString stringAt(int idx) const {
+        const uint *offsetTable = reinterpret_cast<const uint*>((reinterpret_cast<const char *>(this)) + offsetToStringTable);
+        const uint offset = offsetTable[idx];
+        const String *str = reinterpret_cast<const String*>(reinterpret_cast<const char *>(this) + offset);
+        if (str->size == 0)
+            return QString();
+        const QChar *characters = reinterpret_cast<const QChar *>(str + 1);
+        if (flags & StaticData)
+            return QString::fromRawData(characters, str->size);
+        return QString(characters, str->size);
+    }
+
+    const uint *functionOffsetTable() const { return reinterpret_cast<const uint*>((reinterpret_cast<const char *>(this)) + offsetToFunctionTable); }
+
+    const Function *functionAt(int idx) const {
+        const uint *offsetTable = functionOffsetTable();
+        const uint offset = offsetTable[idx];
+        return reinterpret_cast<const Function*>(reinterpret_cast<const char *>(this) + offset);
+    }
+
+    const Lookup *lookupTable() const { return reinterpret_cast<const Lookup*>(reinterpret_cast<const char *>(this) + offsetToLookupTable); }
+    const RegExp *regexpAt(int index) const {
+        return reinterpret_cast<const RegExp*>(reinterpret_cast<const char *>(this) + offsetToRegexpTable + index * sizeof(RegExp));
+    }
+    const QV4::Value *constants() const {
+        return reinterpret_cast<const QV4::Value*>(reinterpret_cast<const char *>(this) + offsetToConstantTable);
+    }
+
+    const JSClassMember *jsClassAt(int idx, int *nMembers) const {
+        const uint *offsetTable = reinterpret_cast<const uint *>(reinterpret_cast<const char *>(this) + offsetToJSClassTable);
+        const uint offset = offsetTable[idx];
+        const char *ptr = reinterpret_cast<const char *>(this) + offset;
+        const JSClass *klass = reinterpret_cast<const JSClass *>(ptr);
+        *nMembers = klass->nMembers;
+        return reinterpret_cast<const JSClassMember*>(ptr + sizeof(JSClass));
+    }
+
+    static int calculateSize(uint nFunctions, uint nRegExps, uint nConstants,
+                             uint nLookups, uint nClasses) {
+        return (sizeof(Unit)
+                + (nFunctions + nClasses) * sizeof(uint)
+                + nRegExps * RegExp::calculateSize()
+                + nConstants * sizeof(QV4::ReturnedValue)
+                + nLookups * Lookup::calculateSize()
+                + 7) & ~7; }
+};
+
+#if defined(Q_CC_MSVC) || defined(Q_CC_GNU)
+#pragma pack(pop)
+#endif
+
+struct TypeReference
+{
+    TypeReference(const Location &loc)
+        : location(loc)
+        , needsCreation(false)
+        , errorWhenNotFound(false)
+    {}
+    Location location; // first use
+    bool needsCreation : 1; // whether the type needs to be creatable or not
+    bool errorWhenNotFound: 1;
+};
+
+// map from name index to location of first use
+struct TypeReferenceMap : QHash<int, TypeReference>
+{
+    TypeReference &add(int nameIndex, const Location &loc) {
+        Iterator it = find(nameIndex);
+        if (it != end())
+            return *it;
+        return *insert(nameIndex, loc);
     }
 };
 
+// index is per-object binding index
+typedef QVector<QQmlPropertyData*> BindingPropertyData;
+
 // This is how this hooks into the existing structures:
 
 //VM::Function
 //    CompilationUnit * (for functions that need to clean up)
 //    CompiledData::Function *compiledFunction
 
-struct Q_QML_PRIVATE_EXPORT CompilationUnit
+struct Q_QML_PRIVATE_EXPORT CompilationUnit : public QQmlRefCount
 {
 #ifdef V4_BOOTSTRAP
     CompilationUnit()
-        : refCount(0)
-        , data(0)
+        : data(0)
     {}
     virtual ~CompilationUnit() {}
 #else
-    CompilationUnit()
-        : refCount(0)
-        , data(0)
-        , engine(0)
-        , runtimeStrings(0)
-        , runtimeLookups(0)
-        , runtimeRegularExpressions(0)
-        , runtimeClasses(0)
-    {}
+    CompilationUnit();
     virtual ~CompilationUnit();
 #endif
 
-    void ref() { ++refCount; }
-    void deref() { if (!--refCount) delete this; }
-
-    int refCount;
     Unit *data;
 
     // Called only when building QML, when we build the header for JS first and append QML data
@@ -592,12 +588,19 @@ struct Q_QML_PRIVATE_EXPORT CompilationUnit
 #ifndef V4_BOOTSTRAP
     ExecutionEngine *engine;
     QString fileName() const { return data->stringAt(data->sourceFileIndex); }
+    QUrl url() const { if (m_url.isNull) m_url = QUrl(fileName()); return m_url; }
 
-    QV4::StringValue *runtimeStrings; // Array
+    QV4::Heap::String **runtimeStrings; // Array
     QV4::Lookup *runtimeLookups;
     QV4::Value *runtimeRegularExpressions;
     QV4::InternalClass **runtimeClasses;
     QVector<QV4::Function *> runtimeFunctions;
+    mutable QQmlNullableValue<QUrl> m_url;
+
+    // index is object index. This allows fast access to the
+    // property data when initializing bindings, avoiding expensive
+    // lookups by string (property name).
+    QVector<BindingPropertyData> bindingPropertyDataPerObject;
 
     QV4::Function *linkToEngine(QV4::ExecutionEngine *engine);
     void unlink();
