@@ -1,40 +1,32 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the QtQml module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -43,14 +35,12 @@
 #include "qv4dateobject_p.h"
 #include "qv4objectproto_p.h"
 #include "qv4scopedvalue_p.h"
-#include <QtCore/qnumeric.h>
-#include <QtCore/qmath.h>
+#include "qv4runtime_p.h"
+
+#include <QtCore/QDebug>
 #include <QtCore/QDateTime>
 #include <QtCore/QStringList>
-#include <QtCore/QDebug>
-#include <cmath>
-#include <qmath.h>
-#include <qnumeric.h>
+
 #include <time.h>
 
 #include <private/qqmljsengine_p.h>
@@ -65,6 +55,7 @@
 #  else
 #    include "qplatformdefs.h"
 #  endif
+#  include <unistd.h> // for _POSIX_THREAD_SAFE_FUNCTIONS
 #endif
 
 using namespace QV4;
@@ -488,6 +479,8 @@ static inline double ParseString(const QString &s)
     QDateTime dt = QDateTime::fromString(s, Qt::TextDate);
     if (!dt.isValid())
         dt = QDateTime::fromString(s, Qt::ISODate);
+    if (!dt.isValid())
+        dt = QDateTime::fromString(s, Qt::RFC2822Date);
     if (!dt.isValid()) {
         QStringList formats;
         formats << QStringLiteral("M/d/yyyy")
@@ -558,16 +551,7 @@ static inline QDateTime ToDateTime(double t, Qt::TimeSpec spec)
 {
     if (std::isnan(t))
         return QDateTime();
-    if (spec == Qt::LocalTime)
-        t = LocalTime(t);
-    int year = int(YearFromTime(t));
-    int month = int(MonthFromTime(t) + 1);
-    int day = int(DateFromTime(t));
-    int hours = HourFromTime(t);
-    int mins = MinFromTime(t);
-    int secs = SecFromTime(t);
-    int ms = msFromTime(t);
-    return QDateTime(QDate(year, month, day), QTime(hours, mins, secs, ms), spec);
+    return QDateTime::fromMSecsSinceEpoch(t, spec);
 }
 
 static inline QString ToString(double t)
@@ -594,7 +578,7 @@ static inline QString ToUTCString(double t)
 {
     if (std::isnan(t))
         return QStringLiteral("Invalid Date");
-    return ToDateTime(t, Qt::UTC).toString() + QStringLiteral(" GMT");
+    return ToDateTime(t, Qt::UTC).toString();
 }
 
 static inline QString ToDateString(double t)
@@ -643,38 +627,36 @@ static double getLocalTZA()
 
 DEFINE_OBJECT_VTABLE(DateObject);
 
-DateObject::DateObject(ExecutionEngine *engine, const QDateTime &date)
-    : Object(engine->dateClass)
+Heap::DateObject::DateObject(QV4::ExecutionEngine *engine, const QDateTime &date)
+    : Heap::Object(engine->emptyClass, engine->datePrototype.asObject())
 {
-    setVTable(staticVTable());
     value.setDouble(date.isValid() ? date.toMSecsSinceEpoch() : qSNaN());
 }
 
 QDateTime DateObject::toQDateTime() const
 {
-    return ToDateTime(value.asDouble(), Qt::LocalTime);
+    return ToDateTime(date().asDouble(), Qt::LocalTime);
 }
 
 DEFINE_OBJECT_VTABLE(DateCtor);
 
-DateCtor::DateCtor(ExecutionContext *scope)
-    : FunctionObject(scope, QStringLiteral("Date"))
+Heap::DateCtor::DateCtor(QV4::ExecutionContext *scope)
+    : Heap::FunctionObject(scope, QStringLiteral("Date"))
 {
-    setVTable(staticVTable());
 }
 
 ReturnedValue DateCtor::construct(Managed *m, CallData *callData)
 {
+    Scope scope(static_cast<DateCtor *>(m)->engine());
     double t = 0;
 
     if (callData->argc == 0)
         t = currentTime();
 
     else if (callData->argc == 1) {
-        Scope scope(m->engine());
         ScopedValue arg(scope, callData->args[0]);
         if (DateObject *d = arg->asDateObject())
-            arg = d->value;
+            arg = d->date();
         else
             arg = RuntimeHelpers::toPrimitive(arg, PREFERREDTYPE_HINT);
 
@@ -698,16 +680,16 @@ ReturnedValue DateCtor::construct(Managed *m, CallData *callData)
         t = TimeClip(UTC(t));
     }
 
-    return Encode(m->engine()->newDateObject(Primitive::fromDouble(t)));
+    return Encode(scope.engine->newDateObject(Primitive::fromDouble(t)));
 }
 
 ReturnedValue DateCtor::call(Managed *m, CallData *)
 {
     double t = currentTime();
-    return m->engine()->newString(ToString(t))->asReturnedValue();
+    return static_cast<DateCtor *>(m)->engine()->newString(ToString(t))->asReturnedValue();
 }
 
-void DatePrototype::init(ExecutionEngine *engine, ObjectRef ctor)
+void DatePrototype::init(ExecutionEngine *engine, Object *ctor)
 {
     Scope scope(engine);
     ScopedObject o(scope);
@@ -770,32 +752,32 @@ void DatePrototype::init(ExecutionEngine *engine, ObjectRef ctor)
 
 double DatePrototype::getThisDate(ExecutionContext *ctx)
 {
-    if (DateObject *thisObject = ctx->callData->thisObject.asDateObject())
-        return thisObject->value.asDouble();
+    if (DateObject *thisObject = ctx->thisObject().asDateObject())
+        return thisObject->date().asDouble();
     else {
-        ctx->throwTypeError();
+        ctx->engine()->throwTypeError();
         return 0;
     }
 }
 
 ReturnedValue DatePrototype::method_parse(CallContext *ctx)
 {
-    if (!ctx->callData->argc)
+    if (!ctx->argc())
         return Encode(qSNaN());
-    return Encode(ParseString(ctx->callData->args[0].toString(ctx)->toQString()));
+    return Encode(ParseString(ctx->args()[0].toQString()));
 }
 
 ReturnedValue DatePrototype::method_UTC(CallContext *ctx)
 {
-    const int numArgs = ctx->callData->argc;
+    const int numArgs = ctx->argc();
     if (numArgs >= 2) {
-        double year  = ctx->callData->args[0].toNumber();
-        double month = ctx->callData->args[1].toNumber();
-        double day   = numArgs >= 3 ? ctx->callData->args[2].toNumber() : 1;
-        double hours = numArgs >= 4 ? ctx->callData->args[3].toNumber() : 0;
-        double mins  = numArgs >= 5 ? ctx->callData->args[4].toNumber() : 0;
-        double secs  = numArgs >= 6 ? ctx->callData->args[5].toNumber() : 0;
-        double ms    = numArgs >= 7 ? ctx->callData->args[6].toNumber() : 0;
+        double year  = ctx->args()[0].toNumber();
+        double month = ctx->args()[1].toNumber();
+        double day   = numArgs >= 3 ? ctx->args()[2].toNumber() : 1;
+        double hours = numArgs >= 4 ? ctx->args()[3].toNumber() : 0;
+        double mins  = numArgs >= 5 ? ctx->args()[4].toNumber() : 0;
+        double secs  = numArgs >= 6 ? ctx->args()[5].toNumber() : 0;
+        double ms    = numArgs >= 7 ? ctx->args()[6].toNumber() : 0;
         if (year >= 0 && year <= 99)
             year += 1900;
         double t = MakeDate(MakeDay(year, month, day),
@@ -815,37 +797,37 @@ ReturnedValue DatePrototype::method_now(CallContext *ctx)
 ReturnedValue DatePrototype::method_toString(CallContext *ctx)
 {
     double t = getThisDate(ctx);
-    return ctx->engine->newString(ToString(t))->asReturnedValue();
+    return ctx->d()->engine->newString(ToString(t))->asReturnedValue();
 }
 
 ReturnedValue DatePrototype::method_toDateString(CallContext *ctx)
 {
     double t = getThisDate(ctx);
-    return ctx->engine->newString(ToDateString(t))->asReturnedValue();
+    return ctx->d()->engine->newString(ToDateString(t))->asReturnedValue();
 }
 
 ReturnedValue DatePrototype::method_toTimeString(CallContext *ctx)
 {
     double t = getThisDate(ctx);
-    return ctx->engine->newString(ToTimeString(t))->asReturnedValue();
+    return ctx->d()->engine->newString(ToTimeString(t))->asReturnedValue();
 }
 
 ReturnedValue DatePrototype::method_toLocaleString(CallContext *ctx)
 {
     double t = getThisDate(ctx);
-    return ctx->engine->newString(ToLocaleString(t))->asReturnedValue();
+    return ctx->d()->engine->newString(ToLocaleString(t))->asReturnedValue();
 }
 
 ReturnedValue DatePrototype::method_toLocaleDateString(CallContext *ctx)
 {
     double t = getThisDate(ctx);
-    return ctx->engine->newString(ToLocaleDateString(t))->asReturnedValue();
+    return ctx->d()->engine->newString(ToLocaleDateString(t))->asReturnedValue();
 }
 
 ReturnedValue DatePrototype::method_toLocaleTimeString(CallContext *ctx)
 {
     double t = getThisDate(ctx);
-    return ctx->engine->newString(ToLocaleTimeString(t))->asReturnedValue();
+    return ctx->d()->engine->newString(ToLocaleTimeString(t))->asReturnedValue();
 }
 
 ReturnedValue DatePrototype::method_valueOf(CallContext *ctx)
@@ -1007,196 +989,196 @@ ReturnedValue DatePrototype::method_getTimezoneOffset(CallContext *ctx)
 ReturnedValue DatePrototype::method_setTime(CallContext *ctx)
 {
     Scope scope(ctx);
-    Scoped<DateObject> self(scope, ctx->callData->thisObject);
+    Scoped<DateObject> self(scope, ctx->thisObject());
     if (!self)
-        return ctx->throwTypeError();
+        return ctx->engine()->throwTypeError();
 
-    double t = ctx->callData->argc ? ctx->callData->args[0].toNumber() : qSNaN();
-    self->value.setDouble(TimeClip(t));
-    return self->value.asReturnedValue();
+    double t = ctx->argc() ? ctx->args()[0].toNumber() : qSNaN();
+    self->date().setDouble(TimeClip(t));
+    return self->date().asReturnedValue();
 }
 
 ReturnedValue DatePrototype::method_setMilliseconds(CallContext *ctx)
 {
     Scope scope(ctx);
-    Scoped<DateObject> self(scope, ctx->callData->thisObject);
+    Scoped<DateObject> self(scope, ctx->thisObject());
     if (!self)
-        return ctx->throwTypeError();
+        return ctx->engine()->throwTypeError();
 
-    double t = LocalTime(self->value.asDouble());
-    double ms = ctx->callData->argc ? ctx->callData->args[0].toNumber() : qSNaN();
-    self->value.setDouble(TimeClip(UTC(MakeDate(Day(t), MakeTime(HourFromTime(t), MinFromTime(t), SecFromTime(t), ms)))));
-    return self->value.asReturnedValue();
+    double t = LocalTime(self->date().asDouble());
+    double ms = ctx->argc() ? ctx->args()[0].toNumber() : qSNaN();
+    self->date().setDouble(TimeClip(UTC(MakeDate(Day(t), MakeTime(HourFromTime(t), MinFromTime(t), SecFromTime(t), ms)))));
+    return self->date().asReturnedValue();
 }
 
 ReturnedValue DatePrototype::method_setUTCMilliseconds(CallContext *ctx)
 {
-    DateObject *self = ctx->callData->thisObject.asDateObject();
+    DateObject *self = ctx->thisObject().asDateObject();
     if (!self)
-        return ctx->throwTypeError();
+        return ctx->engine()->throwTypeError();
 
-    double t = self->value.asDouble();
-    double ms = ctx->callData->argc ? ctx->callData->args[0].toNumber() : qSNaN();
-    self->value.setDouble(TimeClip(UTC(MakeDate(Day(t), MakeTime(HourFromTime(t), MinFromTime(t), SecFromTime(t), ms)))));
-    return self->value.asReturnedValue();
+    double t = self->date().asDouble();
+    double ms = ctx->argc() ? ctx->args()[0].toNumber() : qSNaN();
+    self->date().setDouble(TimeClip(MakeDate(Day(t), MakeTime(HourFromTime(t), MinFromTime(t), SecFromTime(t), ms))));
+    return self->date().asReturnedValue();
 }
 
 ReturnedValue DatePrototype::method_setSeconds(CallContext *ctx)
 {
-    DateObject *self = ctx->callData->thisObject.asDateObject();
+    DateObject *self = ctx->thisObject().asDateObject();
     if (!self)
-        return ctx->throwTypeError();
+        return ctx->engine()->throwTypeError();
 
-    double t = LocalTime(self->value.asDouble());
-    double sec = ctx->callData->argc ? ctx->callData->args[0].toNumber() : qSNaN();
-    double ms = (ctx->callData->argc < 2) ? msFromTime(t) : ctx->callData->args[1].toNumber();
+    double t = LocalTime(self->date().asDouble());
+    double sec = ctx->argc() ? ctx->args()[0].toNumber() : qSNaN();
+    double ms = (ctx->argc() < 2) ? msFromTime(t) : ctx->args()[1].toNumber();
     t = TimeClip(UTC(MakeDate(Day(t), MakeTime(HourFromTime(t), MinFromTime(t), sec, ms))));
-    self->value.setDouble(t);
-    return self->value.asReturnedValue();
+    self->date().setDouble(t);
+    return self->date().asReturnedValue();
 }
 
 ReturnedValue DatePrototype::method_setUTCSeconds(CallContext *ctx)
 {
-    DateObject *self = ctx->callData->thisObject.asDateObject();
+    DateObject *self = ctx->thisObject().asDateObject();
     if (!self)
-        return ctx->throwTypeError();
+        return ctx->engine()->throwTypeError();
 
-    double t = self->value.asDouble();
-    double sec = ctx->callData->argc ? ctx->callData->args[0].toNumber() : qSNaN();
-    double ms = (ctx->callData->argc < 2) ? msFromTime(t) : ctx->callData->args[1].toNumber();
-    t = TimeClip(UTC(MakeDate(Day(t), MakeTime(HourFromTime(t), MinFromTime(t), sec, ms))));
-    self->value.setDouble(t);
-    return self->value.asReturnedValue();
+    double t = self->date().asDouble();
+    double sec = ctx->argc() ? ctx->args()[0].toNumber() : qSNaN();
+    double ms = (ctx->argc() < 2) ? msFromTime(t) : ctx->args()[1].toNumber();
+    t = TimeClip(MakeDate(Day(t), MakeTime(HourFromTime(t), MinFromTime(t), sec, ms)));
+    self->date().setDouble(t);
+    return self->date().asReturnedValue();
 }
 
 ReturnedValue DatePrototype::method_setMinutes(CallContext *ctx)
 {
-    DateObject *self = ctx->callData->thisObject.asDateObject();
+    DateObject *self = ctx->thisObject().asDateObject();
     if (!self)
-        return ctx->throwTypeError();
+        return ctx->engine()->throwTypeError();
 
-    double t = LocalTime(self->value.asDouble());
-    double min = ctx->callData->argc ? ctx->callData->args[0].toNumber() : qSNaN();
-    double sec = (ctx->callData->argc < 2) ? SecFromTime(t) : ctx->callData->args[1].toNumber();
-    double ms = (ctx->callData->argc < 3) ? msFromTime(t) : ctx->callData->args[2].toNumber();
+    double t = LocalTime(self->date().asDouble());
+    double min = ctx->argc() ? ctx->args()[0].toNumber() : qSNaN();
+    double sec = (ctx->argc() < 2) ? SecFromTime(t) : ctx->args()[1].toNumber();
+    double ms = (ctx->argc() < 3) ? msFromTime(t) : ctx->args()[2].toNumber();
     t = TimeClip(UTC(MakeDate(Day(t), MakeTime(HourFromTime(t), min, sec, ms))));
-    self->value.setDouble(t);
-    return self->value.asReturnedValue();
+    self->date().setDouble(t);
+    return self->date().asReturnedValue();
 }
 
 ReturnedValue DatePrototype::method_setUTCMinutes(CallContext *ctx)
 {
-    DateObject *self = ctx->callData->thisObject.asDateObject();
+    DateObject *self = ctx->thisObject().asDateObject();
     if (!self)
-        return ctx->throwTypeError();
+        return ctx->engine()->throwTypeError();
 
-    double t = self->value.asDouble();
-    double min = ctx->callData->argc ? ctx->callData->args[0].toNumber() : qSNaN();
-    double sec = (ctx->callData->argc < 2) ? SecFromTime(t) : ctx->callData->args[1].toNumber();
-    double ms = (ctx->callData->argc < 3) ? msFromTime(t) : ctx->callData->args[2].toNumber();
-    t = TimeClip(UTC(MakeDate(Day(t), MakeTime(HourFromTime(t), min, sec, ms))));
-    self->value.setDouble(t);
-    return self->value.asReturnedValue();
+    double t = self->date().asDouble();
+    double min = ctx->argc() ? ctx->args()[0].toNumber() : qSNaN();
+    double sec = (ctx->argc() < 2) ? SecFromTime(t) : ctx->args()[1].toNumber();
+    double ms = (ctx->argc() < 3) ? msFromTime(t) : ctx->args()[2].toNumber();
+    t = TimeClip(MakeDate(Day(t), MakeTime(HourFromTime(t), min, sec, ms)));
+    self->date().setDouble(t);
+    return self->date().asReturnedValue();
 }
 
 ReturnedValue DatePrototype::method_setHours(CallContext *ctx)
 {
-    DateObject *self = ctx->callData->thisObject.asDateObject();
+    DateObject *self = ctx->thisObject().asDateObject();
     if (!self)
-        return ctx->throwTypeError();
+        return ctx->engine()->throwTypeError();
 
-    double t = LocalTime(self->value.asDouble());
-    double hour = ctx->callData->argc ? ctx->callData->args[0].toNumber() : qSNaN();
-    double min = (ctx->callData->argc < 2) ? MinFromTime(t) : ctx->callData->args[1].toNumber();
-    double sec = (ctx->callData->argc < 3) ? SecFromTime(t) : ctx->callData->args[2].toNumber();
-    double ms = (ctx->callData->argc < 4) ? msFromTime(t) : ctx->callData->args[3].toNumber();
+    double t = LocalTime(self->date().asDouble());
+    double hour = ctx->argc() ? ctx->args()[0].toNumber() : qSNaN();
+    double min = (ctx->argc() < 2) ? MinFromTime(t) : ctx->args()[1].toNumber();
+    double sec = (ctx->argc() < 3) ? SecFromTime(t) : ctx->args()[2].toNumber();
+    double ms = (ctx->argc() < 4) ? msFromTime(t) : ctx->args()[3].toNumber();
     t = TimeClip(UTC(MakeDate(Day(t), MakeTime(hour, min, sec, ms))));
-    self->value.setDouble(t);
-    return self->value.asReturnedValue();
+    self->date().setDouble(t);
+    return self->date().asReturnedValue();
 }
 
 ReturnedValue DatePrototype::method_setUTCHours(CallContext *ctx)
 {
-    DateObject *self = ctx->callData->thisObject.asDateObject();
+    DateObject *self = ctx->thisObject().asDateObject();
     if (!self)
-        return ctx->throwTypeError();
+        return ctx->engine()->throwTypeError();
 
-    double t = self->value.asDouble();
-    double hour = ctx->callData->argc ? ctx->callData->args[0].toNumber() : qSNaN();
-    double min = (ctx->callData->argc < 2) ? MinFromTime(t) : ctx->callData->args[1].toNumber();
-    double sec = (ctx->callData->argc < 3) ? SecFromTime(t) : ctx->callData->args[2].toNumber();
-    double ms = (ctx->callData->argc < 4) ? msFromTime(t) : ctx->callData->args[3].toNumber();
-    t = TimeClip(UTC(MakeDate(Day(t), MakeTime(hour, min, sec, ms))));
-    self->value.setDouble(t);
-    return self->value.asReturnedValue();
+    double t = self->date().asDouble();
+    double hour = ctx->argc() ? ctx->args()[0].toNumber() : qSNaN();
+    double min = (ctx->argc() < 2) ? MinFromTime(t) : ctx->args()[1].toNumber();
+    double sec = (ctx->argc() < 3) ? SecFromTime(t) : ctx->args()[2].toNumber();
+    double ms = (ctx->argc() < 4) ? msFromTime(t) : ctx->args()[3].toNumber();
+    t = TimeClip(MakeDate(Day(t), MakeTime(hour, min, sec, ms)));
+    self->date().setDouble(t);
+    return self->date().asReturnedValue();
 }
 
 ReturnedValue DatePrototype::method_setDate(CallContext *ctx)
 {
-    DateObject *self = ctx->callData->thisObject.asDateObject();
+    DateObject *self = ctx->thisObject().asDateObject();
     if (!self)
-        return ctx->throwTypeError();
+        return ctx->engine()->throwTypeError();
 
-    double t = LocalTime(self->value.asDouble());
-    double date = ctx->callData->argc ? ctx->callData->args[0].toNumber() : qSNaN();
+    double t = LocalTime(self->date().asDouble());
+    double date = ctx->argc() ? ctx->args()[0].toNumber() : qSNaN();
     t = TimeClip(UTC(MakeDate(MakeDay(YearFromTime(t), MonthFromTime(t), date), TimeWithinDay(t))));
-    self->value.setDouble(t);
-    return self->value.asReturnedValue();
+    self->date().setDouble(t);
+    return self->date().asReturnedValue();
 }
 
 ReturnedValue DatePrototype::method_setUTCDate(CallContext *ctx)
 {
-    DateObject *self = ctx->callData->thisObject.asDateObject();
+    DateObject *self = ctx->thisObject().asDateObject();
     if (!self)
-        return ctx->throwTypeError();
+        return ctx->engine()->throwTypeError();
 
-    double t = self->value.asDouble();
-    double date = ctx->callData->argc ? ctx->callData->args[0].toNumber() : qSNaN();
-    t = TimeClip(UTC(MakeDate(MakeDay(YearFromTime(t), MonthFromTime(t), date), TimeWithinDay(t))));
-    self->value.setDouble(t);
-    return self->value.asReturnedValue();
+    double t = self->date().asDouble();
+    double date = ctx->argc() ? ctx->args()[0].toNumber() : qSNaN();
+    t = TimeClip(MakeDate(MakeDay(YearFromTime(t), MonthFromTime(t), date), TimeWithinDay(t)));
+    self->date().setDouble(t);
+    return self->date().asReturnedValue();
 }
 
 ReturnedValue DatePrototype::method_setMonth(CallContext *ctx)
 {
-    DateObject *self = ctx->callData->thisObject.asDateObject();
+    DateObject *self = ctx->thisObject().asDateObject();
     if (!self)
-        return ctx->throwTypeError();
+        return ctx->engine()->throwTypeError();
 
-    double t = LocalTime(self->value.asDouble());
-    double month = ctx->callData->argc ? ctx->callData->args[0].toNumber() : qSNaN();
-    double date = (ctx->callData->argc < 2) ? DateFromTime(t) : ctx->callData->args[1].toNumber();
+    double t = LocalTime(self->date().asDouble());
+    double month = ctx->argc() ? ctx->args()[0].toNumber() : qSNaN();
+    double date = (ctx->argc() < 2) ? DateFromTime(t) : ctx->args()[1].toNumber();
     t = TimeClip(UTC(MakeDate(MakeDay(YearFromTime(t), month, date), TimeWithinDay(t))));
-    self->value.setDouble(t);
-    return self->value.asReturnedValue();
+    self->date().setDouble(t);
+    return self->date().asReturnedValue();
 }
 
 ReturnedValue DatePrototype::method_setUTCMonth(CallContext *ctx)
 {
-    DateObject *self = ctx->callData->thisObject.asDateObject();
+    DateObject *self = ctx->thisObject().asDateObject();
     if (!self)
-        return ctx->throwTypeError();
+        return ctx->engine()->throwTypeError();
 
-    double t = self->value.asDouble();
-    double month = ctx->callData->argc ? ctx->callData->args[0].toNumber() : qSNaN();
-    double date = (ctx->callData->argc < 2) ? DateFromTime(t) : ctx->callData->args[1].toNumber();
-    t = TimeClip(UTC(MakeDate(MakeDay(YearFromTime(t), month, date), TimeWithinDay(t))));
-    self->value.setDouble(t);
-    return self->value.asReturnedValue();
+    double t = self->date().asDouble();
+    double month = ctx->argc() ? ctx->args()[0].toNumber() : qSNaN();
+    double date = (ctx->argc() < 2) ? DateFromTime(t) : ctx->args()[1].toNumber();
+    t = TimeClip(MakeDate(MakeDay(YearFromTime(t), month, date), TimeWithinDay(t)));
+    self->date().setDouble(t);
+    return self->date().asReturnedValue();
 }
 
 ReturnedValue DatePrototype::method_setYear(CallContext *ctx)
 {
-    DateObject *self = ctx->callData->thisObject.asDateObject();
+    DateObject *self = ctx->thisObject().asDateObject();
     if (!self)
-        return ctx->throwTypeError();
+        return ctx->engine()->throwTypeError();
 
-    double t = self->value.asDouble();
+    double t = self->date().asDouble();
     if (std::isnan(t))
         t = 0;
     else
         t = LocalTime(t);
-    double year = ctx->callData->argc ? ctx->callData->args[0].toNumber() : qSNaN();
+    double year = ctx->argc() ? ctx->args()[0].toNumber() : qSNaN();
     double r;
     if (std::isnan(year)) {
         r = qSNaN();
@@ -1207,50 +1189,50 @@ ReturnedValue DatePrototype::method_setYear(CallContext *ctx)
         r = UTC(MakeDate(r, TimeWithinDay(t)));
         r = TimeClip(r);
     }
-    self->value.setDouble(r);
-    return self->value.asReturnedValue();
+    self->date().setDouble(r);
+    return self->date().asReturnedValue();
 }
 
 ReturnedValue DatePrototype::method_setUTCFullYear(CallContext *ctx)
 {
-    DateObject *self = ctx->callData->thisObject.asDateObject();
+    DateObject *self = ctx->thisObject().asDateObject();
     if (!self)
-        return ctx->throwTypeError();
+        return ctx->engine()->throwTypeError();
 
-    double t = self->value.asDouble();
-    double year = ctx->callData->argc ? ctx->callData->args[0].toNumber() : qSNaN();
-    double month = (ctx->callData->argc < 2) ? MonthFromTime(t) : ctx->callData->args[1].toNumber();
-    double date = (ctx->callData->argc < 3) ? DateFromTime(t) : ctx->callData->args[2].toNumber();
-    t = TimeClip(UTC(MakeDate(MakeDay(year, month, date), TimeWithinDay(t))));
-    self->value.setDouble(t);
-    return self->value.asReturnedValue();
+    double t = self->date().asDouble();
+    double year = ctx->argc() ? ctx->args()[0].toNumber() : qSNaN();
+    double month = (ctx->argc() < 2) ? MonthFromTime(t) : ctx->args()[1].toNumber();
+    double date = (ctx->argc() < 3) ? DateFromTime(t) : ctx->args()[2].toNumber();
+    t = TimeClip(MakeDate(MakeDay(year, month, date), TimeWithinDay(t)));
+    self->date().setDouble(t);
+    return self->date().asReturnedValue();
 }
 
 ReturnedValue DatePrototype::method_setFullYear(CallContext *ctx)
 {
-    DateObject *self = ctx->callData->thisObject.asDateObject();
+    DateObject *self = ctx->thisObject().asDateObject();
     if (!self)
-        return ctx->throwTypeError();
+        return ctx->engine()->throwTypeError();
 
-    double t = LocalTime(self->value.asDouble());
+    double t = LocalTime(self->date().asDouble());
     if (std::isnan(t))
         t = 0;
-    double year = ctx->callData->argc ? ctx->callData->args[0].toNumber() : qSNaN();
-    double month = (ctx->callData->argc < 2) ? MonthFromTime(t) : ctx->callData->args[1].toNumber();
-    double date = (ctx->callData->argc < 3) ? DateFromTime(t) : ctx->callData->args[2].toNumber();
+    double year = ctx->argc() ? ctx->args()[0].toNumber() : qSNaN();
+    double month = (ctx->argc() < 2) ? MonthFromTime(t) : ctx->args()[1].toNumber();
+    double date = (ctx->argc() < 3) ? DateFromTime(t) : ctx->args()[2].toNumber();
     t = TimeClip(UTC(MakeDate(MakeDay(year, month, date), TimeWithinDay(t))));
-    self->value.setDouble(t);
-    return self->value.asReturnedValue();
+    self->date().setDouble(t);
+    return self->date().asReturnedValue();
 }
 
 ReturnedValue DatePrototype::method_toUTCString(CallContext *ctx)
 {
-    DateObject *self = ctx->callData->thisObject.asDateObject();
+    DateObject *self = ctx->thisObject().asDateObject();
     if (!self)
-        return ctx->throwTypeError();
+        return ctx->engine()->throwTypeError();
 
-    double t = self->value.asDouble();
-    return ctx->engine->newString(ToUTCString(t))->asReturnedValue();
+    double t = self->date().asDouble();
+    return ctx->d()->engine->newString(ToUTCString(t))->asReturnedValue();
 }
 
 static void addZeroPrefixedInt(QString &str, int num, int nDigits)
@@ -1268,19 +1250,19 @@ static void addZeroPrefixedInt(QString &str, int num, int nDigits)
 
 ReturnedValue DatePrototype::method_toISOString(CallContext *ctx)
 {
-    DateObject *self = ctx->callData->thisObject.asDateObject();
+    DateObject *self = ctx->thisObject().asDateObject();
     if (!self)
-        return ctx->throwTypeError();
+        return ctx->engine()->throwTypeError();
 
-    double t = self->value.asDouble();
+    double t = self->date().asDouble();
     if (!std::isfinite(t))
-        return ctx->throwRangeError(ctx->callData->thisObject);
+        return ctx->engine()->throwRangeError(ctx->thisObject());
 
     QString result;
     int year = (int)YearFromTime(t);
     if (year < 0 || year > 9999) {
         if (qAbs(year) >= 1000000)
-            return ctx->engine->newString(QStringLiteral("Invalid Date"))->asReturnedValue();
+            return ctx->d()->engine->newString(QStringLiteral("Invalid Date"))->asReturnedValue();
         result += year < 0 ? QLatin1Char('-') : QLatin1Char('+');
         year = qAbs(year);
         addZeroPrefixedInt(result, year, 6);
@@ -1301,27 +1283,27 @@ ReturnedValue DatePrototype::method_toISOString(CallContext *ctx)
     addZeroPrefixedInt(result, msFromTime(t), 3);
     result += QLatin1Char('Z');
 
-    return ctx->engine->newString(result)->asReturnedValue();
+    return ctx->d()->engine->newString(result)->asReturnedValue();
 }
 
 ReturnedValue DatePrototype::method_toJSON(CallContext *ctx)
 {
     Scope scope(ctx);
-    ScopedValue O(scope, RuntimeHelpers::toObject(ctx, ValueRef(&ctx->callData->thisObject)));
+    ScopedValue O(scope, RuntimeHelpers::toObject(scope.engine, ctx->thisObject()));
     ScopedValue tv(scope, RuntimeHelpers::toPrimitive(O, NUMBER_HINT));
 
     if (tv->isNumber() && !std::isfinite(tv->toNumber()))
         return Encode::null();
 
-    ScopedString s(scope, ctx->engine->newString(QStringLiteral("toISOString")));
+    ScopedString s(scope, ctx->d()->engine->newString(QStringLiteral("toISOString")));
     ScopedValue v(scope, O->objectValue()->get(s));
     FunctionObject *toIso = v->asFunctionObject();
 
     if (!toIso)
-        return ctx->throwTypeError();
+        return ctx->engine()->throwTypeError();
 
-    ScopedCallData callData(scope, 0);
-    callData->thisObject = ctx->callData->thisObject;
+    ScopedCallData callData(scope);
+    callData->thisObject = ctx->thisObject();
     return toIso->call(callData);
 }
 
