@@ -1,40 +1,32 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the QtQml module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -53,7 +45,7 @@
 // We mean it.
 //
 
-#include <private/qtqmlglobal_p.h>
+#include <private/qv4global_p.h>
 #include <QtCore/qstring.h>
 #include <QtCore/qurl.h>
 
@@ -63,7 +55,7 @@ class Q_QML_PRIVATE_EXPORT QQmlPool
 {
 public:
     // The class has a destructor that needs to be called
-    class Class { 
+    class Class {
     public:
         inline QQmlPool *pool() const;
 
@@ -82,7 +74,7 @@ public:
     class POD {
     public:
         inline QQmlPool *pool() const;
-        
+
     private:
         void *operator new(size_t);
         void *operator new(size_t, void *m) { return m; }
@@ -120,16 +112,16 @@ public:
         int count() const {
             return m_length;
         }
-        int length() const { 
-            return m_length; 
+        int length() const {
+            return m_length;
         }
-        const T &at(int index) const { 
-            Q_ASSERT(index < m_length); 
-            return m_data[index]; 
+        const T &at(int index) const {
+            Q_ASSERT(index < m_length);
+            return m_data[index];
         };
         T &operator[](int index) {
-            Q_ASSERT(index < m_length); 
-            return m_data[index]; 
+            Q_ASSERT(index < m_length);
+            return m_data[index];
         };
         const T *data() const { return m_data; }
     private:
