@@ -1,40 +1,32 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the QtQml module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -95,12 +87,12 @@ QT_BEGIN_NAMESPACE
 class QQmlComponentExtension : public QV8Engine::Deletable
 {
 public:
-    QQmlComponentExtension(QV8Engine *);
+    QQmlComponentExtension(QV4::ExecutionEngine *v4);
     virtual ~QQmlComponentExtension();
 
     QV4::PersistentValue incubationProto;
 };
-V8_DEFINE_EXTENSION(QQmlComponentExtension, componentExtension);
+V4_DEFINE_EXTENSION(QQmlComponentExtension, componentExtension);
 
 /*!
     \class QQmlComponent
@@ -255,9 +247,9 @@ V8_DEFINE_EXTENSION(QQmlComponentExtension, componentExtension);
     \table
     \row
     \li MyItem.qml
-    \li main.qml
-    \row
     \li \snippet qml/component/MyItem.qml 0
+    \row
+    \li main.qml
     \li \snippet qml/component/main.qml 0
     \endtable
 */
@@ -591,7 +583,7 @@ QQmlComponent::QQmlComponent(QQmlEngine *engine, QQmlCompiledData *cc, int start
     d->cc = cc;
     cc->addref();
     d->start = start;
-    d->url = cc->url;
+    d->url = cc->url();
     d->progress = 1.0;
 }
 
@@ -683,9 +675,9 @@ void QQmlComponentPrivate::loadUrl(const QUrl &newUrl, QQmlComponent::Compilatio
         emit q->progressChanged(progress);
     }
 
-    QQmlDataLoader::Mode loaderMode = (mode == QQmlComponent::Asynchronous)
-            ? QQmlDataLoader::Asynchronous
-            : QQmlDataLoader::PreferSynchronous;
+    QQmlTypeLoader::Mode loaderMode = (mode == QQmlComponent::Asynchronous)
+            ? QQmlTypeLoader::Asynchronous
+            : QQmlTypeLoader::PreferSynchronous;
 
     QQmlTypeData *data = QQmlEnginePrivate::get(engine)->typeLoader.getType(url, loaderMode);
 
@@ -1078,11 +1070,25 @@ void QQmlComponent::create(QQmlIncubator &incubator, QQmlContext *context,
 
 class QQmlComponentIncubator;
 
-class QmlIncubatorObject : public QV4::Object
+namespace QV4 {
+
+namespace Heap {
+
+struct QmlIncubatorObject : Object {
+    QmlIncubatorObject(QV4::ExecutionEngine *engine, QQmlIncubator::IncubationMode = QQmlIncubator::Asynchronous);
+    QScopedPointer<QQmlComponentIncubator> incubator;
+    QPointer<QObject> parent;
+    QV4::Value valuemap;
+    QV4::Value qmlGlobal;
+    QV4::Value statusChanged;
+};
+
+}
+
+struct QmlIncubatorObject : public QV4::Object
 {
-    V4_OBJECT
-public:
-    QmlIncubatorObject(QV8Engine *engine, QQmlIncubator::IncubationMode = QQmlIncubator::Asynchronous);
+    V4_OBJECT2(QmlIncubatorObject, Object)
+    V4_NEEDS_DESTROY
 
     static QV4::ReturnedValue method_get_statusChanged(QV4::CallContext *ctx);
     static QV4::ReturnedValue method_set_statusChanged(QV4::CallContext *ctx);
@@ -1090,39 +1096,37 @@ public:
     static QV4::ReturnedValue method_get_object(QV4::CallContext *ctx);
     static QV4::ReturnedValue method_forceCompletion(QV4::CallContext *ctx);
 
-    static void destroy(Managed *that);
-    static void markObjects(Managed *that, QV4::ExecutionEngine *e);
-
-    QScopedPointer<QQmlComponentIncubator> incubator;
-    QV8Engine *v8;
-    QPointer<QObject> parent;
-    QV4::Value valuemap;
-    QV4::Value qmlGlobal;
-    QV4::Value m_statusChanged;
+    static void markObjects(QV4::Heap::Base *that, QV4::ExecutionEngine *e);
 
     void statusChanged(QQmlIncubator::Status);
     void setInitialState(QObject *);
 };
 
-DEFINE_OBJECT_VTABLE(QmlIncubatorObject);
+}
+
+DEFINE_OBJECT_VTABLE(QV4::QmlIncubatorObject);
 
 class QQmlComponentIncubator : public QQmlIncubator
 {
 public:
-    QQmlComponentIncubator(QmlIncubatorObject *inc, IncubationMode mode)
+    QQmlComponentIncubator(QV4::Heap::QmlIncubatorObject *inc, IncubationMode mode)
         : QQmlIncubator(mode)
         , incubatorObject(inc)
     {}
 
     virtual void statusChanged(Status s) {
-        incubatorObject->statusChanged(s);
+        QV4::Scope scope(incubatorObject->internalClass->engine);
+        QV4::Scoped<QV4::QmlIncubatorObject> i(scope, incubatorObject);
+        i->statusChanged(s);
     }
 
     virtual void setInitialState(QObject *o) {
-        incubatorObject->setInitialState(o);
+        QV4::Scope scope(incubatorObject->internalClass->engine);
+        QV4::Scoped<QV4::QmlIncubatorObject> i(scope, incubatorObject);
+        i->setInitialState(o);
     }
 
-    QmlIncubatorObject *incubatorObject;
+    QV4::Heap::QmlIncubatorObject *incubatorObject;
 };
 
 
@@ -1240,7 +1244,7 @@ void QQmlComponent::createObject(QQmlV4Function *args)
         QV4::ScopedValue f(scope, QV4::Script::evaluate(v4, QString::fromLatin1(INITIALPROPERTIES_SOURCE), qmlglobal));
         Q_ASSERT(f->asFunctionObject());
         QV4::ScopedCallData callData(scope, 2);
-        callData->thisObject = v4->globalObject;
+        callData->thisObject = v4->globalObject();
         callData->args[0] = object;
         callData->args[1] = valuemap;
         f->asFunctionObject()->call(callData);
@@ -1255,7 +1259,7 @@ void QQmlComponent::createObject(QQmlV4Function *args)
     if (!rv)
         args->setReturnValue(QV4::Encode::null());
     else
-        args->setReturnValue(object.asReturnedValue());
+        args->setReturnValue(object->asReturnedValue());
 }
 
 /*!
@@ -1356,19 +1360,19 @@ void QQmlComponent::incubateObject(QQmlV4Function *args)
             mode = QQmlIncubator::AsynchronousIfNested;
     }
 
-    QQmlComponentExtension *e = componentExtension(args->engine());
+    QQmlComponentExtension *e = componentExtension(args->v4engine());
 
-    QV4::Scoped<QmlIncubatorObject> r(scope, new (v4->memoryManager) QmlIncubatorObject(args->engine(), mode));
+    QV4::Scoped<QV4::QmlIncubatorObject> r(scope, v4->memoryManager->alloc<QV4::QmlIncubatorObject>(args->v4engine(), mode));
     QV4::ScopedObject p(scope, e->incubationProto.value());
-    r->setPrototype(p.getPointer());
+    r->setPrototype(p);
 
     if (!valuemap->isUndefined()) {
-        r->valuemap = valuemap;
-        r->qmlGlobal = args->qmlGlobal();
+        r->d()->valuemap = valuemap;
+        r->d()->qmlGlobal = args->qmlGlobal();
     }
-    r->parent = parent;
+    r->d()->parent = parent;
 
-    QQmlIncubator *incubator = r.getPointer()->incubator.data();
+    QQmlIncubator *incubator = r->d()->incubator.data();
     create(*incubator, creationContext());
 
     if (incubator->status() == QQmlIncubator::Null) {
@@ -1379,93 +1383,91 @@ void QQmlComponent::incubateObject(QQmlV4Function *args)
 }
 
 // XXX used by QSGLoader
-void QQmlComponentPrivate::initializeObjectWithInitialProperties(const QV4::ValueRef qmlGlobal, const QV4::ValueRef valuemap, QObject *toCreate)
+void QQmlComponentPrivate::initializeObjectWithInitialProperties(const QV4::Value &qmlGlobal, const QV4::Value &valuemap, QObject *toCreate)
 {
     QQmlEnginePrivate *ep = QQmlEnginePrivate::get(engine);
-    QV8Engine *v8engine = ep->v8engine();
-    QV4::ExecutionEngine *v4engine = QV8Engine::getV4(v8engine);
+    QV4::ExecutionEngine *v4engine = QV8Engine::getV4(ep->v8engine());
     QV4::Scope scope(v4engine);
 
     QV4::ScopedValue object(scope, QV4::QObjectWrapper::wrap(v4engine, toCreate));
     Q_ASSERT(object->asObject());
 
-    if (!valuemap->isUndefined()) {
+    if (!valuemap.isUndefined()) {
         QV4::ScopedObject qmlGlobalObj(scope, qmlGlobal);
-        QV4::Scoped<QV4::FunctionObject>  f(scope, QV4::Script::evaluate(QV8Engine::getV4(v8engine),
-                                                                    QString::fromLatin1(INITIALPROPERTIES_SOURCE), qmlGlobalObj));
+        QV4::ScopedFunctionObject f(scope, QV4::Script::evaluate(v4engine,
+                                                                 QString::fromLatin1(INITIALPROPERTIES_SOURCE), qmlGlobalObj));
         QV4::ScopedCallData callData(scope, 2);
-        callData->thisObject = v4engine->globalObject;
+        callData->thisObject = v4engine->globalObject();
         callData->args[0] = object;
         callData->args[1] = valuemap;
         f->call(callData);
     }
 }
 
-QQmlComponentExtension::QQmlComponentExtension(QV8Engine *engine)
+QQmlComponentExtension::QQmlComponentExtension(QV4::ExecutionEngine *v4)
 {
-    QV4::ExecutionEngine *v4 = QV8Engine::getV4(engine);
     QV4::Scope scope(v4);
-    QV4::Scoped<QV4::Object> proto(scope, v4->newObject());
+    QV4::ScopedObject proto(scope, v4->newObject());
     proto->defineAccessorProperty(QStringLiteral("onStatusChanged"),
-                                  QmlIncubatorObject::method_get_statusChanged, QmlIncubatorObject::method_set_statusChanged);
-    proto->defineAccessorProperty(QStringLiteral("status"), QmlIncubatorObject::method_get_status, 0);
-    proto->defineAccessorProperty(QStringLiteral("object"), QmlIncubatorObject::method_get_object, 0);
-    proto->defineDefaultProperty(QStringLiteral("forceCompletion"), QmlIncubatorObject::method_forceCompletion);
+                                  QV4::QmlIncubatorObject::method_get_statusChanged, QV4::QmlIncubatorObject::method_set_statusChanged);
+    proto->defineAccessorProperty(QStringLiteral("status"), QV4::QmlIncubatorObject::method_get_status, 0);
+    proto->defineAccessorProperty(QStringLiteral("object"), QV4::QmlIncubatorObject::method_get_object, 0);
+    proto->defineDefaultProperty(QStringLiteral("forceCompletion"), QV4::QmlIncubatorObject::method_forceCompletion);
 
-    incubationProto = proto;
+    incubationProto.set(v4, proto);
 }
 
-QV4::ReturnedValue QmlIncubatorObject::method_get_object(QV4::CallContext *ctx)
+QV4::ReturnedValue QV4::QmlIncubatorObject::method_get_object(QV4::CallContext *ctx)
 {
     QV4::Scope scope(ctx);
-    QV4::Scoped<QmlIncubatorObject> o(scope, ctx->callData->thisObject.as<QmlIncubatorObject>());
+    QV4::Scoped<QmlIncubatorObject> o(scope, ctx->thisObject().as<QmlIncubatorObject>());
     if (!o)
-        return ctx->throwTypeError();
+        return ctx->engine()->throwTypeError();
 
-    return QV4::QObjectWrapper::wrap(ctx->engine, o->incubator->object());
+    return QV4::QObjectWrapper::wrap(ctx->d()->engine, o->d()->incubator->object());
 }
 
-QV4::ReturnedValue QmlIncubatorObject::method_forceCompletion(QV4::CallContext *ctx)
+QV4::ReturnedValue QV4::QmlIncubatorObject::method_forceCompletion(QV4::CallContext *ctx)
 {
     QV4::Scope scope(ctx);
-    QV4::Scoped<QmlIncubatorObject> o(scope, ctx->callData->thisObject.as<QmlIncubatorObject>());
+    QV4::Scoped<QmlIncubatorObject> o(scope, ctx->thisObject().as<QmlIncubatorObject>());
     if (!o)
-        return ctx->throwTypeError();
+        return ctx->engine()->throwTypeError();
 
-    o->incubator->forceCompletion();
+    o->d()->incubator->forceCompletion();
 
     return QV4::Encode::undefined();
 }
 
-QV4::ReturnedValue QmlIncubatorObject::method_get_status(QV4::CallContext *ctx)
+QV4::ReturnedValue QV4::QmlIncubatorObject::method_get_status(QV4::CallContext *ctx)
 {
     QV4::Scope scope(ctx);
-    QV4::Scoped<QmlIncubatorObject> o(scope, ctx->callData->thisObject.as<QmlIncubatorObject>());
+    QV4::Scoped<QmlIncubatorObject> o(scope, ctx->thisObject().as<QmlIncubatorObject>());
     if (!o)
-        return ctx->throwTypeError();
+        return ctx->engine()->throwTypeError();
 
-    return QV4::Encode(o->incubator->status());
+    return QV4::Encode(o->d()->incubator->status());
 }
 
-QV4::ReturnedValue QmlIncubatorObject::method_get_statusChanged(QV4::CallContext *ctx)
+QV4::ReturnedValue QV4::QmlIncubatorObject::method_get_statusChanged(QV4::CallContext *ctx)
 {
     QV4::Scope scope(ctx);
-    QV4::Scoped<QmlIncubatorObject> o(scope, ctx->callData->thisObject.as<QmlIncubatorObject>());
+    QV4::Scoped<QmlIncubatorObject> o(scope, ctx->thisObject().as<QmlIncubatorObject>());
     if (!o)
-        return ctx->throwTypeError();
+        return ctx->engine()->throwTypeError();
 
-    return o->m_statusChanged.asReturnedValue();
+    return o->d()->statusChanged.asReturnedValue();
 }
 
-QV4::ReturnedValue QmlIncubatorObject::method_set_statusChanged(QV4::CallContext *ctx)
+QV4::ReturnedValue QV4::QmlIncubatorObject::method_set_statusChanged(QV4::CallContext *ctx)
 {
     QV4::Scope scope(ctx);
-    QV4::Scoped<QmlIncubatorObject> o(scope, ctx->callData->thisObject.as<QmlIncubatorObject>());
-    if (!o || ctx->callData->argc < 1)
-        return ctx->throwTypeError();
+    QV4::Scoped<QmlIncubatorObject> o(scope, ctx->thisObject().as<QmlIncubatorObject>());
+    if (!o || ctx->argc() < 1)
+        return ctx->engine()->throwTypeError();
 
 
-    o->m_statusChanged = ctx->callData->args[0];
+    o->d()->statusChanged = ctx->args()[0];
     return QV4::Encode::undefined();
 }
 
@@ -1473,74 +1475,62 @@ QQmlComponentExtension::~QQmlComponentExtension()
 {
 }
 
-QmlIncubatorObject::QmlIncubatorObject(QV8Engine *engine, QQmlIncubator::IncubationMode m)
-    : Object(QV8Engine::getV4(engine))
+QV4::Heap::QmlIncubatorObject::QmlIncubatorObject(ExecutionEngine *engine, QQmlIncubator::IncubationMode m)
+    : QV4::Heap::Object(engine)
+    , valuemap(QV4::Primitive::undefinedValue())
+    , qmlGlobal(QV4::Primitive::undefinedValue())
+    , statusChanged(QV4::Primitive::undefinedValue())
 {
     incubator.reset(new QQmlComponentIncubator(this, m));
-    v8 = engine;
-    setVTable(staticVTable());
-
-    valuemap = QV4::Primitive::undefinedValue();
-    qmlGlobal = QV4::Primitive::undefinedValue();
-    m_statusChanged = QV4::Primitive::undefinedValue();
 }
 
-void QmlIncubatorObject::setInitialState(QObject *o)
+void QV4::QmlIncubatorObject::setInitialState(QObject *o)
 {
-    QQmlComponent_setQmlParent(o, parent);
+    QQmlComponent_setQmlParent(o, d()->parent);
 
-    if (!valuemap.isUndefined()) {
-        QV4::ExecutionEngine *v4 = QV8Engine::getV4(v8);
+    if (!d()->valuemap.isUndefined()) {
+        QV4::ExecutionEngine *v4 = engine();
         QV4::Scope scope(v4);
 
-        QV4::Scoped<QV4::FunctionObject> f(scope, QV4::Script::evaluate(v4, QString::fromLatin1(INITIALPROPERTIES_SOURCE), qmlGlobal));
+        QV4::ScopedFunctionObject f(scope, QV4::Script::evaluate(v4, QString::fromLatin1(INITIALPROPERTIES_SOURCE), d()->qmlGlobal.asObject()));
         QV4::ScopedCallData callData(scope, 2);
-        callData->thisObject = v4->globalObject;
+        callData->thisObject = v4->globalObject();
         callData->args[0] = QV4::QObjectWrapper::wrap(v4, o);
-        callData->args[1] = valuemap;
+        callData->args[1] = d()->valuemap;
         f->call(callData);
     }
 }
 
-void QmlIncubatorObject::destroy(Managed *that)
+void QV4::QmlIncubatorObject::markObjects(QV4::Heap::Base *that, QV4::ExecutionEngine *e)
 {
-    QmlIncubatorObject *o = that->as<QmlIncubatorObject>();
-    Q_ASSERT(o);
-    o->~QmlIncubatorObject();
-}
-
-void QmlIncubatorObject::markObjects(QV4::Managed *that, QV4::ExecutionEngine *e)
-{
-    QmlIncubatorObject *o = that->as<QmlIncubatorObject>();
-    Q_ASSERT(o);
+    QmlIncubatorObject::Data *o = static_cast<QmlIncubatorObject::Data *>(that);
     o->valuemap.mark(e);
     o->qmlGlobal.mark(e);
-    o->m_statusChanged.mark(e);
+    o->statusChanged.mark(e);
     Object::markObjects(that, e);
 }
 
-void QmlIncubatorObject::statusChanged(QQmlIncubator::Status s)
+void QV4::QmlIncubatorObject::statusChanged(QQmlIncubator::Status s)
 {
-    QV4::Scope scope(QV8Engine::getV4(v8));
+    QV4::Scope scope(engine());
     // hold the incubated object in a scoped value to prevent it's destruction before this method returns
-    QV4::ScopedObject incubatedObject(scope, QV4::QObjectWrapper::wrap(scope.engine, incubator->object()));
+    QV4::ScopedObject incubatedObject(scope, QV4::QObjectWrapper::wrap(scope.engine, d()->incubator->object()));
 
     if (s == QQmlIncubator::Ready) {
-        Q_ASSERT(QQmlData::get(incubator->object()));
-        QQmlData::get(incubator->object())->explicitIndestructibleSet = false;
-        QQmlData::get(incubator->object())->indestructible = false;
+        Q_ASSERT(QQmlData::get(d()->incubator->object()));
+        QQmlData::get(d()->incubator->object())->explicitIndestructibleSet = false;
+        QQmlData::get(d()->incubator->object())->indestructible = false;
     }
 
-    QV4::ScopedFunctionObject f(scope, m_statusChanged);
+    QV4::ScopedFunctionObject f(scope, d()->statusChanged);
     if (f) {
-        QV4::ExecutionContext *ctx = scope.engine->currentContext();
         QV4::ScopedCallData callData(scope, 1);
         callData->thisObject = this;
         callData->args[0] = QV4::Primitive::fromUInt32(s);
         f->call(callData);
         if (scope.hasException()) {
-            QQmlError error = QV4::ExecutionEngine::catchExceptionAsQmlError(ctx);
-            QQmlEnginePrivate::warning(QQmlEnginePrivate::get(v8->engine()), error);
+            QQmlError error = scope.engine->catchExceptionAsQmlError();
+            QQmlEnginePrivate::warning(QQmlEnginePrivate::get(scope.engine->qmlEngine()), error);
         }
     }
 }
