@@ -1,40 +1,32 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the QtQml module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -268,6 +260,18 @@ void QQmlEnginePrivate::activateDesignerMode()
         on producing the image without blocking the main thread.
 */
 
+/*!
+    \fn QQmlImageProviderBase::imageType() const
+
+    Implement this method to return the image type supported by this image provider.
+*/
+
+/*!
+    \fn QQmlImageProviderBase::flags() const
+
+    Implement this to return the properties of this image provider.
+*/
+
 /*! \internal */
 QQmlImageProviderBase::QQmlImageProviderBase()
 {
@@ -304,7 +308,7 @@ Text {
 
 \section1 Enums
 
-The Qt object contains the enums available in the \l {Qt Namespace}. For example, you can access
+The Qt object contains the enums available in the \l [QtCore]{Qt}{Qt Namespace}. For example, you can access
 the \l Qt::LeftButton and \l Qt::RightButton enumeration values as \c Qt.LeftButton and \c Qt.RightButton.
 
 
@@ -376,7 +380,7 @@ The following functions are also on the Qt object.
 
 /*!
     \qmlproperty object Qt::platform
-    \since 4.8
+    \since 5.1
 
     The \c platform object provides info about the underlying platform.
 
@@ -400,6 +404,8 @@ The following functions are also on the Qt object.
         \li \c "unix" - Other Unix-based OS
         \li \c "windows" - Windows
         \li \c "wince" - Windows CE
+        \li \c "winrt" - Windows RT
+        \li \c "winphone" - Windows Phone
     \endlist
     \endtable
 */
@@ -499,6 +505,12 @@ The following functions are also on the Qt object.
     \li \c application.domain
     \li This is the organization domain set on the QCoreApplication instance. This property can be written
     to in order to set the organization domain.
+
+    \row
+    \li \c application.supportsMultipleWindows
+    \li This read-only property can be used to determine whether or not the
+        platform supports multiple windows. Some embedded platforms do not support
+        multiple windows, for example.
     \endtable
 
     The object also has one signal, aboutToQuit(), which is the same as \l QCoreApplication::aboutToQuit().
@@ -525,6 +537,34 @@ The following functions are also on the Qt object.
     further details.
 */
 
+/*!
+    \qmlproperty object Qt::styleHints
+    \since 5.5
+
+    The \c styleHints object provides platform-specific style hints and settings.
+    See the QStyleHints documentation for further details.
+
+    \note The \c styleHints object is only available when using the Qt Quick module.
+
+    The following example uses the \c styleHints object to determine whether an
+    item should gain focus on mouse press or touch release:
+    \code
+    import QtQuick 2.4
+
+    MouseArea {
+        id: button
+
+        onPressed: {
+            if (!Qt.styleHints.setFocusOnTouchRelease)
+                button.forceActiveFocus()
+        }
+        onReleased: {
+            if (Qt.styleHints.setFocusOnTouchRelease)
+                button.forceActiveFocus()
+        }
+    }
+    \endcode
+*/
 
 /*!
 \qmlmethod object Qt::include(string url, jsobject callback)
@@ -555,19 +595,21 @@ the same object as is returned from the Qt.include() call.
 
 QQmlEnginePrivate::QQmlEnginePrivate(QQmlEngine *e)
 : propertyCapture(0), rootContext(0), isDebugging(false),
-  profiler(0), outputWarningsToStdErr(true),
+  profiler(0), outputWarningsToMsgLog(true),
   cleanup(0), erroredBindings(0), inProgressCreations(0),
   workerScriptEngine(0),
   activeObjectCreator(0),
   networkAccessManager(0), networkAccessManagerFactory(0), urlInterceptor(0),
   scarceResourcesRefCount(0), typeLoader(e), importDatabase(e), uniqueId(1),
-  incubatorCount(0), incubationController(0), mutex(QMutex::Recursive)
+  incubatorCount(0), incubationController(0)
 {
-    useNewCompiler = true;
 }
 
 QQmlEnginePrivate::~QQmlEnginePrivate()
 {
+    typedef QHash<QPair<QQmlType *, int>, QQmlPropertyCache *>::Iterator TypePropertyCacheIt;
+    typedef QHash<int, QQmlCompiledData *>::Iterator CompositeTypesIt;
+
     if (inProgressCreations)
         qWarning() << QQmlEngine::tr("There are still \"%1\" items in the process of being created at engine destruction.").arg(inProgressCreations);
 
@@ -585,15 +627,16 @@ QQmlEnginePrivate::~QQmlEnginePrivate()
     if (incubationController) incubationController->d = 0;
     incubationController = 0;
 
-    delete rootContext;
-    rootContext = 0;
-
-    for(QHash<const QMetaObject *, QQmlPropertyCache *>::Iterator iter = propertyCache.begin(); iter != propertyCache.end(); ++iter)
+    for (TypePropertyCacheIt iter = typePropertyCache.begin(), end = typePropertyCache.end(); iter != end; ++iter)
         (*iter)->release();
-    for(QHash<QPair<QQmlType *, int>, QQmlPropertyCache *>::Iterator iter = typePropertyCache.begin(); iter != typePropertyCache.end(); ++iter)
-        (*iter)->release();
-    for (QHash<int, QQmlCompiledData *>::Iterator iter = m_compositeTypes.begin(); iter != m_compositeTypes.end(); ++iter)
+    for (CompositeTypesIt iter = m_compositeTypes.begin(), end = m_compositeTypes.end(); iter != end; ++iter) {
         iter.value()->isRegisteredWithEngine = false;
+
+        // since unregisterInternalCompositeType() will not be called in this
+        // case, we have to clean up the type registration manually
+        QMetaType::unregisterType(iter.value()->metaTypeId);
+        QMetaType::unregisterType(iter.value()->listMetaTypeId);
+    }
     delete profiler;
 }
 
@@ -869,7 +912,7 @@ QQuickWorkerScriptEngine *QQmlEnginePrivate::getWorkerScriptEngine()
 
   Note that the \l {Qt Quick 1} version is called QDeclarativeEngine.
 
-  \sa QQmlComponent, QQmlContext
+  \sa QQmlComponent, QQmlContext, {QML Global Object}
 */
 
 /*!
@@ -907,6 +950,8 @@ QQmlEngine::~QQmlEngine()
     if (d->isDebugging)
         QQmlDebugServer::instance()->removeEngine(this);
 
+    d->typeLoader.invalidate();
+
     // Emit onDestruction signals for the root context before
     // we destroy the contexts, engine, Singleton Types etc. that
     // may be required to handle the destruction signal.
@@ -919,6 +964,9 @@ QQmlEngine::~QQmlEngine()
     QList<QQmlType*> singletonTypes = QQmlMetaType::qmlSingletonTypes();
     foreach (QQmlType *currType, singletonTypes)
         currType->singletonInstanceInfo()->destroy(this);
+
+    delete d->rootContext;
+    d->rootContext = 0;
 }
 
 /*! \fn void QQmlEngine::quit()
@@ -1029,7 +1077,7 @@ QQmlAbstractUrlInterceptor *QQmlEngine::urlInterceptor() const
 void QQmlEngine::setNetworkAccessManagerFactory(QQmlNetworkAccessManagerFactory *factory)
 {
     Q_D(QQmlEngine);
-    QMutexLocker locker(&d->mutex);
+    QMutexLocker locker(&d->networkAccessManagerMutex);
     d->networkAccessManagerFactory = factory;
 }
 
@@ -1056,7 +1104,7 @@ void QQmlEnginePrivate::registerFinalizeCallback(QObject *obj, int index)
 
 QNetworkAccessManager *QQmlEnginePrivate::createNetworkAccessManager(QObject *parent) const
 {
-    QMutexLocker locker(&mutex);
+    QMutexLocker locker(&networkAccessManagerMutex);
     QNetworkAccessManager *nam;
     if (networkAccessManagerFactory) {
         nam = networkAccessManagerFactory->create(parent);
@@ -1181,7 +1229,7 @@ void QQmlEngine::setBaseUrl(const QUrl &url)
 bool QQmlEngine::outputWarningsToStandardError() const
 {
     Q_D(const QQmlEngine);
-    return d->outputWarningsToStdErr;
+    return d->outputWarningsToMsgLog;
 }
 
 /*!
@@ -1197,7 +1245,7 @@ bool QQmlEngine::outputWarningsToStandardError() const
 void QQmlEngine::setOutputWarningsToStandardError(bool enabled)
 {
     Q_D(QQmlEngine);
-    d->outputWarningsToStdErr = enabled;
+    d->outputWarningsToMsgLog = enabled;
 }
 
 /*!
@@ -1253,42 +1301,38 @@ void QQmlEngine::setContextForObject(QObject *object, QQmlContext *context)
 /*!
   \enum QQmlEngine::ObjectOwnership
 
-  Ownership controls whether or not QML automatically destroys the
-  QObject when the object is garbage collected by the JavaScript
-  engine.  The two ownership options are:
+  ObjectOwnership controls whether or not QML automatically destroys the
+  QObject when the corresponding JavaScript object is garbage collected by the
+  engine. The two ownership options are:
 
-  \value CppOwnership The object is owned by C++ code, and will
-  never be deleted by QML.  The JavaScript destroy() method cannot be
-  used on objects with CppOwnership.  This option is similar to
-  QScriptEngine::QtOwnership.
+  \value CppOwnership The object is owned by C++ code and QML will never delete
+  it. The JavaScript destroy() method cannot be used on these objects. This
+  option is similar to QScriptEngine::QtOwnership.
 
-  \value JavaScriptOwnership The object is owned by JavaScript.
-  When the object is returned to QML as the return value of a method
-  call or property access, QML will track it, and delete the object
-  if there are no remaining JavaScript references to it and it has no
-  QObject::parent().  An object tracked by one QQmlEngine
-  will be deleted during that QQmlEngine's destructor, and thus
-  JavaScript references between objects with JavaScriptOwnership from
-  two different engines will not be valid after the deletion of one of
-  those engines.  This option is similar to QScriptEngine::ScriptOwnership.
+  \value JavaScriptOwnership The object is owned by JavaScript. When the object
+  is returned to QML as the return value of a method call, QML will track it
+  and delete it if there are no remaining JavaScript references to it and
+  it has no QObject::parent(). An object tracked by one QQmlEngine will be
+  deleted during that QQmlEngine's destructor. Thus, JavaScript references
+  between objects with JavaScriptOwnership from two different engines will
+  not be valid if one of these engines is deleted. This option is similar to
+  QScriptEngine::ScriptOwnership.
 
   Generally an application doesn't need to set an object's ownership
-  explicitly.  QML uses a heuristic to set the default object
-  ownership.  By default, an object that is created by QML has
-  JavaScriptOwnership.  The exception to this are the root objects
-  created by calling QQmlComponent::create() or
-  QQmlComponent::beginCreate() which have CppOwnership by
-  default.  The ownership of these root-level objects is considered to
-  have been transferred to the C++ caller.
+  explicitly. QML uses a heuristic to set the default ownership. By default, an
+  object that is created by QML has JavaScriptOwnership. The exception to this
+  are the root objects created by calling QQmlComponent::create() or
+  QQmlComponent::beginCreate(), which have CppOwnership by default. The
+  ownership of these root-level objects is considered to have been transferred
+  to the C++ caller.
 
-  Objects not-created by QML have CppOwnership by default.  The
-  exception to this is objects returned from C++ method calls; in these cases,
-  the ownership of the returned objects will be set to JavaScriptOwnerShip.
-  Note this applies only to explicit invocations of Q_INVOKABLE methods or slots,
-  and not to property getter invocations.
+  Objects not-created by QML have CppOwnership by default. The exception to this
+  are objects returned from C++ method calls; their ownership will be set to
+  JavaScriptOwnership. This applies only to explicit invocations of Q_INVOKABLE
+  methods or slots, but not to property getter invocations.
 
-  Calling setObjectOwnership() overrides the default ownership
-  heuristic used by QML.
+  Calling setObjectOwnership() overrides the default ownership heuristic used by
+  QML.
 */
 
 /*!
@@ -1322,6 +1366,9 @@ QQmlEngine::ObjectOwnership QQmlEngine::objectOwnership(QObject *object)
         return ddata->indestructible?CppOwnership:JavaScriptOwnership;
 }
 
+/*!
+   \reimp
+*/
 bool QQmlEngine::event(QEvent *e)
 {
     Q_D(QQmlEngine);
@@ -1453,9 +1500,42 @@ QQmlDebuggingEnabler::QQmlDebuggingEnabler(bool printWarning)
         qDebug("QML debugging is enabled. Only use this in a safe environment.");
     }
     QQmlEnginePrivate::qml_debugging_enabled = true;
+#else
+    Q_UNUSED(printWarning);
 #endif
 }
 
+/*!
+ * \enum QQmlDebuggingEnabler::StartMode
+ *
+ * Defines the debug server's start behavior. You can interrupt QML engines starting while a debug
+ * client is connecting, in order to set breakpoints in or profile startup code.
+ *
+ * \value DoNotWaitForClient Run any QML engines as usual while the debug services are connecting.
+ * \value WaitForClient      If a QML engine starts while the debug services are connecting,
+ *                           interrupt it until they are done.
+ */
+
+/*!
+ * Enables debugging for QML engines created after calling this function. The debug server will
+ * listen on \a port at \a hostName and block the QML engine until it receives a connection if
+ * \a mode is \c WaitForClient. If \a mode is not specified it won't block and if \a hostName is not
+ * specified it will listen on all available interfaces. You can only start one debug server at a
+ * time. A debug server may have already been started if the -qmljsdebugger= command line argument
+ * was given. This method returns \c true if a new debug server was successfully started, or
+ * \c false otherwise.
+ */
+bool QQmlDebuggingEnabler::startTcpDebugServer(int port, StartMode mode, const QString &hostName)
+{
+#ifndef QQML_NO_DEBUG_PROTOCOL
+    return QQmlDebugServer::enable(port, port, mode == WaitForClient, hostName);
+#else
+    Q_UNUSED(port);
+    Q_UNUSED(block);
+    Q_UNUSED(hostName);
+    return false;
+#endif
+}
 
 class QQmlDataExtended {
 public:
@@ -1475,16 +1555,28 @@ QQmlDataExtended::~QQmlDataExtended()
 
 void QQmlData::NotifyList::layout(QQmlNotifierEndpoint *endpoint)
 {
-    if (endpoint->next)
-        layout(endpoint->next);
+    // Add a temporary sentinel at beginning of list. This will be overwritten
+    // when the end point is inserted into the notifies further down.
+    endpoint->prev = 0;
 
-    int index = endpoint->sourceSignal;
-    index = qMin(index, 0xFFFF - 1);
+    while (endpoint->next) {
+        Q_ASSERT(reinterpret_cast<QQmlNotifierEndpoint *>(endpoint->next->prev) == endpoint);
+        endpoint = endpoint->next;
+    }
 
-    endpoint->next = notifies[index];
-    if (endpoint->next) endpoint->next->prev = &endpoint->next;
-    endpoint->prev = &notifies[index];
-    notifies[index] = endpoint;
+    while (endpoint) {
+        QQmlNotifierEndpoint *ep = (QQmlNotifierEndpoint *) endpoint->prev;
+
+        int index = endpoint->sourceSignal;
+        index = qMin(index, 0xFFFF - 1);
+
+        endpoint->next = notifies[index];
+        if (endpoint->next) endpoint->next->prev = &endpoint->next;
+        endpoint->prev = &notifies[index];
+        notifies[index] = endpoint;
+
+        endpoint = ep;
+    }
 }
 
 void QQmlData::NotifyList::layout()
@@ -1642,7 +1734,7 @@ void QQmlData::destroyed(QObject *object)
         signalHandler = next;
     }
 
-    if (bindingBits)
+    if (bindingBitsSize > 32)
         free(bindingBits);
 
     if (propertyCache)
@@ -1690,14 +1782,24 @@ void QQmlData::parentChanged(QObject *object, QObject *parent)
 
 static void QQmlData_setBit(QQmlData *data, QObject *obj, int bit)
 {
+    if (data->bindingBitsSize == 0 && bit < 32) {
+        data->bindingBitsSize = 32;
+    }
+
     if (data->bindingBitsSize <= bit) {
         int props = QQmlMetaObject(obj).propertyCount();
         Q_ASSERT(bit < 2 * props);
 
         int arraySize = (2 * props + 31) / 32;
-        int oldArraySize = data->bindingBitsSize / 32;
+        Q_ASSERT(arraySize > 1);
 
-        data->bindingBits = (quint32 *)realloc(data->bindingBits,
+        // special handling for 32 here is to make sure we wipe the first byte
+        // when going from bindingBitsValue to bindingBits, and preserve the old
+        // set bits so we can restore them after the allocation
+        int oldArraySize = data->bindingBitsSize > 32 ? data->bindingBitsSize / 32 : 0;
+        quint32 oldValue = data->bindingBitsSize == 32 ? data->bindingBitsValue : 0;
+
+        data->bindingBits = (quint32 *)realloc((data->bindingBitsSize == 32) ? 0 : data->bindingBits,
                                                arraySize * sizeof(quint32));
 
         memset(data->bindingBits + oldArraySize,
@@ -1705,15 +1807,27 @@ static void QQmlData_setBit(QQmlData *data, QObject *obj, int bit)
                sizeof(quint32) * (arraySize - oldArraySize));
 
         data->bindingBitsSize = arraySize * 32;
+
+        // reinstate bindingBitsValue after we dropped it
+        if (oldValue) {
+            memcpy(data->bindingBits, &oldValue, sizeof(oldValue));
+        }
     }
 
-    data->bindingBits[bit / 32] |= (1 << (bit % 32));
+    if (data->bindingBitsSize == 32)
+        data->bindingBitsValue |= (1 << (bit % 32));
+    else
+        data->bindingBits[bit / 32] |= (1 << (bit % 32));
 }
 
 static void QQmlData_clearBit(QQmlData *data, int bit)
 {
-    if (data->bindingBitsSize > bit)
-        data->bindingBits[bit / 32] &= ~(1 << (bit % 32));
+    if (data->bindingBitsSize > bit) {
+        if (data->bindingBitsSize == 32)
+            data->bindingBitsValue &= ~(1 << (bit % 32));
+        else
+            data->bindingBits[bit / 32] &= ~(1 << (bit % 32));
+    }
 }
 
 void QQmlData::clearBindingBit(int coreIndex)
@@ -1736,13 +1850,13 @@ void QQmlData::setPendingBindingBit(QObject *obj, int coreIndex)
     QQmlData_setBit(this, obj, coreIndex * 2 + 1);
 }
 
-void QQmlData::ensurePropertyCache(QQmlEngine *engine, QObject *object)
+void QQmlData::ensurePropertyCache(QJSEngine *engine, QObject *object)
 {
     Q_ASSERT(engine);
     QQmlData *ddata = QQmlData::get(object, /*create*/true);
     if (ddata->propertyCache)
         return;
-    ddata->propertyCache = QQmlEnginePrivate::get(engine)->cache(object);
+    ddata->propertyCache = QJSEnginePrivate::get(engine)->cache(object);
     if (ddata->propertyCache) ddata->propertyCache->addref();
 }
 
@@ -1772,7 +1886,7 @@ void QQmlEnginePrivate::warning(const QQmlError &error)
 {
     Q_Q(QQmlEngine);
     q->warnings(QList<QQmlError>() << error);
-    if (outputWarningsToStdErr)
+    if (outputWarningsToMsgLog)
         dumpwarning(error);
 }
 
@@ -1780,7 +1894,7 @@ void QQmlEnginePrivate::warning(const QList<QQmlError> &errors)
 {
     Q_Q(QQmlEngine);
     q->warnings(errors);
-    if (outputWarningsToStdErr)
+    if (outputWarningsToMsgLog)
         dumpwarning(errors);
 }
 
@@ -1984,7 +2098,7 @@ void QQmlEngine::setPluginPathList(const QStringList &paths)
 bool QQmlEngine::importPlugin(const QString &filePath, const QString &uri, QList<QQmlError> *errors)
 {
     Q_D(QQmlEngine);
-    return d->importDatabase.importDynamicPlugin(filePath, uri, QString(), errors);
+    return d->importDatabase.importDynamicPlugin(filePath, uri, QString(), -1, errors);
 }
 
 /*!
@@ -2026,22 +2140,6 @@ QString QQmlEngine::offlineStoragePath() const
     return d->offlineStoragePath;
 }
 
-QQmlPropertyCache *QQmlEnginePrivate::createCache(const QMetaObject *mo)
-{
-    Q_Q(QQmlEngine);
-
-    if (!mo->superClass()) {
-        QQmlPropertyCache *rv = new QQmlPropertyCache(q, mo);
-        propertyCache.insert(mo, rv);
-        return rv;
-    } else {
-        QQmlPropertyCache *super = cache(mo->superClass());
-        QQmlPropertyCache *rv = super->copyAndAppend(q, mo);
-        propertyCache.insert(mo, rv);
-        return rv;
-    }
-}
-
 QQmlPropertyCache *QQmlEnginePrivate::createCache(QQmlType *type, int minorVersion,
                                                                   QQmlError &error)
 {
@@ -2157,7 +2255,7 @@ QObject *QQmlEnginePrivate::toQObject(const QVariant &v, bool *ok) const
     int t = v.userType();
     if (t == QMetaType::QObjectStar || m_compositeTypes.contains(t)) {
         if (ok) *ok = true;
-        return *(QObject **)(v.constData());
+        return *(QObject *const *)(v.constData());
     } else {
         return QQmlMetaType::toQObject(v, ok);
     }
@@ -2248,16 +2346,12 @@ void QQmlEnginePrivate::registerInternalCompositeType(QQmlCompiledData *data)
     QByteArray lst = "QQmlListProperty<" + name + '>';
 
     int ptr_type = QMetaType::registerNormalizedType(ptr,
-                                                     QtMetaTypePrivate::QMetaTypeFunctionHelper<QObject*>::Delete,
-                                                     QtMetaTypePrivate::QMetaTypeFunctionHelper<QObject*>::Create,
                                                      QtMetaTypePrivate::QMetaTypeFunctionHelper<QObject*>::Destruct,
                                                      QtMetaTypePrivate::QMetaTypeFunctionHelper<QObject*>::Construct,
                                                      sizeof(QObject*),
                                                      static_cast<QFlags<QMetaType::TypeFlag> >(QtPrivate::QMetaTypeTypeFlags<QObject*>::Flags),
                                                      0);
     int lst_type = QMetaType::registerNormalizedType(lst,
-                                                     QtMetaTypePrivate::QMetaTypeFunctionHelper<QQmlListProperty<QObject> >::Delete,
-                                                     QtMetaTypePrivate::QMetaTypeFunctionHelper<QQmlListProperty<QObject> >::Create,
                                                      QtMetaTypePrivate::QMetaTypeFunctionHelper<QQmlListProperty<QObject> >::Destruct,
                                                      QtMetaTypePrivate::QMetaTypeFunctionHelper<QQmlListProperty<QObject> >::Construct,
                                                      sizeof(QQmlListProperty<QObject>),
@@ -2283,6 +2377,9 @@ void QQmlEnginePrivate::unregisterInternalCompositeType(QQmlCompiledData *data)
     Locker locker(this);
     m_qmlLists.remove(lst_type);
     m_compositeTypes.remove(ptr_type);
+
+    QMetaType::unregisterType(ptr_type);
+    QMetaType::unregisterType(lst_type);
 }
 
 bool QQmlEnginePrivate::isTypeLoaded(const QUrl &url) const
@@ -2315,8 +2412,12 @@ static inline QString shellNormalizeFileName(const QString &name)
         return name;
 #endif
     TCHAR buffer[MAX_PATH];
-    if (!SHGetPathFromIDList(file, buffer))
+    bool gotPath = SHGetPathFromIDList(file, buffer);
+    ILFree(file);
+
+    if (!gotPath)
         return name;
+
     QString canonicalName = QString::fromWCharArray(buffer);
     // Upper case drive letter
     if (canonicalName.size() > 2 && canonicalName.at(1) == QLatin1Char(':'))
