@@ -1,40 +1,32 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the QtQml module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -69,6 +61,7 @@
 #include "qqmldirparser_p.h"
 #include <private/qintrusivelist_p.h>
 #include <private/qrecyclepool_p.h>
+#include <private/qfieldlist_p.h>
 
 #include <QtCore/qlist.h>
 #include <QtCore/qpair.h>
@@ -98,10 +91,10 @@ class QQmlComponentAttached;
 class QQmlCleanup;
 class QQmlDelayedError;
 class QQuickWorkerScriptEngine;
-class QQmlVME;
-class QmlObjectCreator;
+class QQmlObjectCreator;
 class QDir;
 class QQmlIncubator;
+class QQmlProfiler;
 
 // This needs to be declared here so that the pool for it can live in QQmlEnginePrivate.
 // The inline method definitions are in qqmljavascriptexpression_p.h
@@ -145,9 +138,10 @@ public:
 
     QQmlContext *rootContext;
     bool isDebugging;
-    bool useNewCompiler;
+    QQmlProfiler *profiler;
+    void enableProfiler();
 
-    bool outputWarningsToStdErr;
+    bool outputWarningsToMsgLog;
 
     // Registered cleanup handlers
     QQmlCleanup *cleanup;
@@ -167,11 +161,7 @@ public:
     typedef QPair<QPointer<QObject>,int> FinalizeCallback;
     void registerFinalizeCallback(QObject *obj, int index);
 
-    // --- old compiler:
-    QQmlVME *activeVME;
-    // --- new compiler:
-    QmlObjectCreator *activeObjectCreator;
-    // ---
+    QQmlObjectCreator *activeObjectCreator;
 
     QNetworkAccessManager *createNetworkAccessManager(QObject *parent) const;
     QNetworkAccessManager *getNetworkAccessManager() const;
@@ -186,8 +176,9 @@ public:
     void referenceScarceResources();
     void dereferenceScarceResources();
 
-    QQmlImportDatabase importDatabase;
     QQmlTypeLoader typeLoader;
+    QQmlImportDatabase importDatabase;
+
 
     QString offlineStoragePath;
 
@@ -217,9 +208,8 @@ public:
     inline static void deleteInEngineThread(QQmlEngine *, T *);
 
     // These methods may be called from the loader thread
-    inline QQmlPropertyCache *cache(QObject *obj);
-    inline QQmlPropertyCache *cache(const QMetaObject *);
     inline QQmlPropertyCache *cache(QQmlType *, int, QQmlError &error);
+    using QJSEnginePrivate::cache;
 
     // These methods may be called from the loader thread
     bool isQObject(int);
@@ -268,40 +258,14 @@ public:
 
     static bool qml_debugging_enabled;
 
-    mutable QMutex mutex;
+    mutable QMutex networkAccessManagerMutex;
 
 private:
-    // Locker locks the QQmlEnginePrivate data structures for read and write, if necessary.  
-    // Currently, locking is only necessary if the threaded loader is running concurrently.  If it is 
-    // either idle, or is running with the main thread blocked, no locking is necessary.  This way
-    // we only pay for locking when we have to.
-    // Consequently, this class should only be used to protect simple accesses or modifications of the 
-    // QQmlEnginePrivate structures or operations that can be guaranteed not to start activity
-    // on the loader thread.
-    // The Locker API is identical to QMutexLocker.  Locker reuses the QQmlEnginePrivate::mutex 
-    // QMutex instance and multiple Lockers are recursive in the same thread.
-    class Locker 
-    {
-    public:
-        inline Locker(const QQmlEngine *);
-        inline Locker(const QQmlEnginePrivate *);
-        inline ~Locker();
-
-        inline void unlock();
-        inline void relock();
-
-    private:
-        const QQmlEnginePrivate *m_ep;
-        quint32 m_locked:1;
-    };
-
     // Must be called locked
-    QQmlPropertyCache *createCache(const QMetaObject *);
     QQmlPropertyCache *createCache(QQmlType *, int, QQmlError &error);
 
     // These members must be protected by a QQmlEnginePrivate::Locker as they are required by
     // the threaded loader.  Only access them through their respective accessor methods.
-    QHash<const QMetaObject *, QQmlPropertyCache *> propertyCache;
     QHash<QPair<QQmlType *, int>, QQmlPropertyCache *> typePropertyCache;
     QHash<int, int> m_qmlLists;
     QHash<int, QQmlCompiledData *> m_compositeTypes;
@@ -314,40 +278,6 @@ private:
     void doDeleteInEngineThread();
 };
 
-QQmlEnginePrivate::Locker::Locker(const QQmlEngine *e)
-: m_ep(QQmlEnginePrivate::get(e))
-{
-    relock();
-}
-
-QQmlEnginePrivate::Locker::Locker(const QQmlEnginePrivate *e)
-: m_ep(e), m_locked(false)
-{
-    relock();
-}
-
-QQmlEnginePrivate::Locker::~Locker()
-{
-    unlock();
-}
-
-void QQmlEnginePrivate::Locker::unlock()
-{
-    if (m_locked) { 
-        m_ep->mutex.unlock();
-        m_locked = false;
-    }
-}
-
-void QQmlEnginePrivate::Locker::relock()
-{
-    Q_ASSERT(!m_locked);
-    if (m_ep->typeLoader.isConcurrent()) {
-        m_ep->mutex.lock();
-        m_locked = true;
-    }
-}
-
 /*!
 Returns true if the calling thread is the QQmlEngine thread.
 */
@@ -373,7 +303,7 @@ thread, \a value will be deleted immediately.
 This method should be used for *any* type that has resources that need to
 be freed in the engine thread.  This is generally types that use V8 handles.
 As there is some small overhead in checking the current thread, it is best
-practice to check if any V8 handles actually need to be freed and delete 
+practice to check if any V8 handles actually need to be freed and delete
 the instance directly if not.
 */
 template<typename T>
@@ -384,7 +314,7 @@ void QQmlEnginePrivate::deleteInEngineThread(T *value)
     Q_ASSERT(value);
     if (isEngineThread()) {
         delete value;
-    } else { 
+    } else {
         struct I : public Deletable {
             I(T *value) : value(value) {}
             ~I() { delete value; }
@@ -411,52 +341,6 @@ void QQmlEnginePrivate::deleteInEngineThread(QQmlEngine *engine, T *value)
     QQmlEnginePrivate::get(engine)->deleteInEngineThread<T>(value);
 }
 
-/*!
-Returns a QQmlPropertyCache for \a obj if one is available.
-
-If \a obj is null, being deleted or contains a dynamic meta object 0
-is returned.
-
-The returned cache is not referenced, so if it is to be stored, call addref().
-
-XXX thread There is a potential future race condition in this and all the cache()
-functions.  As the QQmlPropertyCache is returned unreferenced, when called 
-from the loader thread, it is possible that the cache will have been dereferenced 
-and deleted before the loader thread has a chance to use or reference it.  This
-can't currently happen as the cache holds a reference to the 
-QQmlPropertyCache until the QQmlEngine is destroyed.
-*/
-QQmlPropertyCache *QQmlEnginePrivate::cache(QObject *obj)
-{
-    if (!obj || QObjectPrivate::get(obj)->metaObject || QObjectPrivate::get(obj)->wasDeleted)
-        return 0;
-
-    Locker locker(this);
-    const QMetaObject *mo = obj->metaObject();
-    QQmlPropertyCache *rv = propertyCache.value(mo);
-    if (!rv) rv = createCache(mo);
-    return rv;
-}
-
-/*!
-Returns a QQmlPropertyCache for \a metaObject.
-
-As the cache is persisted for the life of the engine, \a metaObject must be
-a static "compile time" meta-object, or a meta-object that is otherwise known to
-exist for the lifetime of the QQmlEngine.
-
-The returned cache is not referenced, so if it is to be stored, call addref().
-*/
-QQmlPropertyCache *QQmlEnginePrivate::cache(const QMetaObject *metaObject)
-{
-    Q_ASSERT(metaObject);
-
-    Locker locker(this);
-    QQmlPropertyCache *rv = propertyCache.value(metaObject);
-    if (!rv) rv = createCache(metaObject);
-    return rv;
-}
-
 /*!
 Returns a QQmlPropertyCache for \a type with \a minorVersion.
 
@@ -475,8 +359,8 @@ QQmlPropertyCache *QQmlEnginePrivate::cache(QQmlType *type, int minorVersion, QQ
     return rv;
 }
 
-QV8Engine *QQmlEnginePrivate::getV8Engine(QQmlEngine *e) 
-{ 
+QV8Engine *QQmlEnginePrivate::getV8Engine(QQmlEngine *e)
+{
     Q_ASSERT(e);
 
     return e->d_func()->v8engine();
@@ -489,32 +373,32 @@ QV4::ExecutionEngine *QQmlEnginePrivate::getV4Engine(QQmlEngine *e)
     return e->d_func()->v4engine();
 }
 
-QQmlEnginePrivate *QQmlEnginePrivate::get(QQmlEngine *e) 
-{ 
+QQmlEnginePrivate *QQmlEnginePrivate::get(QQmlEngine *e)
+{
     Q_ASSERT(e);
 
     return e->d_func();
 }
 
-const QQmlEnginePrivate *QQmlEnginePrivate::get(const QQmlEngine *e) 
-{ 
+const QQmlEnginePrivate *QQmlEnginePrivate::get(const QQmlEngine *e)
+{
     Q_ASSERT(e);
 
     return e->d_func();
 }
 
-QQmlEnginePrivate *QQmlEnginePrivate::get(QQmlContext *c) 
-{ 
-    return (c && c->engine()) ? QQmlEnginePrivate::get(c->engine()) : 0; 
+QQmlEnginePrivate *QQmlEnginePrivate::get(QQmlContext *c)
+{
+    return (c && c->engine()) ? QQmlEnginePrivate::get(c->engine()) : 0;
 }
 
-QQmlEnginePrivate *QQmlEnginePrivate::get(QQmlContextData *c) 
-{ 
-    return (c && c->engine) ? QQmlEnginePrivate::get(c->engine) : 0; 
+QQmlEnginePrivate *QQmlEnginePrivate::get(QQmlContextData *c)
+{
+    return (c && c->engine) ? QQmlEnginePrivate::get(c->engine) : 0;
 }
 
-QQmlEngine *QQmlEnginePrivate::get(QQmlEnginePrivate *p) 
-{ 
+QQmlEngine *QQmlEnginePrivate::get(QQmlEnginePrivate *p)
+{
     Q_ASSERT(p);
 
     return p->q_func();
