@@ -1,40 +1,32 @@
 /****************************************************************************
 **
 ** Copyright (C) 2013 BlackBerry Limited. All rights reserved.
-** Contact: http://www.qt-project.org/legal
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the QtQml module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -96,9 +88,9 @@ Q_GLOBAL_STATIC(interceptorSelectorMap, interceptorInstances);
  */
 
 /*!
-  Creates a new QQmlFileSelector, which includes its own QFileSelector.
-  \a engine is the QQmlEngine you wish to apply file selectors too. It will
-  also take ownership of the QQmlFileSelector.
+  Creates a new QQmlFileSelector with parent object \a parent, which includes
+  its own QFileSelector. \a engine is the QQmlEngine you wish to apply file
+  selectors to. It will also take ownership of the QQmlFileSelector.
 */
 
 QQmlFileSelector::QQmlFileSelector(QQmlEngine* engine, QObject* parent)
@@ -106,10 +98,13 @@ QQmlFileSelector::QQmlFileSelector(QQmlEngine* engine, QObject* parent)
 {
     Q_D(QQmlFileSelector);
     d->engine = engine;
-    interceptorInstances()->insert(d->myInstance, this);
-    d->engine->setUrlInterceptor(d->myInstance);
+    interceptorInstances()->insert(d->myInstance.data(), this);
+    d->engine->setUrlInterceptor(d->myInstance.data());
 }
 
+/*!
+   Destroys the QQmlFileSelector object.
+*/
 QQmlFileSelector::~QQmlFileSelector()
 {
     Q_D(QQmlFileSelector);
@@ -117,7 +112,7 @@ QQmlFileSelector::~QQmlFileSelector()
         d->engine->setUrlInterceptor(0);
         d->engine = 0;
     }
-    interceptorInstances()->remove(d->myInstance);
+    interceptorInstances()->remove(d->myInstance.data());
 }
 
 QQmlFileSelectorPrivate::QQmlFileSelectorPrivate()
@@ -125,13 +120,13 @@ QQmlFileSelectorPrivate::QQmlFileSelectorPrivate()
     Q_Q(QQmlFileSelector);
     ownSelector = true;
     selector = new QFileSelector(q);
-    myInstance = new QQmlFileSelectorInterceptor(this);
+    myInstance.reset(new QQmlFileSelectorInterceptor(this));
 }
 
 /*!
-  Sets a different QFileSelector instance for use by the QQmlFileSelector. QQmlFileSelector does not
-  take ownership of the new QFileSelector. To reset QQmlFileSelector to use its internal
-  QFileSelector instance, call setSelector(0).
+  Sets the QFileSelector instance for use by the QQmlFileSelector to \a selector.
+  QQmlFileSelector does not take ownership of the new QFileSelector. To reset QQmlFileSelector
+  to use its internal QFileSelector instance, call setSelector(0).
 */
 
 void QQmlFileSelector::setSelector(QFileSelector *selector)
@@ -152,8 +147,8 @@ void QQmlFileSelector::setSelector(QFileSelector *selector)
 }
 
 /*!
-  Adds extra selectors to the current QFileSelector being used. Use this when
-  extra selectors are all you need to avoid having to create your own
+  Adds extra selectors contained in \a strings to the current QFileSelector being used.
+  Use this when extra selectors are all you need to avoid having to create your own
   QFileSelector instance.
 */
 void QQmlFileSelector::setExtraSelectors(QStringList &strings)
@@ -162,8 +157,20 @@ void QQmlFileSelector::setExtraSelectors(QStringList &strings)
     d->selector->setExtraSelectors(strings);
 }
 
+
 /*!
-  Gets the QQmlFileSelector currently active on the target engine.
+  Adds extra selectors contained in \a strings to the current QFileSelector being used.
+  Use this when extra selectors are all you need to avoid having to create your own
+  QFileSelector instance.
+*/
+void QQmlFileSelector::setExtraSelectors(const QStringList &strings)
+{
+    Q_D(QQmlFileSelector);
+    d->selector->setExtraSelectors(strings);
+}
+
+/*!
+  Gets the QQmlFileSelector currently active on the target \a engine.
 */
 QQmlFileSelector* QQmlFileSelector::get(QQmlEngine* engine)
 {
