@@ -1,40 +1,32 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the QtQml module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -62,7 +54,7 @@
 #include <private/qv8engine_p.h>
 #include <private/qv4serialize_p.h>
 
-#include <private/qv4value_p.h>
+#include <private/qv4value_inl_p.h>
 #include <private/qv4functionobject_p.h>
 #include <private/qv4script_p.h>
 #include <private/qv4scopedvalue_p.h>
@@ -195,15 +187,15 @@ private:
     void reportScriptException(WorkerScript *, const QQmlError &error);
 };
 
-QQuickWorkerScriptEnginePrivate::WorkerEngine::WorkerEngine(QQuickWorkerScriptEnginePrivate *parent) 
+QQuickWorkerScriptEnginePrivate::WorkerEngine::WorkerEngine(QQuickWorkerScriptEnginePrivate *parent)
 : QV8Engine(0), p(parent), accessManager(0)
 {
     m_v4Engine->v8Engine = this;
 }
 
-QQuickWorkerScriptEnginePrivate::WorkerEngine::~WorkerEngine() 
-{ 
-    delete accessManager; 
+QQuickWorkerScriptEnginePrivate::WorkerEngine::~WorkerEngine()
+{
+    delete accessManager;
 }
 
 void QQuickWorkerScriptEnginePrivate::WorkerEngine::init()
@@ -229,18 +221,19 @@ void QQuickWorkerScriptEnginePrivate::WorkerEngine::init()
     "})"
 
     QV4::Scope scope(m_v4Engine);
-    onmessage = QV4::Script(m_v4Engine->rootContext, QString::fromUtf8(CALL_ONMESSAGE_SCRIPT)).run(); // do not use QStringLiteral here, MSVC2012 cannot apply this cleanly to the macro
+    QV4::ScopedContext globalContext(scope, scope.engine->rootContext());
+    onmessage.set(scope.engine, QV4::Script(globalContext, QString::fromUtf8(CALL_ONMESSAGE_SCRIPT)).run()); // do not use QStringLiteral here, MSVC2012 cannot apply this cleanly to the macro
     Q_ASSERT(!scope.engine->hasException);
-    QV4::Script createsendscript(m_v4Engine->rootContext, QString::fromUtf8(SEND_MESSAGE_CREATE_SCRIPT)); // do not use QStringLiteral here, MSVC2012 cannot apply this cleanly to the macro
-    QV4::Scoped<QV4::FunctionObject> createsendconstructor(scope, createsendscript.run());
+    QV4::Script createsendscript(globalContext, QString::fromUtf8(SEND_MESSAGE_CREATE_SCRIPT)); // do not use QStringLiteral here, MSVC2012 cannot apply this cleanly to the macro
+    QV4::ScopedFunctionObject createsendconstructor(scope, createsendscript.run());
     Q_ASSERT(!scope.engine->hasException);
     QV4::ScopedString name(scope, m_v4Engine->newString(QStringLiteral("sendMessage")));
-    QV4::ScopedValue function(scope, m_v4Engine->newBuiltinFunction(m_v4Engine->rootContext, name,
+    QV4::ScopedValue function(scope, QV4::BuiltinFunction::create(globalContext, name,
                                                                     QQuickWorkerScriptEnginePrivate::method_sendMessage));
     QV4::ScopedCallData callData(scope, 1);
     callData->args[0] = function;
     callData->thisObject = global();
-    createsend = createsendconstructor->call(callData);
+    createsend.set(scope.engine, createsendconstructor->call(callData));
 }
 
 // Requires handle and context scope
@@ -252,7 +245,6 @@ QV4::ReturnedValue QQuickWorkerScriptEnginePrivate::WorkerEngine::sendFunction(i
 
     QV4::Scope scope(v4);
     QV4::ScopedFunctionObject f(scope, createsend.value());
-    QV4::ExecutionContext *ctx = v4->currentContext();
 
     QV4::ScopedValue v(scope);
     QV4::ScopedCallData callData(scope, 1);
@@ -260,11 +252,11 @@ QV4::ReturnedValue QQuickWorkerScriptEnginePrivate::WorkerEngine::sendFunction(i
     callData->thisObject = global();
     v = f->call(callData);
     if (scope.hasException())
-        v = ctx->catchException();
-    return v.asReturnedValue();
+        v = scope.engine->catchException();
+    return v->asReturnedValue();
 }
 
-QNetworkAccessManager *QQuickWorkerScriptEnginePrivate::WorkerEngine::networkAccessManager() 
+QNetworkAccessManager *QQuickWorkerScriptEnginePrivate::WorkerEngine::networkAccessManager()
 {
     if (!accessManager) {
         if (p->qmlengine && p->qmlengine->networkAccessManagerFactory()) {
@@ -283,13 +275,13 @@ QQuickWorkerScriptEnginePrivate::QQuickWorkerScriptEnginePrivate(QQmlEngine *eng
 
 QV4::ReturnedValue QQuickWorkerScriptEnginePrivate::method_sendMessage(QV4::CallContext *ctx)
 {
-    WorkerEngine *engine = (WorkerEngine*)ctx->engine->v8Engine;
+    WorkerEngine *engine = (WorkerEngine*)ctx->engine()->v8Engine;
 
-    int id = ctx->callData->argc > 1 ? ctx->callData->args[1].toInt32() : 0;
+    int id = ctx->argc() > 1 ? ctx->args()[1].toInt32() : 0;
 
     QV4::Scope scope(ctx);
-    QV4::ScopedValue v(scope, ctx->callData->argument(2));
-    QByteArray data = QV4::Serialize::serialize(v, engine);
+    QV4::ScopedValue v(scope, ctx->argument(2));
+    QByteArray data = QV4::Serialize::serialize(v, scope.engine);
 
     QMutexLocker locker(&engine->p->m_lock);
     WorkerScript *script = engine->p->workers.value(id);
@@ -308,13 +300,13 @@ QV4::ReturnedValue QQuickWorkerScriptEnginePrivate::getWorker(WorkerScript *scri
         QV4::ExecutionEngine *v4 = QV8Engine::getV4(workerEngine);
         QV4::Scope scope(v4);
 
-        script->object = QV4::QmlContextWrapper::urlScope(workerEngine, script->source);
+        script->object.set(v4, QV4::QmlContextWrapper::urlScope(v4, script->source));
 
         QV4::Scoped<QV4::QmlContextWrapper> w(scope, script->object.value());
         Q_ASSERT(!!w);
         w->setReadOnly(false);
 
-        QV4::Scoped<QV4::Object> api(scope, v4->newObject());
+        QV4::ScopedObject api(scope, v4->newObject());
         api->put(QV4::ScopedString(scope, v4->newString(QStringLiteral("sendMessage"))), QV4::ScopedValue(scope, workerEngine->sendFunction(script->id)));
 
         w->QV4::Object::put(QV4::ScopedString(scope, v4->newString(QStringLiteral("WorkerScript"))), api);
@@ -340,7 +332,11 @@ bool QQuickWorkerScriptEnginePrivate::event(QEvent *event)
         return true;
     } else if (event->type() == (QEvent::Type)WorkerRemoveEvent::WorkerRemove) {
         WorkerRemoveEvent *workerEvent = static_cast<WorkerRemoveEvent *>(event);
-        workers.remove(workerEvent->workerId());
+        QHash<int, WorkerScript *>::iterator itr = workers.find(workerEvent->workerId());
+        if (itr != workers.end()) {
+            delete itr.value();
+            workers.erase(itr);
+        }
         return true;
     } else {
         return QObject::event(event);
@@ -356,9 +352,8 @@ void QQuickWorkerScriptEnginePrivate::processMessage(int id, const QByteArray &d
     QV4::ExecutionEngine *v4 = QV8Engine::getV4(workerEngine);
     QV4::Scope scope(v4);
     QV4::ScopedFunctionObject f(scope, workerEngine->onmessage.value());
-    QV4::ExecutionContext *ctx = v4->currentContext();
 
-    QV4::ScopedValue value(scope, QV4::Serialize::deserialize(data, workerEngine));
+    QV4::ScopedValue value(scope, QV4::Serialize::deserialize(data, v4));
 
     QV4::ScopedCallData callData(scope, 2);
     callData->thisObject = workerEngine->global();
@@ -366,7 +361,7 @@ void QQuickWorkerScriptEnginePrivate::processMessage(int id, const QByteArray &d
     callData->args[1] = value;
     f->call(callData);
     if (scope.hasException()) {
-        QQmlError error = QV4::ExecutionEngine::catchExceptionAsQmlError(ctx);
+        QQmlError error = scope.engine->catchExceptionAsQmlError();
         reportScriptException(script, error);
     }
 }
@@ -378,40 +373,47 @@ void QQuickWorkerScriptEnginePrivate::processLoad(int id, const QUrl &url)
 
     QString fileName = QQmlFile::urlToLocalFileOrQrc(url);
 
-    QFile f(fileName);
-    if (f.open(QIODevice::ReadOnly)) {
+    QV4::ExecutionEngine *v4 = QV8Engine::getV4(workerEngine);
+    QV4::Scope scope(v4);
+    QScopedPointer<QV4::Script> program;
+
+    WorkerScript *script = workers.value(id);
+    if (!script)
+        return;
+    script->source = url;
+
+    QV4::ScopedObject activation(scope, getWorker(script));
+    if (!activation)
+        return;
+
+    if (const QQmlPrivate::CachedQmlUnit *cachedUnit = QQmlMetaType::findCachedCompilationUnit(url)) {
+        QV4::CompiledData::CompilationUnit *jsUnit = cachedUnit->createCompilationUnit();
+        program.reset(new QV4::Script(v4, activation, jsUnit));
+    } else {
+        QFile f(fileName);
+        if (!f.open(QIODevice::ReadOnly)) {
+            qWarning().nospace() << "WorkerScript: Cannot find source file " << url.toString();
+            return;
+        }
+
         QByteArray data = f.readAll();
         QString sourceCode = QString::fromUtf8(data);
-        QQmlScript::Parser::extractPragmas(sourceCode);
+        QmlIR::Document::removeScriptPragmas(sourceCode);
 
-        WorkerScript *script = workers.value(id);
-        if (!script)
-            return;
-        script->source = url;
+        program.reset(new QV4::Script(v4, activation, sourceCode, url.toString()));
+        program->parse();
+    }
 
-        QV4::ExecutionEngine *v4 = QV8Engine::getV4(workerEngine);
-        QV4::Scope scope(v4);
+    if (!v4->hasException)
+        program->run();
 
-        QV4::Scoped<QV4::Object> activation(scope, getWorker(script));
-        if (!activation)
-            return;
-
-        QV4::Script program(v4, activation, sourceCode, url.toString());
-
-        QV4::ExecutionContext *ctx = v4->currentContext();
-        program.parse();
-        if (!v4->hasException)
-            program.run();
-        if (v4->hasException) {
-            QQmlError error = QV4::ExecutionEngine::catchExceptionAsQmlError(ctx);
-            reportScriptException(script, error);
-        }
-    } else {
-        qWarning().nospace() << "WorkerScript: Cannot find source file " << url.toString();
+    if (v4->hasException) {
+        QQmlError error = v4->catchExceptionAsQmlError();
+        reportScriptException(script, error);
     }
 }
 
-void QQuickWorkerScriptEnginePrivate::reportScriptException(WorkerScript *script, 
+void QQuickWorkerScriptEnginePrivate::reportScriptException(WorkerScript *script,
                                                                   const QQmlError &error)
 {
     QQuickWorkerScriptEnginePrivate *p = QQuickWorkerScriptEnginePrivate::get(workerEngine);
@@ -580,7 +582,7 @@ void QQuickWorkerScriptEngine::run()
     that the main GUI thread is not blocked.
 
     Messages can be passed between the new thread and the parent thread
-    using \l sendMessage() and the \l {WorkerScript::onMessage}{onMessage()} handler.
+    using \l sendMessage() and the \c onMessage() handler.
 
     An example:
 
@@ -611,8 +613,8 @@ void QQuickWorkerScriptEngine::run()
 
     Worker script can not use \l {qtqml-javascript-imports.html}{.import} syntax.
 
-    \sa {declarative/threading/workerscript}{WorkerScript example},
-        {declarative/threading/threadedlistmodel}{Threaded ListModel example}
+    \sa {Qt Quick Examples - Threading},
+        {Threaded ListModel Example}
 */
 QQuickWorkerScript::QQuickWorkerScript(QObject *parent)
 : QObject(parent), m_engine(0), m_scriptId(-1), m_componentComplete(true)
@@ -680,7 +682,7 @@ void QQuickWorkerScript::sendMessage(QQmlV4Function *args)
     if (args->length() != 0)
         argument = (*args)[0];
 
-    m_engine->sendMessage(m_scriptId, QV4::Serialize::serialize(argument, args->engine()));
+    m_engine->sendMessage(m_scriptId, QV4::Serialize::serialize(argument, scope.engine));
 }
 
 void QQuickWorkerScript::classBegin()
@@ -716,10 +718,12 @@ void QQuickWorkerScript::componentComplete()
 }
 
 /*!
-    \qmlsignal WorkerScript::onMessage(jsobject msg)
+    \qmlsignal WorkerScript::message(jsobject msg)
 
-    This handler is called when a message \a msg is received from a worker
+    This signal is emitted when a message \a msg is received from a worker
     script in another thread through a call to sendMessage().
+
+    The corresponding handler is \c onMessage.
 */
 
 bool QQuickWorkerScript::event(QEvent *event)
@@ -730,7 +734,7 @@ bool QQuickWorkerScript::event(QEvent *event)
             WorkerDataEvent *workerEvent = static_cast<WorkerDataEvent *>(event);
             QV8Engine *v8engine = QQmlEnginePrivate::get(engine)->v8engine();
             QV4::Scope scope(QV8Engine::getV4(v8engine));
-            QV4::ScopedValue value(scope, QV4::Serialize::deserialize(workerEvent->data(), v8engine));
+            QV4::ScopedValue value(scope, QV4::Serialize::deserialize(workerEvent->data(), scope.engine));
             emit message(QQmlV4Handle(value));
         }
         return true;
