@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
@@ -10,9 +10,9 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia. For licensing terms and
-** conditions see http://qt.digia.com/licensing. For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
@@ -23,8 +23,8 @@
 ** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights. These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** $QT_END_LICENSE$
@@ -475,13 +475,12 @@ void QuickTestResult::stringify(QQmlV4Function *args)
     QV4::ScopedValue value(scope, (*args)[0]);
 
     QString result;
-    QV8Engine *engine = args->engine();
 
     //Check for Object Type
     if (value->isObject()
         && !value->asFunctionObject()
         && !value->asArrayObject()) {
-        QVariant v = engine->toVariant(value, QMetaType::UnknownType);
+        QVariant v = scope.engine->toVariant(value, QMetaType::UnknownType);
         if (v.isValid()) {
             switch (v.type()) {
             case QVariant::Vector3D:
@@ -497,7 +496,9 @@ void QuickTestResult::stringify(QQmlV4Function *args)
         } else {
             result = QLatin1String("Object");
         }
-    } else {
+    }
+
+    if (result.isEmpty()) {
         QString tmp = value->toQStringNoThrow();
         if (value->asArrayObject())
             result.append(QString::fromLatin1("[%1]").arg(tmp));
@@ -505,7 +506,7 @@ void QuickTestResult::stringify(QQmlV4Function *args)
             result.append(tmp);
     }
 
-    args->setReturnValue(args->engine()->toString(result));
+    args->setReturnValue(QV4::Encode(args->v4engine()->newString(result)));
 }
 
 bool QuickTestResult::compare
