@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the QtQuick module of the Qt Toolkit.
 **
@@ -10,9 +10,9 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia. For licensing terms and
-** conditions see http://qt.digia.com/licensing. For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
@@ -23,8 +23,8 @@
 ** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights. These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** $QT_END_LICENSE$
@@ -92,8 +92,8 @@ namespace {
         {
         }
 
-        void paint(QPainter *p) const { p->fillRect(m_rect, m_brush); }
-        QRectF boundingRect() const { return m_rect; }
+        void paint(QPainter *p) const Q_DECL_OVERRIDE { p->fillRect(m_rect, m_brush); }
+        QRectF boundingRect() const Q_DECL_OVERRIDE { return m_rect; }
 
     private:
         QRectF m_rect;
@@ -109,8 +109,8 @@ namespace {
         {
         }
 
-        void paint(QPainter *p) const { p->fillPath(m_path, m_brush); }
-        QRectF boundingRect() const { return m_path.boundingRect(); }
+        void paint(QPainter *p) const Q_DECL_OVERRIDE { p->fillPath(m_path, m_brush); }
+        QRectF boundingRect() const Q_DECL_OVERRIDE { return m_path.boundingRect(); }
 
     private:
         QPainterPath m_path;
@@ -126,9 +126,9 @@ namespace {
         {
         }
 
-        void paint(QPainter *p) const { p->strokePath(m_path, m_pen); }
+        void paint(QPainter *p) const Q_DECL_OVERRIDE { p->strokePath(m_path, m_pen); }
 
-        QRectF boundingRect() const
+        QRectF boundingRect() const Q_DECL_OVERRIDE
         {
             qreal d = qMax(qreal(1), m_pen.widthF());
             return m_path.boundingRect().adjusted(-d, -d, d, d);
@@ -148,9 +148,9 @@ namespace {
         {
         }
 
-        void paint(QPainter *p) const { p->drawImage(m_offset, m_image); }
+        void paint(QPainter *p) const Q_DECL_OVERRIDE { p->drawImage(m_offset, m_image); }
 
-        QRectF boundingRect() const { return QRectF(m_image.rect()).translated(m_offset); }
+        QRectF boundingRect() const Q_DECL_OVERRIDE { return QRectF(m_image.rect()).translated(m_offset); }
 
     private:
         QImage m_image;
