@@ -1,46 +1,41 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the QtQuick module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
 #include "qquickdrag_p.h"
 
+#include <private/qguiapplication_p.h>
+#include <qpa/qplatformintegration.h>
+#include <qpa/qplatformdrag.h>
 #include <private/qquickitem_p.h>
 #include <QtQuick/private/qquickevents_p_p.h>
 #include <private/qquickitemchangelistener_p.h>
@@ -79,8 +74,8 @@ public:
     {
     }
 
-    void itemGeometryChanged(QQuickItem *, const QRectF &, const QRectF &);
-    void itemParentChanged(QQuickItem *, QQuickItem *parent);
+    void itemGeometryChanged(QQuickItem *, const QRectF &, const QRectF &) Q_DECL_OVERRIDE;
+    void itemParentChanged(QQuickItem *, QQuickItem *parent) Q_DECL_OVERRIDE;
     void updatePosition();
     void restartDrag();
     void deliverEnterEvent();
@@ -120,23 +115,23 @@ public:
     \ingroup qtquick-input
     \brief For specifying drag and drop events for moved Items
 
-    Using the Drag attached property any Item can be made a source of drag and drop
+    Using the Drag attached property, any Item can be made a source of drag and drop
     events within a scene.
 
-    When a drag is \l active on an item any change in that item's position will
+    When a drag is \l active on an item, any change in that item's position will
     generate a drag event that will be sent to any DropArea that intersects
     with the new position of the item.  Other items which implement drag and
     drop event handlers can also receive these events.
 
     The following snippet shows how an item can be dragged with a MouseArea.
-    However, dragging is not limited to mouse drags, anything that can move an item
-    can generate drag events, this can include touch events, animations and bindings.
+    However, dragging is not limited to mouse drags; anything that can move an item
+    can generate drag events, including touch events, animations and bindings.
 
     \snippet qml/drag.qml 0
 
     A drag can be terminated either by canceling it with Drag.cancel() or setting
     Drag.active to false, or it can be terminated with a drop event by calling
-    Drag.drop().  If the drop event is accepted Drag.drop() will return the
+    Drag.drop().  If the drop event is accepted, Drag.drop() will return the
     \l {supportedActions}{drop action} chosen by the recipient of the event,
     otherwise it will return Qt.IgnoreAction.
 
@@ -322,7 +317,8 @@ void QQuickDragAttached::setActive(bool active)
     \qmlattachedproperty Object QtQuick::Drag::source
 
     This property holds an object that is identified to recipients of drag events as
-    the source of the events.  By default this is the item Drag property is attached to.
+    the source of the events.  By default this is the item that the Drag
+    property is attached to.
 
     Changing the source while a drag is active will reset the sequence of drag events by
     sending a drag leave event followed by a drag enter event with the new source.
@@ -523,8 +519,8 @@ void QQuickDragAttached::setProposedAction(Qt::DropAction action)
     \endlist
 
     When using \c Drag.Automatic you should also define \l mimeData and bind the
-    \l active property to the active property of \l MouseArea.drag.
- */
+    \l active property to the active property of MouseArea : \l {MouseArea::drag.active}
+*/
 
 QQuickDrag::DragType QQuickDragAttached::dragType() const
 {
@@ -731,7 +727,8 @@ Qt::DropAction QQuickDragAttachedPrivate::startDrag(Qt::DropActions supportedAct
 
     Qt::DropAction dropAction = drag->exec(supportedActions);
 
-    delete drag;
+    if (!QGuiApplicationPrivate::platformIntegration()->drag()->ownsDragObject())
+        drag->deleteLater();
 
     deliverLeaveEvent();
 
@@ -791,7 +788,7 @@ void QQuickDragAttached::startDrag(QQmlV4Function *args)
 QQuickDrag::QQuickDrag(QObject *parent)
 : QObject(parent), _target(0), _axis(XAndYAxis), _xmin(-FLT_MAX),
 _xmax(FLT_MAX), _ymin(-FLT_MAX), _ymax(FLT_MAX), _active(false), _filterChildren(false),
-  _threshold(qApp->styleHints()->startDragDistance())
+  _smoothed(true), _threshold(QGuiApplication::styleHints()->startDragDistance())
 {
 }
 
@@ -885,6 +882,18 @@ void QQuickDrag::setYmax(qreal m)
     emit maximumYChanged();
 }
 
+bool QQuickDrag::smoothed() const
+{
+    return _smoothed;
+}
+
+void QQuickDrag::setSmoothed(bool smooth)
+{
+    if (_smoothed != smooth) {
+        _smoothed = smooth;
+        emit smoothedChanged();
+    }
+}
 
 qreal QQuickDrag::threshold() const
 {
@@ -901,7 +910,7 @@ void QQuickDrag::setThreshold(qreal value)
 
 void QQuickDrag::resetThreshold()
 {
-    setThreshold(qApp->styleHints()->startDragDistance());
+    setThreshold(QGuiApplication::styleHints()->startDragDistance());
 }
 
 bool QQuickDrag::active() const
