@@ -1,40 +1,32 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the QtQuick module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -43,9 +35,9 @@
 #include <QtQuick/qquickwindow.h>
 #include <private/qsgadaptationlayer_p.h>
 #include <private/qquickitem_p.h>
+#include <private/qguiapplication_p.h>
 #include <QEvent>
 #include <QMouseEvent>
-#include <math.h>
 #include <QDebug>
 #include <qpa/qplatformnativeinterface.h>
 
@@ -235,7 +227,14 @@ void QQuickTouchPoint::setSceneY(qreal sceneY)
     A MultiPointTouchArea is an invisible item that is used to track multiple touch points.
 
     The \l Item::enabled property is used to enable and disable touch handling. When disabled,
-    the touch area becomes transparent to mouse/touch events.
+    the touch area becomes transparent to mouse and touch events.
+
+    By default, the mouse will be handled the same way as a single touch point,
+    and items under the touch area will not receive mouse events because the
+    touch area is handling them. But if the \l mouseEnabled property is set to
+    false, it becomes transparent to mouse events so that another
+    mouse-sensitive Item (such as a MouseArea) can be used to handle mouse
+    interaction separately.
 
     MultiPointTouchArea can be used in two ways:
 
@@ -251,60 +250,72 @@ void QQuickTouchPoint::setSceneY(qreal sceneY)
 */
 
 /*!
-    \qmlsignal QtQuick::MultiPointTouchArea::onPressed(list<TouchPoint> touchPoints)
+    \qmlsignal QtQuick::MultiPointTouchArea::pressed(list<TouchPoint> touchPoints)
 
-    This handler is called when new touch points are added. \a touchPoints is a list of these new points.
+    This signal is emitted when new touch points are added. \a touchPoints is a list of these new points.
 
-    If minimumTouchPoints is set to a value greater than one, this handler will not be called until the minimum number
-    of required touch points has been reached. At that point, onPressed will be called with all the current touch points.
+    If minimumTouchPoints is set to a value greater than one, this signal will not be emitted until the minimum number
+    of required touch points has been reached.
+
+    The corresponding handler is \c onPressed.
 */
 
 /*!
-    \qmlsignal QtQuick::MultiPointTouchArea::onUpdated(list<TouchPoint> touchPoints)
+    \qmlsignal QtQuick::MultiPointTouchArea::updated(list<TouchPoint> touchPoints)
 
-    This handler is called when existing touch points are updated. \a touchPoints is a list of these updated points.
+    This signal is emitted when existing touch points are updated. \a touchPoints is a list of these updated points.
+
+    The corresponding handler is \c onUpdated.
 */
 
 /*!
-    \qmlsignal QtQuick::MultiPointTouchArea::onReleased(list<TouchPoint> touchPoints)
+    \qmlsignal QtQuick::MultiPointTouchArea::released(list<TouchPoint> touchPoints)
 
-    This handler is called when existing touch points are removed. \a touchPoints is a list of these removed points.
+    This signal is emitted when existing touch points are removed. \a touchPoints is a list of these removed points.
+
+    The corresponding handler is \c onReleased.
 */
 
 /*!
-    \qmlsignal QtQuick::MultiPointTouchArea::onCanceled(list<TouchPoint> touchPoints)
+    \qmlsignal QtQuick::MultiPointTouchArea::canceled(list<TouchPoint> touchPoints)
 
-    This handler is called when new touch events have been canceled because another item stole the touch event handling.
+    This signal is emitted when new touch events have been canceled because another item stole the touch event handling.
 
     This signal is for advanced use: it is useful when there is more than one MultiPointTouchArea
     that is handling input, or when there is a MultiPointTouchArea inside a \l Flickable. In the latter
-    case, if you execute some logic on the onPressed signal and then start dragging, the
+    case, if you execute some logic in the \c onPressed signal handler and then start dragging, the
     \l Flickable may steal the touch handling from the MultiPointTouchArea. In these cases, to reset
     the logic when the MultiPointTouchArea has lost the touch handling to the \l Flickable,
-    \c onCanceled should be used in addition to onReleased.
+    \c canceled should be handled in addition to \l released.
 
     \a touchPoints is the list of canceled points.
+
+    The corresponding handler is \c onCanceled.
 */
 
 /*!
-    \qmlsignal QtQuick::MultiPointTouchArea::onGestureStarted(GestureEvent gesture)
+    \qmlsignal QtQuick::MultiPointTouchArea::gestureStarted(GestureEvent gesture)
 
-    This handler is called when the global drag threshold has been reached.
+    This signal is emitted when the global drag threshold has been reached.
 
-    This function is typically used when a MultiPointTouchAreas has been nested in a Flickable or another MultiPointTouchArea.
-    When the threshold has been reached, and the handler called, you can determine whether or not the touch
+    This signal is typically used when a MultiPointTouchArea has been nested in a Flickable or another MultiPointTouchArea.
+    When the threshold has been reached and the signal is handled, you can determine whether or not the touch
     area should grab the current touch points. By default they will not be grabbed; to grab them call \c gesture.grab(). If the
     gesture is not grabbed, the nesting Flickable, for example, would also have an opportunity to grab.
 
     The gesture object also includes information on the current set of \c touchPoints and the \c dragThreshold.
+
+    The corresponding handler is \c onGestureStarted.
 */
 
 /*!
-    \qmlsignal QtQuick::MultiPointTouchArea::onTouchUpdated(list<TouchPoint> touchPoints)
+    \qmlsignal QtQuick::MultiPointTouchArea::touchUpdated(list<TouchPoint> touchPoints)
 
-    This handler is called when the touch points handled by the MultiPointTouchArea change. This includes adding new touch points,
+    This signal is emitted when the touch points handled by the MultiPointTouchArea change. This includes adding new touch points,
     removing or canceling previous touch points, as well as updating current touch point data. \a touchPoints is the list of all current touch
     points.
+
+    The corresponding handler is \c onTouchUpdated.
 */
 
 /*!
@@ -312,6 +323,10 @@ void QQuickTouchPoint::setSceneY(qreal sceneY)
 
     This property holds a set of user-defined touch point objects that can be bound to.
 
+    If mouseEnabled is true (the default) and the left mouse button is pressed
+    while the mouse is over the touch area, the current mouse position will be
+    one of these touch points.
+
     In the following example, we have two small rectangles that follow our touch points.
 
     \snippet qml/multipointtoucharea/multipointtoucharea.qml 0
@@ -325,7 +340,8 @@ QQuickMultiPointTouchArea::QQuickMultiPointTouchArea(QQuickItem *parent)
     : QQuickItem(parent),
       _minimumTouchPoints(0),
       _maximumTouchPoints(INT_MAX),
-      _stealMouse(false)
+      _stealMouse(false),
+      _mouseEnabled(true)
 {
     setAcceptedMouseButtons(Qt::LeftButton);
     setFiltersChildMouseEvents(true);
@@ -357,6 +373,11 @@ QQuickMultiPointTouchArea::~QQuickMultiPointTouchArea()
     one handling two finger touches, and another handling three finger touches.
 
     By default, all touch points within the touch area are handled.
+
+    If mouseEnabled is true, the mouse acts as a touch point, so it is also
+    subject to these constraints: for example if maximumTouchPoints is two, you
+    can use the mouse as one touch point and a finger as another touch point
+    for a total of two.
 */
 
 int QQuickMultiPointTouchArea::minimumTouchPoints() const
@@ -385,6 +406,25 @@ void QQuickMultiPointTouchArea::setMaximumTouchPoints(int num)
     emit maximumTouchPointsChanged();
 }
 
+/*!
+    \qmlproperty bool QtQuick::MultiPointTouchArea::mouseEnabled
+
+    This property controls whether the MultiPointTouchArea will handle mouse
+    events too. If it is true (the default), the touch area will treat the
+    mouse the same as a single touch point; if it is false, the touch area will
+    ignore mouse events and allow them to "pass through" so that they can be
+    handled by other items underneath.
+*/
+void QQuickMultiPointTouchArea::setMouseEnabled(bool arg)
+{
+    if (_mouseEnabled != arg) {
+        _mouseEnabled = arg;
+        if (_mouseTouchPoint && !arg)
+            _mouseTouchPoint = Q_NULLPTR;
+        emit mouseEnabledChanged();
+    }
+}
+
 void QQuickMultiPointTouchArea::touchEvent(QTouchEvent *event)
 {
     switch (event->type()) {
@@ -433,10 +473,46 @@ void QQuickMultiPointTouchArea::updateTouchData(QEvent *event)
     bool ended = false;
     bool moved = false;
     bool started = false;
+    bool isMouseEvent = false;
 
     clearTouchLists();
-    QTouchEvent *e = static_cast<QTouchEvent*>(event);
-    QList<QTouchEvent::TouchPoint> touchPoints = e->touchPoints();
+    QList<QTouchEvent::TouchPoint> touchPoints;
+
+    switch (event->type()) {
+    case QEvent::TouchBegin:
+    case QEvent::TouchUpdate:
+    case QEvent::TouchEnd:
+        touchPoints = static_cast<QTouchEvent*>(event)->touchPoints();
+        break;
+    case QEvent::MouseButtonPress:
+    case QEvent::MouseMove:
+    case QEvent::MouseButtonRelease: {
+        QMouseEvent *me = static_cast<QMouseEvent*>(event);
+        _mouseQpaTouchPoint.setPos(me->localPos());
+        _mouseQpaTouchPoint.setScenePos(me->windowPos());
+        _mouseQpaTouchPoint.setScreenPos(me->screenPos());
+        if (event->type() == QEvent::MouseMove)
+            _mouseQpaTouchPoint.setState(Qt::TouchPointMoved);
+        else if (event->type() == QEvent::MouseButtonRelease)
+            _mouseQpaTouchPoint.setState(Qt::TouchPointReleased);
+        else { // QEvent::MouseButtonPress
+            _mouseQpaTouchPoint.setState(Qt::TouchPointPressed);
+            _pressedTouchPoints.append(_mouseTouchPoint);
+        }
+        touchPoints << _mouseQpaTouchPoint;
+        isMouseEvent = true;
+        break;
+    }
+    default:
+        qWarning("updateTouchData: unhandled event type %d", event->type());
+        break;
+    }
+
+    if (!isMouseEvent && _mouseTouchPoint) {
+        QQuickWindow *c = window();
+        if (c && c->mouseGrabberItem() == this)
+            touchPoints << _mouseQpaTouchPoint;
+    }
     int numTouchPoints = touchPoints.count();
     //always remove released touches, and make sure we handle all releases before adds.
     foreach (const QTouchEvent::TouchPoint &p, touchPoints) {
@@ -479,7 +555,7 @@ void QQuickMultiPointTouchArea::updateTouchData(QEvent *event)
         //see if we should be grabbing the gesture
         if (!_stealMouse /* !ignoring gesture*/) {
             bool offerGrab = false;
-            const int dragThreshold = qApp->styleHints()->startDragDistance();
+            const int dragThreshold = QGuiApplication::styleHints()->startDragDistance();
             foreach (const QTouchEvent::TouchPoint &p, touchPoints) {
                 if (p.state() == Qt::TouchPointReleased)
                     continue;
@@ -516,10 +592,12 @@ void QQuickMultiPointTouchArea::clearTouchLists()
 {
     foreach (QObject *obj, _releasedTouchPoints) {
         QQuickTouchPoint *dtp = static_cast<QQuickTouchPoint*>(obj);
-        if (!dtp->isQmlDefined())
+        if (!dtp->isQmlDefined()) {
+            _touchPoints.remove(dtp->pointId());
             delete dtp;
-        else
+        } else {
             dtp->setInUse(false);
+        }
     }
     _releasedTouchPoints.clear();
     _pressedTouchPoints.clear();
@@ -546,6 +624,25 @@ void QQuickMultiPointTouchArea::addTouchPoint(const QTouchEvent::TouchPoint *p)
     _pressedTouchPoints.append(dtp);
 }
 
+void QQuickMultiPointTouchArea::addTouchPoint(const QMouseEvent *e)
+{
+    QQuickTouchPoint *dtp = 0;
+    foreach (QQuickTouchPoint *tp, _touchPrototypes)
+        if (!tp->inUse()) {
+            tp->setInUse(true);
+            dtp = tp;
+            break;
+        }
+
+    if (dtp == 0)
+        dtp = new QQuickTouchPoint(false);
+    updateTouchPoint(dtp, e);
+    dtp->setPressed(true);
+    _touchPoints.insert(-1, dtp);
+    _pressedTouchPoints.append(dtp);
+    _mouseTouchPoint = dtp;
+}
+
 #ifdef Q_OS_OSX
 void QQuickMultiPointTouchArea::hoverEnterEvent(QHoverEvent *event)
 {
@@ -596,9 +693,23 @@ void QQuickMultiPointTouchArea::updateTouchPoint(QQuickTouchPoint *dtp, const QT
     dtp->setSceneY(p->scenePos().y());
 }
 
+void QQuickMultiPointTouchArea::updateTouchPoint(QQuickTouchPoint *dtp, const QMouseEvent *e)
+{
+    dtp->setPreviousX(dtp->x());
+    dtp->setPreviousY(dtp->y());
+    dtp->setX(e->localPos().x());
+    dtp->setY(e->localPos().y());
+    if (e->type() == QEvent::MouseButtonPress) {
+        dtp->setStartX(e->localPos().x());
+        dtp->setStartY(e->localPos().y());
+    }
+    dtp->setSceneX(e->windowPos().x());
+    dtp->setSceneY(e->windowPos().y());
+}
+
 void QQuickMultiPointTouchArea::mousePressEvent(QMouseEvent *event)
 {
-    if (!isEnabled()) {
+    if (!isEnabled() || !_mouseEnabled || event->button() != Qt::LeftButton) {
         QQuickItem::mousePressEvent(event);
         return;
     }
@@ -606,25 +717,53 @@ void QQuickMultiPointTouchArea::mousePressEvent(QMouseEvent *event)
     _stealMouse = false;
     setKeepMouseGrab(false);
     event->setAccepted(true);
+    _mousePos = event->localPos();
+
+    if (event->source() != Qt::MouseEventNotSynthesized)
+        return;
+
+    if (_touchPoints.count() >= _minimumTouchPoints - 1 && _touchPoints.count() < _maximumTouchPoints) {
+        addTouchPoint(event);
+        updateTouchData(event);
+        emit pressed(_pressedTouchPoints);
+    }
 }
 
 void QQuickMultiPointTouchArea::mouseMoveEvent(QMouseEvent *event)
 {
-    if (!isEnabled()) {
+    if (!isEnabled() || !_mouseEnabled) {
         QQuickItem::mouseMoveEvent(event);
         return;
     }
 
-    //do nothing
+    if (event->source() != Qt::MouseEventNotSynthesized)
+        return;
+
+    _movedTouchPoints.clear();
+    updateTouchData(event);
 }
 
 void QQuickMultiPointTouchArea::mouseReleaseEvent(QMouseEvent *event)
 {
     _stealMouse = false;
-    if (!isEnabled()) {
+    if (!isEnabled() || !_mouseEnabled) {
         QQuickItem::mouseReleaseEvent(event);
         return;
     }
+
+    if (event->source() != Qt::MouseEventNotSynthesized)
+        return;
+
+    if (_mouseTouchPoint) {
+        updateTouchData(event);
+        _mouseTouchPoint->setPressed(false);
+        _mouseTouchPoint->setInUse(false);
+        _releasedTouchPoints.append(_mouseTouchPoint);
+        emit released(_releasedTouchPoints);
+        _releasedTouchPoints.removeAll(_mouseTouchPoint);
+        _mouseTouchPoint = Q_NULLPTR;
+    }
+
     QQuickWindow *c = window();
     if (c && c->mouseGrabberItem() == this)
         ungrabMouse();
@@ -677,6 +816,10 @@ bool QQuickMultiPointTouchArea::sendMouseEvent(QMouseEvent *event)
         QMouseEvent mouseEvent(event->type(), localPos, event->windowPos(), event->screenPos(),
                                event->button(), event->buttons(), event->modifiers());
         mouseEvent.setAccepted(false);
+        QGuiApplicationPrivate::setMouseEventCapsAndVelocity(&mouseEvent,
+                                                             QGuiApplicationPrivate::mouseEventCaps(event),
+                                                             QGuiApplicationPrivate::mouseEventVelocity(event));
+        QGuiApplicationPrivate::setMouseEventSource(&mouseEvent, Qt::MouseEventSynthesizedByQt);
 
         switch (mouseEvent.type()) {
         case QEvent::MouseMove:
@@ -691,7 +834,7 @@ bool QQuickMultiPointTouchArea::sendMouseEvent(QMouseEvent *event)
         default:
             break;
         }
-        grabber = c->mouseGrabberItem();
+        grabber = c ? c->mouseGrabberItem() : 0;
         if (grabber && stealThisEvent && !grabber->keepMouseGrab() && grabber != this)
             grabMouse();
 
