@@ -1,40 +1,32 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the QtQuick module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -44,7 +36,6 @@
 
 #include <QtQml/qqml.h>
 #include <QtQml/qqmlinfo.h>
-#include <QtCore/qmath.h>
 #include <QtCore/qcoreapplication.h>
 
 #include <QtQuick/private/qquickstate_p.h>
@@ -305,6 +296,8 @@ void QQuickBasePositioner::prePositioning()
 
     for (int ii = 0; ii < children.count(); ++ii) {
         QQuickItem *child = children.at(ii);
+        if (QQuickItemPrivate::get(child)->isTransparentForPositioner())
+            continue;
         QQuickItemPrivate *childPrivate = QQuickItemPrivate::get(child);
         PositionedItem posItem(child);
         int wIdx = oldItems.find(posItem);
@@ -869,8 +862,31 @@ void QQuickColumn::reportConflictingAnchors()
   \sa Grid::spacing
 */
 
+class QQuickRowPrivate : public QQuickBasePositionerPrivate
+{
+    Q_DECLARE_PUBLIC(QQuickRow)
+
+public:
+    QQuickRowPrivate()
+        : QQuickBasePositionerPrivate()
+    {}
+
+    void effectiveLayoutDirectionChange()
+    {
+        Q_Q(QQuickRow);
+        // For RTL layout the positioning changes when the width changes.
+        if (getEffectiveLayoutDirection(q) == Qt::RightToLeft)
+            addItemChangeListener(this, QQuickItemPrivate::Geometry);
+        else
+            removeItemChangeListener(this, QQuickItemPrivate::Geometry);
+        // Don't postpone, as it might be the only trigger for visible changes.
+        q->prePositioning();
+        emit q->effectiveLayoutDirectionChanged();
+    }
+};
+
 QQuickRow::QQuickRow(QQuickItem *parent)
-: QQuickBasePositioner(Horizontal, parent)
+: QQuickBasePositioner(*new QQuickRowPrivate, Horizontal, parent)
 {
 }
 /*!
@@ -900,14 +916,8 @@ void QQuickRow::setLayoutDirection(Qt::LayoutDirection layoutDirection)
     QQuickBasePositionerPrivate *d = static_cast<QQuickBasePositionerPrivate* >(QQuickBasePositionerPrivate::get(this));
     if (d->layoutDirection != layoutDirection) {
         d->layoutDirection = layoutDirection;
-        // For RTL layout the positioning changes when the width changes.
-        if (d->layoutDirection == Qt::RightToLeft)
-            d->addItemChangeListener(d, QQuickItemPrivate::Geometry);
-        else
-            d->removeItemChangeListener(d, QQuickItemPrivate::Geometry);
-        prePositioning();
         emit layoutDirectionChanged();
-        emit effectiveLayoutDirectionChanged();
+        d->effectiveLayoutDirectionChange();
     }
 }
 /*!
@@ -1021,7 +1031,7 @@ void QQuickRow::reportConflictingAnchors()
 
     \image gridLayout_example.png
 
-    If an item within a Column is not \l {Item::}{visible}, or if it has a width or
+    If an item within a Grid is not \l {Item::}{visible}, or if it has a width or
     height of 0, the item will not be laid out and it will not be visible within the
     column. Also, since a Grid automatically positions its children, a child
     item within a Grid should not set its \l {Item::x}{x} or \l {Item::y}{y} positions
@@ -1111,8 +1121,33 @@ void QQuickRow::reportConflictingAnchors()
 
   \sa rows, columns
 */
+
+class QQuickGridPrivate : public QQuickBasePositionerPrivate
+{
+    Q_DECLARE_PUBLIC(QQuickGrid)
+
+public:
+    QQuickGridPrivate()
+        : QQuickBasePositionerPrivate()
+    {}
+
+    void effectiveLayoutDirectionChange()
+    {
+        Q_Q(QQuickGrid);
+        // For RTL layout the positioning changes when the width changes.
+        if (getEffectiveLayoutDirection(q) == Qt::RightToLeft)
+            addItemChangeListener(this, QQuickItemPrivate::Geometry);
+        else
+            removeItemChangeListener(this, QQuickItemPrivate::Geometry);
+        // Don't postpone, as it might be the only trigger for visible changes.
+        q->prePositioning();
+        emit q->effectiveLayoutDirectionChanged();
+        emit q->effectiveHorizontalAlignmentChanged(q->effectiveHAlign());
+    }
+};
+
 QQuickGrid::QQuickGrid(QQuickItem *parent)
-    : QQuickBasePositioner(Both, parent)
+    : QQuickBasePositioner(*new QQuickGridPrivate, Both, parent)
     , m_rows(-1)
     , m_columns(-1)
     , m_rowSpacing(-1)
@@ -1260,15 +1295,8 @@ void QQuickGrid::setLayoutDirection(Qt::LayoutDirection layoutDirection)
     QQuickBasePositionerPrivate *d = static_cast<QQuickBasePositionerPrivate*>(QQuickBasePositionerPrivate::get(this));
     if (d->layoutDirection != layoutDirection) {
         d->layoutDirection = layoutDirection;
-        // For RTL layout the positioning changes when the width changes.
-        if (d->layoutDirection == Qt::RightToLeft)
-            d->addItemChangeListener(d, QQuickItemPrivate::Geometry);
-        else
-            d->removeItemChangeListener(d, QQuickItemPrivate::Geometry);
-        prePositioning();
         emit layoutDirectionChanged();
-        emit effectiveLayoutDirectionChanged();
-        emit effectiveHorizontalAlignmentChanged(effectiveHAlign());
+        d->effectiveLayoutDirectionChange();
     }
 }
 
@@ -1652,6 +1680,14 @@ public:
         : QQuickBasePositionerPrivate(), flow(QQuickFlow::LeftToRight)
     {}
 
+    void effectiveLayoutDirectionChange()
+    {
+        Q_Q(QQuickFlow);
+        // Don't postpone, as it might be the only trigger for visible changes.
+        q->prePositioning();
+        emit q->effectiveLayoutDirectionChanged();
+    }
+
     QQuickFlow::Flow flow;
 };
 
@@ -1724,9 +1760,8 @@ void QQuickFlow::setLayoutDirection(Qt::LayoutDirection layoutDirection)
     Q_D(QQuickFlow);
     if (d->layoutDirection != layoutDirection) {
         d->layoutDirection = layoutDirection;
-        prePositioning();
         emit layoutDirectionChanged();
-        emit effectiveLayoutDirectionChanged();
+        d->effectiveLayoutDirectionChange();
     }
 }
 
