@@ -1,40 +1,32 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the QtQuick module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -118,10 +110,10 @@ public:
 
 protected:
     QQuickBasePositioner(QQuickBasePositionerPrivate &dd, PositionerType at, QQuickItem *parent);
-    virtual void componentComplete();
-    virtual void itemChange(ItemChange, const ItemChangeData &);
+    void componentComplete() Q_DECL_OVERRIDE;
+    void itemChange(ItemChange, const ItemChangeData &) Q_DECL_OVERRIDE;
 
-    virtual void updatePolish();
+    void updatePolish() Q_DECL_OVERRIDE;
 
 Q_SIGNALS:
     void spacingChanged();
@@ -181,12 +173,13 @@ public:
     QQuickColumn(QQuickItem *parent=0);
 
 protected:
-    virtual void doPositioning(QSizeF *contentSize);
-    virtual void reportConflictingAnchors();
+    void doPositioning(QSizeF *contentSize) Q_DECL_OVERRIDE;
+    void reportConflictingAnchors() Q_DECL_OVERRIDE;
 private:
     Q_DISABLE_COPY(QQuickColumn)
 };
 
+class QQuickRowPrivate;
 class Q_AUTOTEST_EXPORT QQuickRow: public QQuickBasePositioner
 {
     Q_OBJECT
@@ -204,12 +197,14 @@ Q_SIGNALS:
     void effectiveLayoutDirectionChanged();
 
 protected:
-    virtual void doPositioning(QSizeF *contentSize);
-    virtual void reportConflictingAnchors();
+    void doPositioning(QSizeF *contentSize) Q_DECL_OVERRIDE;
+    void reportConflictingAnchors() Q_DECL_OVERRIDE;
 private:
     Q_DISABLE_COPY(QQuickRow)
+    Q_DECLARE_PRIVATE(QQuickRow)
 };
 
+class QQuickGridPrivate;
 class Q_AUTOTEST_EXPORT QQuickGrid : public QQuickBasePositioner
 {
     Q_OBJECT
@@ -279,8 +274,8 @@ Q_SIGNALS:
     Q_REVISION(1) void verticalAlignmentChanged(VAlignment alignment);
 
 protected:
-    virtual void doPositioning(QSizeF *contentSize);
-    virtual void reportConflictingAnchors();
+    void doPositioning(QSizeF *contentSize) Q_DECL_OVERRIDE;
+    void reportConflictingAnchors() Q_DECL_OVERRIDE;
 
 private:
     int m_rows;
@@ -293,6 +288,7 @@ private:
     HAlignment m_hItemAlign;
     VAlignment m_vItemAlign;
     Q_DISABLE_COPY(QQuickGrid)
+    Q_DECLARE_PRIVATE(QQuickGrid)
 };
 
 class QQuickFlowPrivate;
@@ -320,8 +316,8 @@ Q_SIGNALS:
     void effectiveLayoutDirectionChanged();
 
 protected:
-    virtual void doPositioning(QSizeF *contentSize);
-    virtual void reportConflictingAnchors();
+    void doPositioning(QSizeF *contentSize) Q_DECL_OVERRIDE;
+    void reportConflictingAnchors() Q_DECL_OVERRIDE;
 protected:
     QQuickFlow(QQuickFlowPrivate &dd, QQuickItem *parent);
 private:
