@@ -1,40 +1,32 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the QtQuick module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -97,7 +89,8 @@ namespace {
     // Returns -1 if not found, returns index to first character after the name if found.
     int qt_search_for_variable(const char *s, int length, int index, VariableQualifier &decl,
                                int &typeIndex, int &typeLength,
-                               int &nameIndex, int &nameLength)
+                               int &nameIndex, int &nameLength,
+                               QQuickShaderEffectCommon::Key::ShaderType shaderType)
     {
         enum Identifier {
             QualifierIdentifier, // Base state
@@ -124,6 +117,7 @@ namespace {
                 int idLength = index - idIndex;
 
                 const int attrLen = sizeof("attribute") - 1;
+                const int inLen = sizeof("in") - 1;
                 const int uniLen = sizeof("uniform") - 1;
                 const int loLen = sizeof("lowp") - 1;
                 const int medLen = sizeof("mediump") - 1;
@@ -134,6 +128,10 @@ namespace {
                     if (idLength == attrLen && qstrncmp("attribute", s + idIndex, attrLen) == 0) {
                         decl = AttributeQualifier;
                         expected = PrecisionIdentifier;
+                    } else if (shaderType == QQuickShaderEffectCommon::Key::VertexShader
+                               && idLength == inLen && qstrncmp("in", s + idIndex, inLen) == 0) {
+                        decl = AttributeQualifier;
+                        expected = PrecisionIdentifier;
                     } else if (idLength == uniLen && qstrncmp("uniform", s + idIndex, uniLen) == 0) {
                         decl = UniformQualifier;
                         expected = PrecisionIdentifier;
@@ -287,7 +285,7 @@ void QQuickShaderEffectCommon::lookThroughShaderCode(QQuickItem *item, Key::Shad
     const char *s = code.constData();
     VariableQualifier decl = AttributeQualifier;
     while ((index = qt_search_for_variable(s, code.size(), index, decl, typeIndex, typeLength,
-                                           nameIndex, nameLength)) != -1)
+                                           nameIndex, nameLength, shaderType)) != -1)
     {
         if (decl == AttributeQualifier) {
             if (shaderType == Key::VertexShader)
@@ -559,7 +557,8 @@ void QQuickShaderEffectCommon::propertyChanged(QQuickItem *item, int mappedId,
        position (0, 0), the bottom-right (\l{Item::width}{width},
        \l{Item::height}{height}).
     \li attribute vec2 qt_MultiTexCoord0 - texture coordinate, the top-left
-       coordinate is (0, 0), the bottom-right (1, 1).
+       coordinate is (0, 0), the bottom-right (1, 1). If \l supportsAtlasTextures
+       is true, coordinates will be based on position in the atlas instead.
     \endlist
 
     In addition, any property that can be mapped to an OpenGL Shading Language
@@ -579,8 +578,12 @@ void QQuickShaderEffectCommon::propertyChanged(QQuickItem *item, int mappedId,
     \li QTransform -> mat3
     \li QMatrix4x4 -> mat4
     \li QQuaternion -> vec4, scalar value is \c w.
-    \li \l Image, \l ShaderEffectSource -> sampler2D - Origin is in the top-left
-       corner, and the color values are premultiplied.
+    \li \l Image -> sampler2D - Origin is in the top-left corner, and the
+        color values are premultiplied. The texture is provided as is,
+        excluding the Image item's fillMode. To include fillMode, use a
+        ShaderEffectSource or Image::layer::enabled.
+    \li \l ShaderEffectSource -> sampler2D - Origin is in the top-left
+        corner, and the color values are premultiplied.
     \endlist
 
     The QML scene graph back-end may choose to allocate textures in texture
@@ -588,7 +591,8 @@ void QQuickShaderEffectCommon::propertyChanged(QQuickItem *item, int mappedId,
     it is by default copied from the texture atlas into a stand-alone texture
     so that the texture coordinates span from 0 to 1, and you get the expected
     wrap modes. However, this will increase the memory usage. To avoid the
-    texture copy, you can for each "uniform sampler2D <name>" declare a
+    texture copy, set \l supportsAtlasTextures for simple shaders using
+    qt_MultiTexCoord0, or for each "uniform sampler2D <name>" declare a
     "uniform vec4 qt_SubRect_<name>" which will be assigned the texture's
     normalized source rectangle. For stand-alone textures, the source rectangle
     is [0, 1]x[0, 1]. For textures in an atlas, the source rectangle corresponds
@@ -601,7 +605,7 @@ void QQuickShaderEffectCommon::propertyChanged(QQuickItem *item, int mappedId,
     \l blending is enabled, source-over blending is used. However, additive
     blending can be achieved by outputting zero in the alpha channel.
 
-    \table
+    \table 70%
     \row
     \li \image declarative-shadereffectitem.png
     \li \qml
@@ -610,7 +614,8 @@ void QQuickShaderEffectCommon::propertyChanged(QQuickItem *item, int mappedId,
         Rectangle {
             width: 200; height: 100
             Row {
-                Image { id: img; sourceSize { width: 100; height: 100 } source: "qt-logo.png" }
+                Image { id: img;
+                        sourceSize { width: 100; height: 100 } source: "qt-logo.png" }
                 ShaderEffect {
                     width: 100; height: 100
                     property variant src: img
@@ -629,7 +634,9 @@ void QQuickShaderEffectCommon::propertyChanged(QQuickItem *item, int mappedId,
                         uniform lowp float qt_Opacity;
                         void main() {
                             lowp vec4 tex = texture2D(src, coord);
-                            gl_FragColor = vec4(vec3(dot(tex.rgb, vec3(0.344, 0.5, 0.156))), tex.a) * qt_Opacity;
+                            gl_FragColor = vec4(vec3(dot(tex.rgb,
+                                                vec3(0.344, 0.5, 0.156))),
+                                                     tex.a) * qt_Opacity;
                         }"
                 }
             }
@@ -641,8 +648,36 @@ void QQuickShaderEffectCommon::propertyChanged(QQuickItem *item, int mappedId,
     corner. For non-linear vertex transformations, like page curl, you can
     specify a fine grid of vertices by specifying a \l mesh resolution.
 
+    \section1 ShaderEffect and Item Layers
+
+    The ShaderEffect type can be combined with \l {Item Layers} {layered items}.
+
+    \table
+    \row
+      \li \b {Layer with effect disabled} \inlineimage qml-shadereffect-nolayereffect.png
+      \li \b {Layer with effect enabled} \inlineimage qml-shadereffect-layereffect.png
+    \row
+      \li \snippet qml/layerwitheffect.qml 1
+    \endtable
+
+    It is also possible to combine multiple layered items:
+
+    \table
+    \row
+      \li \inlineimage qml-shadereffect-opacitymask.png
+    \row
+      \li \snippet qml/opacitymask.qml 1
+    \endtable
+
+    The \l {Qt Graphical Effects} module contains several ready-made effects
+    for using with Qt Quick applications.
+
     \note Scene Graph textures have origin in the top-left corner rather than
     bottom-left which is common in OpenGL.
+
+    For information about the GLSL version being used, see \l QtQuick::OpenGLInfo.
+
+    \sa {Item Layers}
 */
 
 QQuickShaderEffect::QQuickShaderEffect(QQuickItem *parent)
@@ -659,6 +694,8 @@ QQuickShaderEffect::QQuickShaderEffect(QQuickItem *parent)
     , m_dirtyParseLog(true)
     , m_dirtyMesh(true)
     , m_dirtyGeometry(true)
+    , m_customVertexShader(false)
+    , m_supportsAtlasTextures(false)
 {
     setFlag(QQuickItem::ItemHasContents);
 }
@@ -712,6 +749,7 @@ void QQuickShaderEffect::setVertexShader(const QByteArray &code)
     m_common.source.sourceCode[Key::VertexShader] = code;
     m_dirtyProgram = true;
     m_dirtyParseLog = true;
+    m_customVertexShader = true;
 
     if (isComponentComplete())
         m_common.updateShader(this, Key::VertexShader);
@@ -822,6 +860,36 @@ void QQuickShaderEffect::setCullMode(CullMode face)
     emit cullModeChanged();
 }
 
+/*!
+    \qmlproperty bool QtQuick::ShaderEffect::supportsAtlasTextures
+
+    Set this property true to confirm that your shader code doesn't rely on
+    qt_MultiTexCoord0 ranging from (0,0) to (1,1) relative to the mesh.
+    In this case the range of qt_MultiTexCoord0 will rather be based on the position
+    of the texture within the atlas. This property currently has no effect if there
+    is less, or more, than one sampler uniform used as input to your shader.
+
+    This differs from providing qt_SubRect_<name> uniforms in that the latter allows
+    drawing one or more textures from the atlas in a single ShaderEffect item, while
+    supportsAtlasTextures allows multiple instances of a ShaderEffect component using
+    a different source image from the atlas to be batched in a single draw.
+    Both prevent a texture from being copied out of the atlas when referenced by a ShaderEffect.
+
+    The default value is false.
+
+    \since 5.4
+    \since QtQuick 2.4
+*/
+
+void QQuickShaderEffect::setSupportsAtlasTextures(bool supports)
+{
+    if (supports == m_supportsAtlasTextures)
+        return;
+    m_supportsAtlasTextures = supports;
+    updateGeometry();
+    emit supportsAtlasTexturesChanged();
+}
+
 QString QQuickShaderEffect::parseLog()
 {
     if (m_dirtyParseLog) {
@@ -934,39 +1002,6 @@ QSGNode *QQuickShaderEffect::updatePaintNode(QSGNode *oldNode, UpdatePaintNodeDa
         connect(node, SIGNAL(logAndStatusChanged(QString,int)), this, SLOT(updateLogAndStatus(QString,int)));
     }
 
-    if (m_dirtyMesh) {
-        node->setGeometry(0);
-        m_dirtyMesh = false;
-        m_dirtyGeometry = true;
-    }
-
-    if (m_dirtyGeometry) {
-        node->setFlag(QSGNode::OwnsGeometry, false);
-        QSGGeometry *geometry = node->geometry();
-        QRectF rect(0, 0, width(), height());
-        QQuickShaderEffectMesh *mesh = m_mesh ? m_mesh : &m_defaultMesh;
-
-        geometry = mesh->updateGeometry(geometry, m_common.attributes, rect);
-        if (!geometry) {
-            QString log = mesh->log();
-            if (!log.isNull()) {
-                m_log = parseLog();
-                m_log += QLatin1String("*** Mesh ***\n");
-                m_log += log;
-                m_status = Error;
-                emit logChanged();
-                emit statusChanged();
-            }
-            delete node;
-            return 0;
-        }
-
-        node->setGeometry(geometry);
-        node->setFlag(QSGNode::OwnsGeometry, true);
-
-        m_dirtyGeometry = false;
-    }
-
     QQuickShaderEffectMaterial *material = static_cast<QQuickShaderEffectMaterial *>(node->material());
 
     // Update blending
@@ -1008,6 +1043,59 @@ QSGNode *QQuickShaderEffect::updatePaintNode(QSGNode *oldNode, UpdatePaintNodeDa
         m_dirtyUniforms = m_dirtyUniformValues = m_dirtyTextureProviders = false;
     }
 
+    QRectF srcRect(0, 0, 1, 1);
+    bool geometryUsesTextureSubRect = false;
+    if (m_supportsAtlasTextures && material->textureProviders.size() == 1) {
+        QSGTextureProvider *provider = material->textureProviders.at(0);
+        if (provider->texture()) {
+            srcRect = provider->texture()->normalizedTextureSubRect();
+            geometryUsesTextureSubRect = true;
+        }
+    }
+
+    if (bool(material->flags() & QSGMaterial::RequiresFullMatrix) != m_customVertexShader) {
+        material->setFlag(QSGMaterial::RequiresFullMatrix, m_customVertexShader);
+        node->markDirty(QSGNode::DirtyMaterial);
+    }
+
+    if (material->geometryUsesTextureSubRect != geometryUsesTextureSubRect) {
+        material->geometryUsesTextureSubRect = geometryUsesTextureSubRect;
+        node->markDirty(QSGNode::DirtyMaterial);
+    }
+
+    if (m_dirtyMesh) {
+        node->setGeometry(0);
+        m_dirtyMesh = false;
+        m_dirtyGeometry = true;
+    }
+
+    if (m_dirtyGeometry) {
+        node->setFlag(QSGNode::OwnsGeometry, false);
+        QSGGeometry *geometry = node->geometry();
+        QRectF rect(0, 0, width(), height());
+        QQuickShaderEffectMesh *mesh = m_mesh ? m_mesh : &m_defaultMesh;
+
+        geometry = mesh->updateGeometry(geometry, m_common.attributes, srcRect, rect);
+        if (!geometry) {
+            QString log = mesh->log();
+            if (!log.isNull()) {
+                m_log = parseLog();
+                m_log += QLatin1String("*** Mesh ***\n");
+                m_log += log;
+                m_status = Error;
+                emit logChanged();
+                emit statusChanged();
+            }
+            delete node;
+            return 0;
+        }
+
+        node->setGeometry(geometry);
+        node->setFlag(QSGNode::OwnsGeometry, true);
+
+        m_dirtyGeometry = false;
+    }
+
     return node;
 }
 
