@@ -1,40 +1,32 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the QtQuick module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -47,6 +39,7 @@
 #include <QPainter>
 #include <QSet>
 #include <QtGui/qopengl.h>
+#include <QOpenGLFunctions>
 
 QT_BEGIN_NAMESPACE
 
@@ -208,10 +201,12 @@ int QQuickSpriteEngine::spriteFrames(int sprite)
         extra = (m_sprites[state]->m_generatedCount - 1) - extra;
 
 
-    if (extra == m_sprites[state]->m_generatedCount - 1)//last state
-        return m_sprites[state]->frames() % m_sprites[state]->m_framesPerRow;
-    else
-        return m_sprites[state]->m_framesPerRow;
+    if (extra == m_sprites[state]->m_generatedCount - 1) {//last state
+        const int framesRemaining = m_sprites[state]->frames() % m_sprites[state]->m_framesPerRow;
+        if (framesRemaining > 0)
+            return framesRemaining;
+    }
+    return m_sprites[state]->m_framesPerRow;
 }
 
 int QQuickSpriteEngine::spriteDuration(int sprite)//Full duration, not per frame
@@ -226,10 +221,12 @@ int QQuickSpriteEngine::spriteDuration(int sprite)//Full duration, not per frame
     if (m_sprites[state]->reverse())
         extra = (m_sprites[state]->m_generatedCount - 1) - extra;
 
-    if (extra == m_sprites[state]->m_generatedCount - 1)//last state
-        return m_duration[sprite] % rowDuration;
-    else
-        return rowDuration;
+    if (extra == m_sprites[state]->m_generatedCount - 1) {//last state
+        const int durationRemaining = m_duration[sprite] % rowDuration;
+        if (durationRemaining > 0)
+            return durationRemaining;
+    }
+    return rowDuration;
 }
 
 int QQuickSpriteEngine::spriteY(int sprite)
@@ -388,7 +385,11 @@ QImage QQuickSpriteEngine::assembledImage()
     m_imageStateCount = 0;
     int maxSize = 0;
 
-    glGetIntegerv(GL_MAX_TEXTURE_SIZE, &maxSize);
+    //If there is no current OpenGL Context
+    if (!QOpenGLContext::currentContext())
+        return QImage();
+
+    QOpenGLContext::currentContext()->functions()->glGetIntegerv(GL_MAX_TEXTURE_SIZE, &maxSize);
 #ifdef SPRITE_IMAGE_DEBUG
     qDebug() << "MAX TEXTURE SIZE" << maxSize;
 #endif
@@ -515,7 +516,9 @@ void QQuickStochasticEngine::start(int index, int state)
     else
         m_startTimes[index] = 0;
     m_goals[index] = -1;
+    m_addAdvance = false;
     restart(index);
+    m_addAdvance = true;
 }
 
 void QQuickStochasticEngine::stop(int index)
