@@ -1,40 +1,32 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the QtQuick module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -49,15 +41,47 @@
 #include <QtGui/qtextobject.h>
 #include <QtGui/qtexttable.h>
 #include <QtGui/qtextlist.h>
-#include <QtQuick/qsgsimplerectnode.h>
 
 #include <private/qquicktext_p_p.h>
 #include <private/qtextdocumentlayout_p.h>
 #include <private/qtextimagehandler_p.h>
 #include <private/qrawfont_p.h>
+#include <private/qglyphrun_p.h>
 
 QT_BEGIN_NAMESPACE
 
+QQuickTextNodeEngine::BinaryTreeNodeKey::BinaryTreeNodeKey(BinaryTreeNode *node)
+    : fontEngine(QRawFontPrivate::get(node->glyphRun.rawFont())->fontEngine)
+    , clipNode(node->clipNode)
+    , color(node->color.rgba())
+    , selectionState(node->selectionState)
+{
+}
+
+QQuickTextNodeEngine::BinaryTreeNode::BinaryTreeNode(const QGlyphRun &g,
+                                                     SelectionState selState,
+                                                     const QRectF &brect,
+                                                     const QQuickTextNode::Decorations &decs,
+                                                     const QColor &c,
+                                                     const QColor &bc,
+                                                     const QPointF &pos, qreal a)
+    : glyphRun(g)
+    , boundingRect(brect)
+    , selectionState(selState)
+    , clipNode(0)
+    , decorations(decs)
+    , color(c)
+    , backgroundColor(bc)
+    , position(pos)
+    , ascent(a)
+    , leftChildIndex(-1)
+    , rightChildIndex(-1)
+{
+    QGlyphRunPrivate *d = QGlyphRunPrivate::get(g);
+    ranges.append(qMakePair(d->textRangeStart, d->textRangeEnd));
+}
+
+
 void QQuickTextNodeEngine::BinaryTreeNode::insert(QVarLengthArray<BinaryTreeNode, 16> *binaryTree, const QGlyphRun &glyphRun, SelectionState selectionState,
                                              QQuickTextNode::Decorations decorations, const QColor &textColor,
                                              const QColor &backgroundColor, const QPointF &position)
@@ -74,8 +98,14 @@ void QQuickTextNodeEngine::BinaryTreeNode::insert(QVarLengthArray<BinaryTreeNode
     decorations |= (backgroundColor.isValid() ? QQuickTextNode::Background : QQuickTextNode::NoDecoration);
 
     qreal ascent = glyphRun.rawFont().ascent();
-    insert(binaryTree, BinaryTreeNode(glyphRun, selectionState, searchRect, decorations,
-                                      textColor, backgroundColor, position, ascent));
+    insert(binaryTree, BinaryTreeNode(glyphRun,
+                                      selectionState,
+                                      searchRect,
+                                      decorations,
+                                      textColor,
+                                      backgroundColor,
+                                      position,
+                                      ascent));
 }
 
 void QQuickTextNodeEngine::BinaryTreeNode::insert(QVarLengthArray<BinaryTreeNode, 16> *binaryTree, const BinaryTreeNode &binaryTreeNode)
@@ -169,7 +199,10 @@ void QQuickTextNodeEngine::addTextDecorations(const QVarLengthArray<TextDecorati
 
         {
             QRectF &rect = textDecoration.rect;
-            rect.setY(qRound(rect.y() + m_currentLine.ascent() + offset));
+            rect.setY(qRound(rect.y()
+                             + m_currentLine.ascent()
+                             + (m_currentLine.leadingIncluded() ? m_currentLine.leading() : qreal(0.0f))
+                             + offset));
             rect.setHeight(thickness);
         }
 
@@ -256,12 +289,9 @@ void QQuickTextNodeEngine::processCurrentLine()
             // selection rect to the graph, and we add decoration every time the
             // selection state changes, because that means the text color changes
             if (node == 0 || node->selectionState != currentSelectionState) {
-                if (node != 0)
-                    currentRect.setRight(node->boundingRect.left());
                 currentRect.setY(m_position.y() + m_currentLine.y());
                 currentRect.setHeight(m_currentLine.height());
 
-                // Draw selection all the way up to the left edge of the unselected item
                 if (currentSelectionState == Selected)
                     m_selectionRects.append(currentRect);
 
@@ -297,8 +327,10 @@ void QQuickTextNodeEngine::processCurrentLine()
             }
 
             if (node != 0) {
-                node->clipNode = currentClipNode;
-                currentClipNodeUsed = true;
+                if (node->selectionState == Selected) {
+                    node->clipNode = currentClipNode;
+                    currentClipNodeUsed = true;
+                }
 
                 decorationRect = node->boundingRect;
 
@@ -384,7 +416,7 @@ void QQuickTextNodeEngine::addImage(const QRectF &rect, const QImage &image, qre
     QRectF searchRect = rect;
     if (layoutPosition == QTextFrameFormat::InFlow) {
         if (m_currentLineTree.isEmpty()) {
-            searchRect.moveTopLeft(m_position + m_currentLine.position());
+            searchRect.moveTopLeft(m_position + m_currentLine.position() + QPointF(0,1));
         } else {
             const BinaryTreeNode *lastNode = m_currentLineTree.data() + m_currentLineTree.size() - 1;
             if (lastNode->glyphRun.isRightToLeft()) {
@@ -453,15 +485,25 @@ void QQuickTextNodeEngine::addTextObject(const QPointF &position, const QTextCha
 
 void QQuickTextNodeEngine::addUnselectedGlyphs(const QGlyphRun &glyphRun)
 {
-    BinaryTreeNode::insert(&m_currentLineTree, glyphRun, Unselected,
-                           QQuickTextNode::NoDecoration, m_textColor, m_backgroundColor, m_position);
+    BinaryTreeNode::insert(&m_currentLineTree,
+                           glyphRun,
+                           Unselected,
+                           QQuickTextNode::NoDecoration,
+                           m_textColor,
+                           m_backgroundColor,
+                           m_position);
 }
 
 void QQuickTextNodeEngine::addSelectedGlyphs(const QGlyphRun &glyphRun)
 {
     int currentSize = m_currentLineTree.size();
-    BinaryTreeNode::insert(&m_currentLineTree, glyphRun, Selected,
-                           QQuickTextNode::NoDecoration, m_textColor, m_backgroundColor, m_position);
+    BinaryTreeNode::insert(&m_currentLineTree,
+                           glyphRun,
+                           Selected,
+                           QQuickTextNode::NoDecoration,
+                           m_textColor,
+                           m_backgroundColor,
+                           m_position);
     m_hasSelection = m_hasSelection || m_currentLineTree.size() > currentSize;
 }
 
@@ -538,10 +580,8 @@ void QQuickTextNodeEngine::addGlyphsInRange(int rangeStart, int rangeLength,
         }
     } else {
         if (rangeStart < selectionStart) {
-            QList<QGlyphRun> glyphRuns = line.glyphRuns(rangeStart,
-                                                        qMin(selectionStart - rangeStart,
-                                                             rangeLength));
-
+            int length = qMin(selectionStart - rangeStart, rangeLength);
+            QList<QGlyphRun> glyphRuns = line.glyphRuns(rangeStart, length);
             for (int j=0; j<glyphRuns.size(); ++j) {
                 const QGlyphRun &glyphRun = glyphRuns.at(j);
                 addUnselectedGlyphs(glyphRun);
@@ -560,7 +600,9 @@ void QQuickTextNodeEngine::addGlyphsInRange(int rangeStart, int rangeLength,
         }
 
         if (selectionEnd >= rangeStart && selectionEnd < rangeEnd) {
-            QList<QGlyphRun> glyphRuns = line.glyphRuns(selectionEnd + 1, rangeEnd - selectionEnd - 1);
+            int start = selectionEnd + 1;
+            int length = rangeEnd - selectionEnd - 1;
+            QList<QGlyphRun> glyphRuns = line.glyphRuns(start, length);
             for (int j=0; j<glyphRuns.size(); ++j) {
                 const QGlyphRun &glyphRun = glyphRuns.at(j);
                 addUnselectedGlyphs(glyphRun);
@@ -635,6 +677,72 @@ void QQuickTextNodeEngine::addFrameDecorations(QTextDocument *document, QTextFra
     }
 }
 
+uint qHash(const QQuickTextNodeEngine::BinaryTreeNodeKey &key)
+{
+    // Just use the default hash for pairs
+    return qHash(qMakePair(key.fontEngine, qMakePair(key.clipNode,
+                                                     qMakePair(key.color, key.selectionState))));
+}
+
+void QQuickTextNodeEngine::mergeProcessedNodes(QList<BinaryTreeNode *> *regularNodes,
+                                               QList<BinaryTreeNode *> *imageNodes)
+{
+    QHash<BinaryTreeNodeKey, QList<BinaryTreeNode *> > map;
+
+    for (int i = 0; i < m_processedNodes.size(); ++i) {
+        BinaryTreeNode *node = m_processedNodes.data() + i;
+
+        if (node->image.isNull()) {
+            BinaryTreeNodeKey key(node);
+
+            QList<BinaryTreeNode *> &nodes = map[key];
+            if (nodes.isEmpty())
+                regularNodes->append(node);
+
+            nodes.append(node);
+        } else {
+            imageNodes->append(node);
+        }
+    }
+
+    for (int i = 0; i < regularNodes->size(); ++i) {
+        BinaryTreeNode *primaryNode = regularNodes->at(i);
+        BinaryTreeNodeKey key(primaryNode);
+
+        const QList<BinaryTreeNode *> &nodes = map.value(key);
+        Q_ASSERT(nodes.first() == primaryNode);
+
+        int count = 0;
+        for (int j = 0; j < nodes.size(); ++j)
+            count += nodes.at(j)->glyphRun.glyphIndexes().size();
+
+        if (count != primaryNode->glyphRun.glyphIndexes().size()) {
+            QGlyphRun &glyphRun = primaryNode->glyphRun;
+            QVector<quint32> glyphIndexes = glyphRun.glyphIndexes();
+            glyphIndexes.reserve(count);
+
+            QVector<QPointF> glyphPositions = glyphRun.positions();
+            glyphPositions.reserve(count);
+
+            for (int j = 1; j < nodes.size(); ++j) {
+                BinaryTreeNode *otherNode = nodes.at(j);
+                glyphIndexes += otherNode->glyphRun.glyphIndexes();
+                primaryNode->ranges += otherNode->ranges;
+
+                QVector<QPointF> otherPositions = otherNode->glyphRun.positions();
+                for (int k = 0; k < otherPositions.size(); ++k)
+                    glyphPositions += otherPositions.at(k) + (otherNode->position - primaryNode->position);
+            }
+
+            Q_ASSERT(glyphPositions.size() == count);
+            Q_ASSERT(glyphIndexes.size() == count);
+
+            glyphRun.setGlyphIndexes(glyphIndexes);
+            glyphRun.setPositions(glyphPositions);
+        }
+    }
+}
+
 void  QQuickTextNodeEngine::addToSceneGraph(QQuickTextNode *parentNode,
                                             QQuickText::TextStyle style,
                                             const QColor &styleColor)
@@ -642,95 +750,130 @@ void  QQuickTextNodeEngine::addToSceneGraph(QQuickTextNode *parentNode,
     if (m_currentLine.isValid())
         processCurrentLine();
 
+    QList<BinaryTreeNode *> nodes;
+    QList<BinaryTreeNode *> imageNodes;
+    mergeProcessedNodes(&nodes, &imageNodes);
 
-    for (int i=0; i<m_backgrounds.size(); ++i) {
+    for (int i = 0; i < m_backgrounds.size(); ++i) {
         const QRectF &rect = m_backgrounds.at(i).first;
         const QColor &color = m_backgrounds.at(i).second;
 
-        parentNode->appendChildNode(new QSGSimpleRectNode(rect, color));
+        parentNode->addRectangleNode(rect, color);
     }
 
-    // First, prepend all selection rectangles to the tree
-    for (int i=0; i<m_selectionRects.size(); ++i) {
+    // Add all text with unselected color first
+    for (int i = 0; i < nodes.size(); ++i) {
+        const BinaryTreeNode *node = nodes.at(i);
+        parentNode->addGlyphs(node->position, node->glyphRun, node->color, style, styleColor, 0);
+    }
+
+    for (int i = 0; i < imageNodes.size(); ++i) {
+        const BinaryTreeNode *node = imageNodes.at(i);
+        if (node->selectionState == Unselected)
+            parentNode->addImage(node->boundingRect, node->image);
+    }
+
+    // Then, prepend all selection rectangles to the tree
+    for (int i = 0; i < m_selectionRects.size(); ++i) {
         const QRectF &rect = m_selectionRects.at(i);
 
-        parentNode->appendChildNode(new QSGSimpleRectNode(rect, m_selectionColor));
+        parentNode->addRectangleNode(rect, m_selectionColor);
     }
 
-    // Finally, add decorations for each node to the tree.
-    for (int i=0; i<m_lines.size(); ++i) {
+    // Add decorations for each node to the tree.
+    for (int i = 0; i < m_lines.size(); ++i) {
         const TextDecoration &textDecoration = m_lines.at(i);
 
         QColor color = textDecoration.selectionState == Selected
                 ? m_selectedTextColor
                 : textDecoration.color;
 
-        parentNode->appendChildNode(new QSGSimpleRectNode(textDecoration.rect, color));
+        parentNode->addRectangleNode(textDecoration.rect, color);
     }
 
-    // Then, go through all the nodes for all lines and combine all QGlyphRuns with a common
-    // font, selection state and clip node.
-    typedef QPair<QFontEngine *, QPair<QQuickDefaultClipNode *, QPair<QRgb, int> > > KeyType;
-    QHash<KeyType, BinaryTreeNode *> map;
-    QList<BinaryTreeNode *> nodes;
-    for (int i = 0; i < m_processedNodes.size(); ++i) {
-        BinaryTreeNode *node = m_processedNodes.data() + i;
+    // Finally add the selected text on top of everything
+    for (int i = 0; i < nodes.size(); ++i) {
+        const BinaryTreeNode *node = nodes.at(i);
+        QQuickDefaultClipNode *clipNode = node->clipNode;
+        if (clipNode != 0 && clipNode->parent() == 0)
+            parentNode->appendChildNode(clipNode);
 
-        if (node->image.isNull()) {
-            QGlyphRun glyphRun = node->glyphRun;
-            QRawFont rawFont = glyphRun.rawFont();
-            QRawFontPrivate *rawFontD = QRawFontPrivate::get(rawFont);
+        if (node->selectionState == Selected) {
+            QColor color = m_selectedTextColor;
+            int previousNodeIndex = i - 1;
+            int nextNodeIndex = i + 1;
+            const BinaryTreeNode *previousNode = previousNodeIndex < 0 ? 0 : nodes.at(previousNodeIndex);
+            while (previousNode != 0 && qFuzzyCompare(previousNode->boundingRect.left(), node->boundingRect.left()))
+                previousNode = --previousNodeIndex < 0 ? 0 : nodes.at(previousNodeIndex);
 
-            QFontEngine *fontEngine = rawFontD->fontEngine;
+            const BinaryTreeNode *nextNode = nextNodeIndex == nodes.size() ? 0 : nodes.at(nextNodeIndex);
 
-            KeyType key(qMakePair(fontEngine,
-                                  qMakePair(node->clipNode,
-                                            qMakePair(node->color.rgba(), int(node->selectionState)))));
+            if (previousNode != 0 && previousNode->selectionState == Unselected)
+                parentNode->addGlyphs(previousNode->position, previousNode->glyphRun, color, style, styleColor, clipNode);
 
-            BinaryTreeNode *otherNode = map.value(key, 0);
-            if (otherNode != 0) {
-                QGlyphRun &otherGlyphRun = otherNode->glyphRun;
+            if (nextNode != 0 && nextNode->selectionState == Unselected)
+                parentNode->addGlyphs(nextNode->position, nextNode->glyphRun, color, style, styleColor, clipNode);
 
-                QVector<quint32> otherGlyphIndexes = otherGlyphRun.glyphIndexes();
-                QVector<QPointF> otherGlyphPositions = otherGlyphRun.positions();
+            // If the previous or next node completely overlaps this one, then we have already drawn the glyphs of
+            // this node
+            bool drawCurrent = false;
+            if (previousNode != 0 || nextNode != 0) {
+                for (int i = 0; i < node->ranges.size(); ++i) {
+                    const QPair<int, int> &range = node->ranges.at(i);
 
-                otherGlyphIndexes += glyphRun.glyphIndexes();
+                    int rangeLength = range.second - range.first;
+                    if (previousNode != 0) {
+                        for (int j = 0; j < previousNode->ranges.size(); ++j) {
+                            const QPair<int, int> &otherRange = previousNode->ranges.at(j);
+                            if (range.first < otherRange.second && range.second > otherRange.first) {
+                                int start = qMax(range.first, otherRange.first);
+                                int end = qMin(range.second, otherRange.second);
+                                rangeLength -= end - start;
+                                if (rangeLength == 0)
+                                    break;
+                            }
+                        }
+                    }
 
-                QVector<QPointF> glyphPositions = glyphRun.positions();
-                otherGlyphPositions.reserve(otherGlyphPositions.size() + glyphPositions.size());
-                for (int j = 0; j < glyphPositions.size(); ++j) {
-                    otherGlyphPositions += glyphPositions.at(j) + (node->position - otherNode->position);
+                    if (nextNode != 0 && rangeLength > 0) {
+                        for (int j = 0; j < nextNode->ranges.size(); ++j) {
+                            const QPair<int, int> &otherRange = nextNode->ranges.at(j);
+
+                            if (range.first < otherRange.second && range.second > otherRange.first) {
+                                int start = qMax(range.first, otherRange.first);
+                                int end = qMin(range.second, otherRange.second);
+                                rangeLength -= end - start;
+                                if (rangeLength == 0)
+                                    break;
+                            }
+                        }
+                    }
+
+                    if (rangeLength > 0) {
+                        drawCurrent = true;
+                        break;
+                    }
                 }
-
-                otherGlyphRun.setGlyphIndexes(otherGlyphIndexes);
-                otherGlyphRun.setPositions(otherGlyphPositions);
-
             } else {
-                map.insert(key, node);
-                nodes.append(node);
+                drawCurrent = true;
             }
-        } else {
+
+            if (drawCurrent)
+                parentNode->addGlyphs(node->position, node->glyphRun, color, style, styleColor, clipNode);
+        }
+    }
+
+    for (int i = 0; i < imageNodes.size(); ++i) {
+        const BinaryTreeNode *node = imageNodes.at(i);
+        if (node->selectionState == Selected) {
             parentNode->addImage(node->boundingRect, node->image);
             if (node->selectionState == Selected) {
                 QColor color = m_selectionColor;
                 color.setAlpha(128);
-                parentNode->appendChildNode(new QSGSimpleRectNode(node->boundingRect, color));
+                parentNode->addRectangleNode(node->boundingRect, color);
             }
         }
     }
-
-    foreach (const BinaryTreeNode *node, nodes) {
-
-        QQuickDefaultClipNode *clipNode = node->clipNode;
-        if (clipNode != 0 && clipNode->parent() == 0 )
-            parentNode->appendChildNode(clipNode);
-
-        QColor color = node->selectionState == Selected
-                ? m_selectedTextColor
-                : node->color;
-
-        parentNode->addGlyphs(node->position, node->glyphRun, color, style, styleColor, clipNode);
-    }
 }
 
 void QQuickTextNodeEngine::mergeFormats(QTextLayout *textLayout, QVarLengthArray<QTextLayout::FormatRange> *mergedFormats)
@@ -867,7 +1010,18 @@ void QQuickTextNodeEngine::addTextBlock(QTextDocument *textDocument, const QText
             continue;
 
         QTextCharFormat charFormat = fragment.charFormat();
-        setPosition(blockPosition);
+        QFont font(charFormat.font());
+        QFontMetricsF fontMetrics(font);
+
+        int fontHeight = fontMetrics.descent() + fontMetrics.ascent();
+        int valign = charFormat.verticalAlignment();
+        if (valign == QTextCharFormat::AlignSuperScript)
+            setPosition(QPointF(blockPosition.x(), blockPosition.y() - fontHeight / 2));
+        else if (valign == QTextCharFormat::AlignSubScript)
+            setPosition(QPointF(blockPosition.x(), blockPosition.y() + fontHeight / 6));
+        else
+            setPosition(blockPosition);
+
         if (text.contains(QChar::ObjectReplacementCharacter)) {
             QTextFrame *frame = qobject_cast<QTextFrame *>(textDocument->objectForFormat(charFormat));
             if (frame && frame->frameFormat().position() == QTextFrameFormat::InFlow) {
@@ -903,6 +1057,13 @@ void QQuickTextNodeEngine::addTextBlock(QTextDocument *textDocument, const QText
                 fragmentEnd += preeditLength;
             }
 #endif
+            if (charFormat.background().style() != Qt::NoBrush) {
+                QTextLayout::FormatRange additionalFormat;
+                additionalFormat.start = textPos - block.position();
+                additionalFormat.length = fragmentEnd - textPos;
+                additionalFormat.format = charFormat;
+                colorChanges << additionalFormat;
+            }
 
             textPos = addText(block, charFormat, textColor, colorChanges, textPos, fragmentEnd,
                                                  selectionStart, selectionEnd);
