@@ -1,40 +1,32 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the QtQuick module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -42,8 +34,6 @@
 #include "qquicktranslate_p.h"
 #include "qquickitem_p.h"
 
-#include <QtCore/qmath.h>
-
 QT_BEGIN_NAMESPACE
 
 class QQuickTranslatePrivate : public QQuickTransformPrivate
@@ -336,7 +326,7 @@ public:
 
     \image axisrotation.png
 
-    \sa {declarative/ui-components/dialcontrol}{Dial Control example}, {Qt Quick Demo - Clocks}
+    \sa {customitems/dialcontrol}{Dial Control example}, {Qt Quick Demo - Clocks}
 */
 QQuickRotation::QQuickRotation(QObject *parent)
     : QQuickTransform(*new QQuickRotationPrivate, parent)
@@ -458,4 +448,79 @@ void QQuickRotation::applyTo(QMatrix4x4 *matrix) const
     matrix->translate(-d->origin);
 }
 
+class QQuickMatrix4x4Private : public QQuickTransformPrivate
+{
+public:
+    QQuickMatrix4x4Private()
+        : matrix() {}
+    QMatrix4x4 matrix;
+};
+
+/*!
+    \qmltype Matrix4x4
+    \instantiates QQuickMatrix4x4
+    \inqmlmodule QtQuick
+    \ingroup qtquick-visual-transforms
+    \brief Provides a way to apply a 4x4 tranformation matrix to an \l Item
+
+    The Matrix4x4 type provides a way to apply a transformation to an
+    \l Item through a 4x4 matrix.
+
+    It allows for a combination of rotation, scale, translatation and shearing
+    by using just one tranformation provided in a 4x4-matrix.
+
+    The following example rotates a Rectangle 45 degress (PI/4):
+
+    \qml
+    Rectangle {
+        width: 100
+        height: 100
+        color: "red"
+
+        transform: Matrix4x4 {
+            property real a: Math.PI / 4
+            matrix: Qt.matrix4x4(Math.cos(a), -Math.sin(a), 0, 0,
+                                 Math.sin(a),  Math.cos(a), 0, 0,
+                                 0,           0,            1, 0,
+                                 0,           0,            0, 1)
+        }
+    }
+    \endqml
+*/
+QQuickMatrix4x4::QQuickMatrix4x4(QObject *parent)
+    : QQuickTransform(*new QQuickMatrix4x4Private, parent)
+{
+}
+
+QQuickMatrix4x4::~QQuickMatrix4x4()
+{
+}
+
+/*!
+    \qmlproperty QMatrix4x4 QtQuick::Matrix4x4::matrix
+
+    4x4-matrix which will be used in the tranformation of an \l Item
+*/
+QMatrix4x4 QQuickMatrix4x4::matrix() const
+{
+    Q_D(const QQuickMatrix4x4);
+    return d->matrix;
+}
+
+void QQuickMatrix4x4::setMatrix(const QMatrix4x4 &matrix)
+{
+    Q_D(QQuickMatrix4x4);
+    if (d->matrix == matrix)
+         return;
+    d->matrix = matrix;
+    update();
+    emit matrixChanged();
+}
+
+void QQuickMatrix4x4::applyTo(QMatrix4x4 *matrix) const
+{
+    Q_D(const QQuickMatrix4x4);
+    *matrix *= d->matrix;
+}
+
 QT_END_NAMESPACE
