@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the Qt scene graph research project.
 **
@@ -10,9 +10,9 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia. For licensing terms and
-** conditions see http://qt.digia.com/licensing. For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
@@ -23,8 +23,8 @@
 ** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights. These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** $QT_END_LICENSE$
@@ -522,7 +522,8 @@ void QSGGeometry::setDrawingMode(GLenum mode)
 /*!
     Gets the current line or point width or to be used for this geometry. This property
     only applies to line width when the drawingMode is \c GL_LINES, \c GL_LINE_STRIP, or
-    \c GL_LINE_LOOP, and only applies to point size when the drawingMode is \c GL_POINTS.
+    \c GL_LINE_LOOP. For desktop OpenGL, it also applies to point size when the drawingMode
+    is \c GL_POINTS.
 
     The default value is \c 1.0
 
@@ -536,7 +537,12 @@ float QSGGeometry::lineWidth() const
 /*!
     Sets the line or point width to be used for this geometry to \a width. This property
     only applies to line width when the drawingMode is \c GL_LINES, \c GL_LINE_STRIP, or
-    \c GL_LINE_LOOP, and only applies to point size when the drawingMode is \c GL_POINTS.
+    \c GL_LINE_LOOP. For Desktop OpenGL, it also applies to point size when the drawingMode
+    is \c GL_POINTS.
+
+    \note How line width and point size are treated is implementation dependent: The application
+    should not rely on these, but rather create triangles or similar to draw areas. On OpenGL ES,
+    line width support is limited and point size is unsupported.
 
     \sa lineWidth(), drawingMode()
 */
