@@ -1,40 +1,32 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the QtQuick module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -99,8 +91,8 @@ static void qt_print_node_count()
     together. Nodes in a blocked subtree will not be preprocessed() and not
     rendered.
 
-    \warning Anything related to QSGNode should happen on the scene graph
-    rendering thread.
+    \note All classes with QSG prefix should be used solely on the scene graph's
+    rendering thread. See \l {Scene Graph and Rendering} for more information.
  */
 
 /*!
@@ -135,6 +127,7 @@ static void qt_print_node_count()
     \value OwnsOpaqueMaterial Only valid for QSGGeometryNode. The node has
     ownership over the opaque material and will delete it when the node is
     destroyed or a material is assigned.
+    \value InternalReserved Reserved for internal use.
  */
 
 /*!
@@ -511,7 +504,7 @@ void QSGNode::insertChildNodeAfter(QSGNode *node, QSGNode *after)
 {
     //Q_ASSERT_X(!m_children.contains(node), "QSGNode::insertChildNodeAfter", "QSGNode is already a child!");
     Q_ASSERT_X(!node->m_parent, "QSGNode::insertChildNodeAfter", "QSGNode already has a parent");
-    Q_ASSERT_X(after && after->m_parent == this, "QSGNode::insertChildNodeBefore", "The parent of \'before\' is wrong");
+    Q_ASSERT_X(after && after->m_parent == this, "QSGNode::insertChildNodeAfter", "The parent of \'after\' is wrong");
 
 #ifndef QT_NO_DEBUG
     if (node->type() == QSGNode::GeometryNodeType) {
@@ -582,6 +575,19 @@ void QSGNode::removeAllChildNodes()
     }
 }
 
+/*!
+ * \internal
+ *
+ * Reparents all nodes of this node to \a newParent.
+ */
+void QSGNode::reparentChildNodesTo(QSGNode *newParent)
+{
+    for (QSGNode *c = firstChild(); c; c = firstChild()) {
+        removeChildNode(c);
+        newParent->appendChildNode(c);
+    }
+}
+
 
 int QSGNode::childCount() const
 {
@@ -648,10 +654,7 @@ void QSGNode::setFlags(Flags f, bool enabled)
 
 
 /*!
-    Marks this node with the states in \a bits as dirty.
-
-    When a node is marked dirty, it recursively marks the parent chain
-    as dirty and notify all connected renderers that the has dirty states.
+    Notifies all connected renderers that the node has dirty \a bits.
  */
 
 void QSGNode::markDirty(DirtyState bits)
@@ -671,12 +674,15 @@ void QSGNode::markDirty(DirtyState bits)
     }
 }
 
-#ifdef QSG_RUNTIME_DESCRIPTION
 void qsgnode_set_description(QSGNode *node, const QString &description)
 {
+#ifdef QSG_RUNTIME_DESCRIPTION
     QSGNodePrivate::setDescription(node, description);
-}
+#else
+    Q_UNUSED(node);
+    Q_UNUSED(description);
 #endif
+}
 
 /*!
     \class QSGBasicGeometryNode
@@ -686,6 +692,9 @@ void qsgnode_set_description(QSGNode *node, const QString &description)
 
     The QSGBasicGeometryNode class should not be used by itself. It is only encapsulates
     shared functionality between the QSGGeometryNode and QSGClipNode classes.
+
+    \note All classes with QSG prefix should be used solely on the scene graph's
+    rendering thread. See \l {Scene Graph and Rendering} for more information.
   */
 
 
@@ -778,7 +787,7 @@ QSGBasicGeometryNode::~QSGBasicGeometryNode()
 
 void QSGBasicGeometryNode::setGeometry(QSGGeometry *geometry)
 {
-    if (flags() & OwnsGeometry)
+    if ((flags() & OwnsGeometry) != 0 && m_geometry != geometry)
         delete m_geometry;
     m_geometry = geometry;
     markDirty(DirtyGeometry);
@@ -797,7 +806,7 @@ void QSGBasicGeometryNode::setGeometry(QSGGeometry *geometry)
     the vertices and their structure, to be drawn. The Material defines how the shape is
     filled.
 
-    The following is a code snipped illustrating how to create a red
+    The following is a code snippet illustrating how to create a red
     line using a QSGGeometryNode:
     \code
         QSGGeometry *geometry = new QSGGeometry(QSGGeometry::defaultAttributes_Point2D(), 2);
@@ -827,6 +836,9 @@ void QSGBasicGeometryNode::setGeometry(QSGGeometry *geometry)
     to avoid an extra operation in the fragment shader can have significant performance
     impact on embedded graphics chips. The opaque material is optional.
 
+    \note All classes with QSG prefix should be used solely on the scene graph's
+    rendering thread. See \l {Scene Graph and Rendering} for more information.
+
     \sa QSGGeometry, QSGMaterial, QSGSimpleMaterial
  */
 
@@ -942,7 +954,7 @@ void QSGGeometryNode::setRenderOrder(int order)
  */
 void QSGGeometryNode::setMaterial(QSGMaterial *material)
 {
-    if (flags() & OwnsMaterial)
+    if ((flags() & OwnsMaterial) != 0 && m_material != material)
         delete m_material;
     m_material = material;
 #ifndef QT_NO_DEBUG
@@ -973,7 +985,7 @@ void QSGGeometryNode::setMaterial(QSGMaterial *material)
  */
 void QSGGeometryNode::setOpaqueMaterial(QSGMaterial *material)
 {
-    if (flags() & OwnsOpaqueMaterial)
+    if ((flags() & OwnsOpaqueMaterial) != 0 && m_opaque_material != m_material)
         delete m_opaque_material;
     m_opaque_material = material;
 #ifndef QT_NO_DEBUG
@@ -1002,7 +1014,6 @@ void QSGGeometryNode::setOpaqueMaterial(QSGMaterial *material)
  */
 QSGMaterial *QSGGeometryNode::activeMaterial() const
 {
-    Q_ASSERT_X(dirtyState() == 0, "QSGGeometryNode::activeMaterial()", "function assumes that all dirty states are cleaned up");
     if (m_opaque_material && m_opacity > 0.999)
         return m_opaque_material;
     return m_material;
@@ -1039,6 +1050,9 @@ void QSGGeometryNode::setInheritedOpacity(qreal opacity)
     Clip nodes must have a geometry before they can be added to the scene graph.
 
     Clipping is usually implemented by using the stencil buffer.
+
+    \note All classes with QSG prefix should be used solely on the scene graph's
+    rendering thread. See \l {Scene Graph and Rendering} for more information.
  */
 
 
@@ -1130,6 +1144,10 @@ void QSGClipNode::setClipRect(const QRectF &rect)
     transformations. However, because the renderer optimizes for 2D use-cases rather
     than 3D use-cases, rendering a scene with full 3D transformations needs to
     be done with some care.
+
+    \note All classes with QSG prefix should be used solely on the scene graph's
+    rendering thread. See \l {Scene Graph and Rendering} for more information.
+
  */
 
 
@@ -1267,6 +1285,8 @@ void QSGRootNode::notifyNodeChange(QSGNode *node, DirtyState state)
     be marked as blocked, causing isSubtreeBlocked() to return true. This
     is done for performance reasons.
 
+    \note All classes with QSG prefix should be used solely on the scene graph's
+    rendering thread. See \l {Scene Graph and Rendering} for more information.
  */
 
 
@@ -1437,7 +1457,7 @@ QDebug operator<<(QDebug d, const QSGGeometryNode *n)
         d << "Geometry(null)";
         return d;
     }
-    d << "GeometryNode(" << hex << (void *) n << dec;
+    d << "GeometryNode(" << hex << (const void *) n << dec;
 
     const QSGGeometry *g = n->geometry();
 
@@ -1479,7 +1499,6 @@ QDebug operator<<(QDebug d, const QSGGeometryNode *n)
 #ifdef QSG_RUNTIME_DESCRIPTION
     d << QSGNodePrivate::description(n);
 #endif
-    d << "dirty=" << hex << (int) n->dirtyState() << dec;
     return d;
 }
 
@@ -1489,7 +1508,7 @@ QDebug operator<<(QDebug d, const QSGClipNode *n)
         d << "ClipNode(null)";
         return d;
     }
-    d << "ClipNode(" << hex << (void *) n << dec;
+    d << "ClipNode(" << hex << (const void *) n << dec;
 
     if (n->childCount())
         d << "children=" << n->childCount();
@@ -1500,7 +1519,7 @@ QDebug operator<<(QDebug d, const QSGClipNode *n)
 #ifdef QSG_RUNTIME_DESCRIPTION
     d << QSGNodePrivate::description(n);
 #endif
-    d << "dirty=" << hex << (int) n->dirtyState() << dec << (n->isSubtreeBlocked() ? "*BLOCKED*" : "");
+    d << (n->isSubtreeBlocked() ? "*BLOCKED*" : "");
     return d;
 }
 
@@ -1512,7 +1531,7 @@ QDebug operator<<(QDebug d, const QSGTransformNode *n)
     }
     const QMatrix4x4 m = n->matrix();
     d << "TransformNode(";
-    d << hex << (void *) n << dec;
+    d << hex << (const void *) n << dec;
     if (m.isIdentity())
         d << "identity";
     else if (m.determinant() == 1 && m(0, 0) == 1 && m(1, 1) == 1 && m(2, 2) == 1)
@@ -1522,7 +1541,7 @@ QDebug operator<<(QDebug d, const QSGTransformNode *n)
 #ifdef QSG_RUNTIME_DESCRIPTION
     d << QSGNodePrivate::description(n);
 #endif
-    d << "dirty=" << hex << (int) n->dirtyState() << dec << (n->isSubtreeBlocked() ? "*BLOCKED*" : "");
+    d << (n->isSubtreeBlocked() ? "*BLOCKED*" : "");
     d << ')';
     return d;
 }
@@ -1534,14 +1553,13 @@ QDebug operator<<(QDebug d, const QSGOpacityNode *n)
         return d;
     }
     d << "OpacityNode(";
-    d << hex << (void *) n << dec;
+    d << hex << (const void *) n << dec;
     d << "opacity=" << n->opacity()
       << "combined=" << n->combinedOpacity()
       << (n->isSubtreeBlocked() ? "*BLOCKED*" : "");
 #ifdef QSG_RUNTIME_DESCRIPTION
     d << QSGNodePrivate::description(n);
 #endif
-    d << "dirty=" << hex << (int) n->dirtyState() << dec;
     d << ')';
     return d;
 }
@@ -1553,8 +1571,7 @@ QDebug operator<<(QDebug d, const QSGRootNode *n)
         d << "RootNode(null)";
         return d;
     }
-    d << "RootNode" << hex << (void *) n << "dirty=" << (int) n->dirtyState() << dec
-      << (n->isSubtreeBlocked() ? "*BLOCKED*" : "");
+    d << "RootNode" << hex << (const void *) n << (n->isSubtreeBlocked() ? "*BLOCKED*" : "");
 #ifdef QSG_RUNTIME_DESCRIPTION
     d << QSGNodePrivate::description(n);
 #endif
@@ -1587,8 +1604,7 @@ QDebug operator<<(QDebug d, const QSGNode *n)
         d << static_cast<const QSGOpacityNode *>(n);
         break;
     case QSGNode::RenderNodeType:
-        d << "RenderNode(" << hex << (void *) n << dec
-          << "dirty=" << hex << (int) n->dirtyState()
+        d << "RenderNode(" << hex << (const void *) n << dec
           << "flags=" << (int) n->flags() << dec
           << (n->isSubtreeBlocked() ? "*BLOCKED*" : "");
 #ifdef QSG_RUNTIME_DESCRIPTION
@@ -1597,8 +1613,7 @@ QDebug operator<<(QDebug d, const QSGNode *n)
         d << ')';
         break;
     default:
-        d << "Node(" << hex << (void *) n << dec
-          << "dirty=" << hex << (int) n->dirtyState()
+        d << "Node(" << hex << (const void *) n << dec
           << "flags=" << (int) n->flags() << dec
           << (n->isSubtreeBlocked() ? "*BLOCKED*" : "");
 #ifdef QSG_RUNTIME_DESCRIPTION
