@@ -1,40 +1,32 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the QtQuick module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -46,6 +38,7 @@
 #include <QtQml/private/qqmlglobal_p.h>
 #include <QtQuick/private/qsgdistancefieldutil_p.h>
 #include <qopenglfunctions.h>
+#include <qopenglframebufferobject.h>
 #include <qmath.h>
 
 #if !defined(QT_OPENGL_ES_2)
@@ -55,34 +48,30 @@
 QT_BEGIN_NAMESPACE
 
 DEFINE_BOOL_CONFIG_OPTION(qmlUseGlyphCacheWorkaround, QML_USE_GLYPHCACHE_WORKAROUND)
+DEFINE_BOOL_CONFIG_OPTION(qsgPreferFullSizeGlyphCacheTextures, QSG_PREFER_FULLSIZE_GLYPHCACHE_TEXTURES)
+
+#if !defined(QSG_DEFAULT_DISTANCEFIELD_GLYPH_CACHE_PADDING)
+#  define QSG_DEFAULT_DISTANCEFIELD_GLYPH_CACHE_PADDING 2
+#endif
 
 QSGDefaultDistanceFieldGlyphCache::QSGDefaultDistanceFieldGlyphCache(QSGDistanceFieldGlyphCacheManager *man, QOpenGLContext *c, const QRawFont &font)
     : QSGDistanceFieldGlyphCache(man, c, font)
     , m_maxTextureSize(0)
     , m_maxTextureCount(3)
     , m_blitProgram(0)
+    , m_blitBuffer(QOpenGLBuffer::VertexBuffer)
     , m_fboGuard(0)
+    , m_funcs(c->functions())
 #if !defined(QT_OPENGL_ES_2)
-    , m_funcs(0)
+    , m_coreFuncs(0)
 #endif
 {
-    m_blitVertexCoordinateArray[0] = -1.0f;
-    m_blitVertexCoordinateArray[1] = -1.0f;
-    m_blitVertexCoordinateArray[2] =  1.0f;
-    m_blitVertexCoordinateArray[3] = -1.0f;
-    m_blitVertexCoordinateArray[4] =  1.0f;
-    m_blitVertexCoordinateArray[5] =  1.0f;
-    m_blitVertexCoordinateArray[6] = -1.0f;
-    m_blitVertexCoordinateArray[7] =  1.0f;
-
-    m_blitTextureCoordinateArray[0] = 0.0f;
-    m_blitTextureCoordinateArray[1] = 0.0f;
-    m_blitTextureCoordinateArray[2] = 1.0f;
-    m_blitTextureCoordinateArray[3] = 0.0f;
-    m_blitTextureCoordinateArray[4] = 1.0f;
-    m_blitTextureCoordinateArray[5] = 1.0f;
-    m_blitTextureCoordinateArray[6] = 0.0f;
-    m_blitTextureCoordinateArray[7] = 1.0f;
+    m_blitBuffer.create();
+    m_blitBuffer.bind();
+    static GLfloat buffer[16] = {-1.0f, -1.0f, 1.0f, -1.0f, 1.0f, 1.0f, -1.0f, 1.0f,
+                                 0.0f, 0.0f, 1.0f, 0.0f, 1.0f, 1.0f, 0.0f, 1.0f};
+    m_blitBuffer.allocate(buffer, sizeof(buffer));
+    m_blitBuffer.release();
 
     m_areaAllocator = new QSGAreaAllocator(QSize(maxTextureSize(), m_maxTextureCount * maxTextureSize()));
 }
@@ -90,7 +79,7 @@ QSGDefaultDistanceFieldGlyphCache::QSGDefaultDistanceFieldGlyphCache(QSGDistance
 QSGDefaultDistanceFieldGlyphCache::~QSGDefaultDistanceFieldGlyphCache()
 {
     for (int i = 0; i < m_textures.count(); ++i)
-        glDeleteTextures(1, &m_textures[i].texture);
+        m_funcs->glDeleteTextures(1, &m_textures[i].texture);
 
     if (m_fboGuard != 0)
         m_fboGuard->free();
@@ -107,8 +96,9 @@ void QSGDefaultDistanceFieldGlyphCache::requestGlyphs(const QSet<glyph_t> &glyph
     for (QSet<glyph_t>::const_iterator it = glyphs.constBegin(); it != glyphs.constEnd() ; ++it) {
         glyph_t glyphIndex = *it;
 
+        int padding = QSG_DEFAULT_DISTANCEFIELD_GLYPH_CACHE_PADDING;
         int glyphWidth = qCeil(glyphData(glyphIndex).boundingRect.width()) + distanceFieldRadius() * 2;
-        QSize glyphSize(glyphWidth, QT_DISTANCEFIELD_TILESIZE(doubleGlyphResolution()));
+        QSize glyphSize(glyphWidth + padding * 2, QT_DISTANCEFIELD_TILESIZE(doubleGlyphResolution()) + padding * 2);
         QRect alloc = m_areaAllocator->allocate(glyphSize);
 
         if (alloc.isNull()) {
@@ -118,7 +108,10 @@ void QSGDefaultDistanceFieldGlyphCache::requestGlyphs(const QSet<glyph_t> &glyph
 
                 TexCoord unusedCoord = glyphTexCoord(unusedGlyph);
                 int unusedGlyphWidth = qCeil(glyphData(unusedGlyph).boundingRect.width()) + distanceFieldRadius() * 2;
-                m_areaAllocator->deallocate(QRect(unusedCoord.x, unusedCoord.y, unusedGlyphWidth, QT_DISTANCEFIELD_TILESIZE(doubleGlyphResolution())));
+                m_areaAllocator->deallocate(QRect(unusedCoord.x - padding,
+                                                  unusedCoord.y - padding,
+                                                  padding * 2 + unusedGlyphWidth,
+                                                  padding * 2 + QT_DISTANCEFIELD_TILESIZE(doubleGlyphResolution())));
 
                 m_unusedGlyphs.remove(unusedGlyph);
                 m_glyphsTexture.remove(unusedGlyph);
@@ -134,11 +127,14 @@ void QSGDefaultDistanceFieldGlyphCache::requestGlyphs(const QSet<glyph_t> &glyph
 
         TextureInfo *tex = textureInfo(alloc.y() / maxTextureSize());
         alloc = QRect(alloc.x(), alloc.y() % maxTextureSize(), alloc.width(), alloc.height());
+
         tex->allocatedArea |= alloc;
+        Q_ASSERT(tex->padding == padding || tex->padding < 0);
+        tex->padding = padding;
 
         GlyphPosition p;
         p.glyph = glyphIndex;
-        p.position = alloc.topLeft();
+        p.position = alloc.topLeft() + QPoint(padding, padding);
 
         glyphPositions.append(p);
         glyphsToRender.append(glyphIndex);
@@ -151,13 +147,16 @@ void QSGDefaultDistanceFieldGlyphCache::requestGlyphs(const QSet<glyph_t> &glyph
 
 void QSGDefaultDistanceFieldGlyphCache::storeGlyphs(const QList<QDistanceField> &glyphs)
 {
-    QHash<TextureInfo *, QVector<glyph_t> > glyphTextures;
+    typedef QHash<TextureInfo *, QVector<glyph_t> > GlyphTextureHash;
+    typedef GlyphTextureHash::const_iterator GlyphTextureHashConstIt;
+
+    GlyphTextureHash glyphTextures;
 
     GLint alignment = 4; // default value
-    glGetIntegerv(GL_UNPACK_ALIGNMENT, &alignment);
+    m_funcs->glGetIntegerv(GL_UNPACK_ALIGNMENT, &alignment);
 
     // Distance field data is always tightly packed
-    glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
+    m_funcs->glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
 
     for (int i = 0; i < glyphs.size(); ++i) {
         QDistanceField glyph = glyphs.at(i);
@@ -166,17 +165,18 @@ void QSGDefaultDistanceFieldGlyphCache::storeGlyphs(const QList<QDistanceField>
         TextureInfo *texInfo = m_glyphsTexture.value(glyphIndex);
 
         resizeTexture(texInfo, texInfo->allocatedArea.width(), texInfo->allocatedArea.height());
-        glBindTexture(GL_TEXTURE_2D, texInfo->texture);
+        m_funcs->glBindTexture(GL_TEXTURE_2D, texInfo->texture);
 
         glyphTextures[texInfo].append(glyphIndex);
 
+        int padding = texInfo->padding;
         int expectedWidth = qCeil(c.width + c.xMargin * 2);
-        if (glyph.width() != expectedWidth)
-            glyph = glyph.copy(0, 0, expectedWidth, glyph.height());
+        glyph = glyph.copy(-padding, -padding,
+                           expectedWidth + padding  * 2, glyph.height() + padding * 2);
 
         if (useTextureResizeWorkaround()) {
             uchar *inBits = glyph.scanLine(0);
-            uchar *outBits = texInfo->image.scanLine(int(c.y)) + int(c.x);
+            uchar *outBits = texInfo->image.scanLine(int(c.y) - padding) + int(c.x) - padding;
             for (int y = 0; y < glyph.height(); ++y) {
                 memcpy(outBits, inBits, glyph.width());
                 inBits += glyph.width();
@@ -191,24 +191,23 @@ void QSGDefaultDistanceFieldGlyphCache::storeGlyphs(const QList<QDistanceField>
 #endif
         if (useTextureUploadWorkaround()) {
             for (int i = 0; i < glyph.height(); ++i) {
-                glTexSubImage2D(GL_TEXTURE_2D, 0,
-                                c.x, c.y + i, glyph.width(),1,
-                                format, GL_UNSIGNED_BYTE,
-                                glyph.scanLine(i));
+                m_funcs->glTexSubImage2D(GL_TEXTURE_2D, 0,
+                                         c.x - padding, c.y + i - padding, glyph.width(),1,
+                                         format, GL_UNSIGNED_BYTE,
+                                         glyph.scanLine(i));
             }
         } else {
-            glTexSubImage2D(GL_TEXTURE_2D, 0,
-                            c.x, c.y, glyph.width(), glyph.height(),
-                            format, GL_UNSIGNED_BYTE,
-                            glyph.constBits());
+            m_funcs->glTexSubImage2D(GL_TEXTURE_2D, 0,
+                                     c.x - padding, c.y - padding, glyph.width(), glyph.height(),
+                                     format, GL_UNSIGNED_BYTE,
+                                     glyph.constBits());
         }
     }
 
     // restore to previous alignment
-    glPixelStorei(GL_UNPACK_ALIGNMENT, alignment);
+    m_funcs->glPixelStorei(GL_UNPACK_ALIGNMENT, alignment);
 
-    QHash<TextureInfo *, QVector<glyph_t> >::const_iterator i;
-    for (i = glyphTextures.constBegin(); i != glyphTextures.constEnd(); ++i) {
+    for (GlyphTextureHashConstIt i = glyphTextures.constBegin(), cend = glyphTextures.constEnd(); i != cend; ++i) {
         Texture t;
         t.textureId = i.key()->texture;
         t.size = i.key()->size;
@@ -231,17 +230,18 @@ void QSGDefaultDistanceFieldGlyphCache::createTexture(TextureInfo *texInfo, int
     if (useTextureResizeWorkaround() && texInfo->image.isNull())
         texInfo->image = QDistanceField(width, height);
 
-    while (glGetError() != GL_NO_ERROR) { }
+    while (m_funcs->glGetError() != GL_NO_ERROR) { }
 
-    glGenTextures(1, &texInfo->texture);
-    glBindTexture(GL_TEXTURE_2D, texInfo->texture);
+    m_funcs->glGenTextures(1, &texInfo->texture);
+    m_funcs->glBindTexture(GL_TEXTURE_2D, texInfo->texture);
 
-    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
-    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
-    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
-    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
+    m_funcs->glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
+    m_funcs->glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
+    m_funcs->glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
+    m_funcs->glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
 #if !defined(QT_OPENGL_ES_2)
-    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAX_LEVEL, 0);
+    if (!QOpenGLContext::currentContext()->isOpenGLES())
+        m_funcs->glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAX_LEVEL, 0);
     const GLint internalFormat = isCoreProfile() ? GL_R8 : GL_ALPHA;
     const GLenum format = isCoreProfile() ? GL_RED : GL_ALPHA;
 #else
@@ -249,14 +249,14 @@ void QSGDefaultDistanceFieldGlyphCache::createTexture(TextureInfo *texInfo, int
     const GLenum format = GL_ALPHA;
 #endif
 
-    glTexImage2D(GL_TEXTURE_2D, 0, internalFormat, width, height, 0, format, GL_UNSIGNED_BYTE, 0);
+    m_funcs->glTexImage2D(GL_TEXTURE_2D, 0, internalFormat, width, height, 0, format, GL_UNSIGNED_BYTE, 0);
 
     texInfo->size = QSize(width, height);
 
-    GLuint error = glGetError();
+    GLuint error = m_funcs->glGetError();
     if (error != GL_NO_ERROR) {
-        glBindTexture(GL_TEXTURE_2D, 0);
-        glDeleteTextures(1, &texInfo->texture);
+        m_funcs->glBindTexture(GL_TEXTURE_2D, 0);
+        m_funcs->glDeleteTextures(1, &texInfo->texture);
         texInfo->texture = 0;
     }
 
@@ -290,42 +290,42 @@ void QSGDefaultDistanceFieldGlyphCache::resizeTexture(TextureInfo *texInfo, int
         // For an OpenGL Core Profile we can use http://www.opengl.org/wiki/Framebuffer#Blitting
         // to efficiently copy the contents of the old texture to the new texture
         // TODO: Use ARB_copy_image if available of if we have >=4.3 context
-        if (!m_funcs) {
-            m_funcs = ctx->versionFunctions<QOpenGLFunctions_3_2_Core>();
-            Q_ASSERT(m_funcs);
-            m_funcs->initializeOpenGLFunctions();
+        if (!m_coreFuncs) {
+            m_coreFuncs = ctx->versionFunctions<QOpenGLFunctions_3_2_Core>();
+            Q_ASSERT(m_coreFuncs);
+            m_coreFuncs->initializeOpenGLFunctions();
         }
 
         // Create a framebuffer object to which we can attach our old and new textures (to
         // the first two color buffer attachment points)
         if (!m_fboGuard) {
             GLuint fbo;
-            m_funcs->glGenFramebuffers(1, &fbo);
+            m_coreFuncs->glGenFramebuffers(1, &fbo);
             m_fboGuard = new QOpenGLSharedResourceGuard(ctx, fbo, freeFramebufferFunc);
         }
 
         // Bind the FBO to both the GL_READ_FRAMEBUFFER? and GL_DRAW_FRAMEBUFFER targets
-        m_funcs->glBindFramebuffer(GL_FRAMEBUFFER, m_fboGuard->id());
+        m_coreFuncs->glBindFramebuffer(GL_FRAMEBUFFER, m_fboGuard->id());
 
         // Bind the old texture to GL_COLOR_ATTACHMENT0
-        m_funcs->glFramebufferTexture2D(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0,
+        m_coreFuncs->glFramebufferTexture2D(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0,
                                         GL_TEXTURE_2D, oldTexture, 0);
 
         // Bind the new texture to GL_COLOR_ATTACHMENT1
-        m_funcs->glFramebufferTexture2D(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT1,
+        m_coreFuncs->glFramebufferTexture2D(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT1,
                                         GL_TEXTURE_2D, texInfo->texture, 0);
 
         // Set the source and destination buffers
-        m_funcs->glReadBuffer(GL_COLOR_ATTACHMENT0);
-        m_funcs->glDrawBuffer(GL_COLOR_ATTACHMENT1);
+        m_coreFuncs->glReadBuffer(GL_COLOR_ATTACHMENT0);
+        m_coreFuncs->glDrawBuffer(GL_COLOR_ATTACHMENT1);
 
         // Do the blit
-        m_funcs->glBlitFramebuffer(0, 0, oldWidth, oldHeight,
+        m_coreFuncs->glBlitFramebuffer(0, 0, oldWidth, oldHeight,
                                    0, 0, oldWidth, oldHeight,
                                    GL_COLOR_BUFFER_BIT, GL_NEAREST);
 
         // Reset the default framebuffer
-        m_funcs->glBindFramebuffer(GL_FRAMEBUFFER, 0);
+        QOpenGLFramebufferObject::bindDefault();
 
         return;
     } else if (useTextureResizeWorkaround()) {
@@ -333,8 +333,8 @@ void QSGDefaultDistanceFieldGlyphCache::resizeTexture(TextureInfo *texInfo, int
     if (useTextureResizeWorkaround()) {
 #endif
         GLint alignment = 4; // default value
-        glGetIntegerv(GL_UNPACK_ALIGNMENT, &alignment);
-        glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
+        m_funcs->glGetIntegerv(GL_UNPACK_ALIGNMENT, &alignment);
+        m_funcs->glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
 
 #if !defined(QT_OPENGL_ES_2)
         const GLenum format = isCoreProfile() ? GL_RED : GL_ALPHA;
@@ -344,22 +344,22 @@ void QSGDefaultDistanceFieldGlyphCache::resizeTexture(TextureInfo *texInfo, int
 
         if (useTextureUploadWorkaround()) {
             for (int i = 0; i < texInfo->image.height(); ++i) {
-                glTexSubImage2D(GL_TEXTURE_2D, 0,
-                                0, i, oldWidth, 1,
-                                format, GL_UNSIGNED_BYTE,
-                                texInfo->image.scanLine(i));
+                m_funcs->glTexSubImage2D(GL_TEXTURE_2D, 0,
+                                         0, i, oldWidth, 1,
+                                         format, GL_UNSIGNED_BYTE,
+                                         texInfo->image.scanLine(i));
             }
         } else {
-            glTexSubImage2D(GL_TEXTURE_2D, 0,
-                            0, 0, oldWidth, oldHeight,
-                            format, GL_UNSIGNED_BYTE,
-                            texInfo->image.constBits());
+            m_funcs->glTexSubImage2D(GL_TEXTURE_2D, 0,
+                                     0, 0, oldWidth, oldHeight,
+                                     format, GL_UNSIGNED_BYTE,
+                                     texInfo->image.constBits());
         }
 
-        glPixelStorei(GL_UNPACK_ALIGNMENT, alignment); // restore to previous value
+        m_funcs->glPixelStorei(GL_UNPACK_ALIGNMENT, alignment); // restore to previous value
 
         texInfo->image = texInfo->image.copy(0, 0, width, height);
-        glDeleteTextures(1, &oldTexture);
+        m_funcs->glDeleteTextures(1, &oldTexture);
         return;
     }
 
@@ -370,29 +370,30 @@ void QSGDefaultDistanceFieldGlyphCache::resizeTexture(TextureInfo *texInfo, int
 
     if (!m_fboGuard) {
         GLuint fbo;
-        ctx->functions()->glGenFramebuffers(1, &fbo);
+        m_funcs->glGenFramebuffers(1, &fbo);
         m_fboGuard = new QOpenGLSharedResourceGuard(ctx, fbo, freeFramebufferFunc);
     }
-    ctx->functions()->glBindFramebuffer(GL_FRAMEBUFFER, m_fboGuard->id());
+    m_funcs->glBindFramebuffer(GL_FRAMEBUFFER, m_fboGuard->id());
 
     GLuint tmp_texture;
-    glGenTextures(1, &tmp_texture);
-    glBindTexture(GL_TEXTURE_2D, tmp_texture);
-    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
-    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
-    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
-    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
+    m_funcs->glGenTextures(1, &tmp_texture);
+    m_funcs->glBindTexture(GL_TEXTURE_2D, tmp_texture);
+    m_funcs->glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
+    m_funcs->glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
+    m_funcs->glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
+    m_funcs->glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
 #if !defined(QT_OPENGL_ES_2)
-    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAX_LEVEL, 0);
+    if (!ctx->isOpenGLES())
+        m_funcs->glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAX_LEVEL, 0);
 #endif
-    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, oldWidth, oldHeight, 0,
-                 GL_RGBA, GL_UNSIGNED_BYTE, NULL);
-    glBindTexture(GL_TEXTURE_2D, 0);
-    ctx->functions()->glFramebufferTexture2D(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0,
-                                             GL_TEXTURE_2D, tmp_texture, 0);
+    m_funcs->glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, oldWidth, oldHeight, 0,
+                          GL_RGBA, GL_UNSIGNED_BYTE, NULL);
+    m_funcs->glBindTexture(GL_TEXTURE_2D, 0);
+    m_funcs->glFramebufferTexture2D(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0,
+                                    GL_TEXTURE_2D, tmp_texture, 0);
 
-    ctx->functions()->glActiveTexture(GL_TEXTURE0);
-    glBindTexture(GL_TEXTURE_2D, oldTexture);
+    m_funcs->glActiveTexture(GL_TEXTURE0);
+    m_funcs->glBindTexture(GL_TEXTURE_2D, oldTexture);
 
     // save current render states
     GLboolean stencilTestEnabled;
@@ -401,61 +402,72 @@ void QSGDefaultDistanceFieldGlyphCache::resizeTexture(TextureInfo *texInfo, int
     GLboolean blendEnabled;
     GLint viewport[4];
     GLint oldProgram;
-    glGetBooleanv(GL_STENCIL_TEST, &stencilTestEnabled);
-    glGetBooleanv(GL_DEPTH_TEST, &depthTestEnabled);
-    glGetBooleanv(GL_SCISSOR_TEST, &scissorTestEnabled);
-    glGetBooleanv(GL_BLEND, &blendEnabled);
-    glGetIntegerv(GL_VIEWPORT, &viewport[0]);
-    glGetIntegerv(GL_CURRENT_PROGRAM, &oldProgram);
+    m_funcs->glGetBooleanv(GL_STENCIL_TEST, &stencilTestEnabled);
+    m_funcs->glGetBooleanv(GL_DEPTH_TEST, &depthTestEnabled);
+    m_funcs->glGetBooleanv(GL_SCISSOR_TEST, &scissorTestEnabled);
+    m_funcs->glGetBooleanv(GL_BLEND, &blendEnabled);
+    m_funcs->glGetIntegerv(GL_VIEWPORT, &viewport[0]);
+    m_funcs->glGetIntegerv(GL_CURRENT_PROGRAM, &oldProgram);
 
-    glDisable(GL_STENCIL_TEST);
-    glDisable(GL_DEPTH_TEST);
-    glDisable(GL_SCISSOR_TEST);
-    glDisable(GL_BLEND);
+    m_funcs->glDisable(GL_STENCIL_TEST);
+    m_funcs->glDisable(GL_DEPTH_TEST);
+    m_funcs->glDisable(GL_SCISSOR_TEST);
+    m_funcs->glDisable(GL_BLEND);
 
-    glViewport(0, 0, oldWidth, oldHeight);
-
-    ctx->functions()->glVertexAttribPointer(QT_VERTEX_COORDS_ATTR, 2, GL_FLOAT, GL_FALSE, 0, m_blitVertexCoordinateArray);
-    ctx->functions()->glVertexAttribPointer(QT_TEXTURE_COORDS_ATTR, 2, GL_FLOAT, GL_FALSE, 0, m_blitTextureCoordinateArray);
+    m_funcs->glViewport(0, 0, oldWidth, oldHeight);
 
+    const bool vaoInit = m_vao.isCreated();
+    if (isCoreProfile()) {
+        if ( !vaoInit )
+            m_vao.create();
+        m_vao.bind();
+    }
     m_blitProgram->bind();
-    m_blitProgram->enableAttributeArray(int(QT_VERTEX_COORDS_ATTR));
-    m_blitProgram->enableAttributeArray(int(QT_TEXTURE_COORDS_ATTR));
+    if (!vaoInit || !isCoreProfile()) {
+        m_blitBuffer.bind();
+
+        m_blitProgram->enableAttributeArray(int(QT_VERTEX_COORDS_ATTR));
+        m_blitProgram->enableAttributeArray(int(QT_TEXTURE_COORDS_ATTR));
+        m_blitProgram->setAttributeBuffer(int(QT_VERTEX_COORDS_ATTR), GL_FLOAT, 0, 2);
+        m_blitProgram->setAttributeBuffer(int(QT_TEXTURE_COORDS_ATTR), GL_FLOAT, 32, 2);
+    }
     m_blitProgram->disableAttributeArray(int(QT_OPACITY_ATTR));
     m_blitProgram->setUniformValue("imageTexture", GLuint(0));
 
-    glDrawArrays(GL_TRIANGLE_FAN, 0, 4);
+    m_funcs->glDrawArrays(GL_TRIANGLE_FAN, 0, 4);
 
-    glBindTexture(GL_TEXTURE_2D, texInfo->texture);
+    m_funcs->glBindTexture(GL_TEXTURE_2D, texInfo->texture);
 
     if (useTextureUploadWorkaround()) {
         for (int i = 0; i < oldHeight; ++i)
-            glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, i, 0, i, oldWidth, 1);
+            m_funcs->glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, i, 0, i, oldWidth, 1);
     } else {
-        glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, 0, 0, oldWidth, oldHeight);
+        m_funcs->glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, 0, 0, oldWidth, oldHeight);
     }
 
-    ctx->functions()->glFramebufferRenderbuffer(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0,
-                                                GL_RENDERBUFFER, 0);
-    glDeleteTextures(1, &tmp_texture);
-    glDeleteTextures(1, &oldTexture);
+    m_funcs->glFramebufferRenderbuffer(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0,
+                                       GL_RENDERBUFFER, 0);
+    m_funcs->glDeleteTextures(1, &tmp_texture);
+    m_funcs->glDeleteTextures(1, &oldTexture);
 
-    ctx->functions()->glBindFramebuffer(GL_FRAMEBUFFER, 0);
+    QOpenGLFramebufferObject::bindDefault();
 
     // restore render states
     if (stencilTestEnabled)
-        glEnable(GL_STENCIL_TEST);
+        m_funcs->glEnable(GL_STENCIL_TEST);
     if (depthTestEnabled)
-        glEnable(GL_DEPTH_TEST);
+        m_funcs->glEnable(GL_DEPTH_TEST);
     if (scissorTestEnabled)
-        glEnable(GL_SCISSOR_TEST);
+        m_funcs->glEnable(GL_SCISSOR_TEST);
     if (blendEnabled)
-        glEnable(GL_BLEND);
-    glViewport(viewport[0], viewport[1], viewport[2], viewport[3]);
-    ctx->functions()->glUseProgram(oldProgram);
+        m_funcs->glEnable(GL_BLEND);
+    m_funcs->glViewport(viewport[0], viewport[1], viewport[2], viewport[3]);
+    m_funcs->glUseProgram(oldProgram);
 
     m_blitProgram->disableAttributeArray(int(QT_VERTEX_COORDS_ATTR));
     m_blitProgram->disableAttributeArray(int(QT_TEXTURE_COORDS_ATTR));
+    if (isCoreProfile())
+        m_vao.release();
 }
 
 bool QSGDefaultDistanceFieldGlyphCache::useTextureResizeWorkaround() const
@@ -476,17 +488,22 @@ bool QSGDefaultDistanceFieldGlyphCache::useTextureUploadWorkaround() const
     static bool set = false;
     static bool useWorkaround = false;
     if (!set) {
-        useWorkaround = qstrcmp(reinterpret_cast<const char*>(glGetString(GL_RENDERER)),
+        useWorkaround = qstrcmp(reinterpret_cast<const char*>(m_funcs->glGetString(GL_RENDERER)),
                                 "Mali-400 MP") == 0;
         set = true;
     }
     return useWorkaround;
 }
 
+bool QSGDefaultDistanceFieldGlyphCache::createFullSizeTextures() const
+{
+    return qsgPreferFullSizeGlyphCacheTextures() && glyphCount() > QT_DISTANCEFIELD_HIGHGLYPHCOUNT;
+}
+
 int QSGDefaultDistanceFieldGlyphCache::maxTextureSize() const
 {
     if (!m_maxTextureSize)
-        glGetIntegerv(GL_MAX_TEXTURE_SIZE, &m_maxTextureSize);
+        m_funcs->glGetIntegerv(GL_MAX_TEXTURE_SIZE, &m_maxTextureSize);
     return m_maxTextureSize;
 }
 
