@@ -1,40 +1,32 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the QtQuick module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -49,6 +41,7 @@
 #include <private/qfontengine_p.h>
 #include <private/qopenglextensions_p.h>
 
+#include <QtQuick/qquickwindow.h>
 #include <QtQuick/private/qsgtexture_p.h>
 
 #include <private/qrawfont_p.h>
@@ -60,12 +53,31 @@ QT_BEGIN_NAMESPACE
 #define GL_FRAMEBUFFER_SRGB 0x8DB9
 #endif
 
+#ifndef GL_FRAMEBUFFER_SRGB_CAPABLE
+#define GL_FRAMEBUFFER_SRGB_CAPABLE 0x8DBA
+#endif
+
 static inline QVector4D qsg_premultiply(const QVector4D &c, float globalOpacity)
 {
     float o = c.w() * globalOpacity;
     return QVector4D(c.x() * o, c.y() * o, c.z() * o, o);
 }
 
+static inline int qsg_device_pixel_ratio(QOpenGLContext *ctx)
+{
+    int devicePixelRatio = 1;
+    if (ctx->surface()->surfaceClass() == QSurface::Window) {
+        QWindow *w = static_cast<QWindow *>(ctx->surface());
+        if (QQuickWindow *qw = qobject_cast<QQuickWindow *>(w))
+            devicePixelRatio = qw->effectiveDevicePixelRatio();
+        else
+            devicePixelRatio = w->devicePixelRatio();
+    } else {
+        devicePixelRatio = ctx->screen()->devicePixelRatio();
+    }
+    return devicePixelRatio;
+}
+
 class QSGTextMaskShader : public QSGMaterialShader
 {
 public:
@@ -80,6 +92,7 @@ protected:
     int m_matrix_id;
     int m_color_id;
     int m_textureScale_id;
+    float m_devicePixelRatio;
 
     QFontEngine::GlyphFormat m_glyphFormat;
 };
@@ -109,6 +122,8 @@ void QSGTextMaskShader::initialize()
     m_matrix_id = program()->uniformLocation("matrix");
     m_color_id = program()->uniformLocation("color");
     m_textureScale_id = program()->uniformLocation("textureScale");
+    m_devicePixelRatio = (float) qsg_device_pixel_ratio(QOpenGLContext::currentContext());
+    program()->setUniformValue("dpr", m_devicePixelRatio);
 }
 
 void QSGTextMaskShader::updateState(const RenderState &state, QSGMaterial *newEffect, QSGMaterial *oldEffect)
@@ -125,38 +140,25 @@ void QSGTextMaskShader::updateState(const RenderState &state, QSGMaterial *newEf
             || oldMaterial->texture()->textureId() != material->texture()->textureId()) {
         program()->setUniformValue(m_textureScale_id, QVector2D(1.0 / material->cacheTextureWidth(),
                                                                1.0 / material->cacheTextureHeight()));
-        glBindTexture(GL_TEXTURE_2D, material->texture()->textureId());
+        QOpenGLFunctions *funcs = QOpenGLContext::currentContext()->functions();
+        funcs->glBindTexture(GL_TEXTURE_2D, material->texture()->textureId());
 
         // Set the mag/min filters to be nearest. We only need to do this when the texture
         // has been recreated.
         if (updated) {
-            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
-            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
+            funcs->glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
+            funcs->glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
         }
     }
 
-    if (state.isMatrixDirty()) {
-        QMatrix4x4 transform = state.modelViewMatrix();
-        qreal xTranslation = transform(0, 3);
-        qreal yTranslation = transform(1, 3);
-
-        // Remove translation and check identity to see if matrix is only translating.
-        // If it is, we can round the translation to make sure the text is pixel aligned,
-        // which is the only thing that works with GL_NEAREST filtering. Adding rotations
-        // and scales to native rendered text is not a prioritized use case, since the
-        // default rendering type is designed for that.
-        transform(0, 3) = 0.0;
-        transform(1, 3) = 0.0;
-        if (transform.isIdentity()) {
-            transform(0, 3) = qRound(xTranslation);
-            transform(1, 3) = qRound(yTranslation);
-
-            transform = state.projectionMatrix() * transform;
-            program()->setUniformValue(m_matrix_id, transform);
-        } else {
-            program()->setUniformValue(m_matrix_id, state.combinedMatrix());
-        }
+    float devicePixelRatio = (float) qsg_device_pixel_ratio(QOpenGLContext::currentContext());
+    if (m_devicePixelRatio != devicePixelRatio) {
+        m_devicePixelRatio = devicePixelRatio;
+        program()->setUniformValue("dpr", m_devicePixelRatio);
     }
+
+    if (state.isMatrixDirty())
+        program()->setUniformValue(m_matrix_id, state.combinedMatrix());
 }
 
 class QSG8BitTextMaskShader : public QSGTextMaskShader
@@ -209,22 +211,28 @@ void QSG24BitTextMaskShader::initialize()
     if (QOpenGLContext::currentContext()->hasExtension(QByteArrayLiteral("GL_ARB_framebuffer_sRGB"))
             && m_glyphFormat == QFontEngine::Format_A32
             && qAbs(fontSmoothingGamma() - 2.2) < 0.25) {
-        m_useSRGB = true;
+        QOpenGLFunctions *funcs = QOpenGLContext::currentContext()->functions();
+        GLint srgbCapable = 0;
+        funcs->glGetIntegerv(GL_FRAMEBUFFER_SRGB_CAPABLE, &srgbCapable);
+        if (srgbCapable)
+            m_useSRGB = true;
     }
 }
 
 void QSG24BitTextMaskShader::activate()
 {
-    glBlendFunc(GL_CONSTANT_COLOR, GL_ONE_MINUS_SRC_COLOR);
+    QOpenGLFunctions *funcs = QOpenGLContext::currentContext()->functions();
+    funcs->glBlendFunc(GL_CONSTANT_COLOR, GL_ONE_MINUS_SRC_COLOR);
     if (m_useSRGB)
-        glEnable(GL_FRAMEBUFFER_SRGB);
+        funcs->glEnable(GL_FRAMEBUFFER_SRGB);
 }
 
 void QSG24BitTextMaskShader::deactivate()
 {
-    glBlendFunc(GL_ONE, GL_ONE_MINUS_SRC_ALPHA);
+    QOpenGLFunctions *funcs = QOpenGLContext::currentContext()->functions();
+    funcs->glBlendFunc(GL_ONE, GL_ONE_MINUS_SRC_ALPHA);
     if (m_useSRGB)
-        glDisable(GL_FRAMEBUFFER_SRGB);
+        funcs->glDisable(GL_FRAMEBUFFER_SRGB);
 }
 
 static inline qreal qt_sRGB_to_linear_RGB(qreal f)
@@ -250,12 +258,36 @@ void QSG24BitTextMaskShader::updateState(const RenderState &state, QSGMaterial *
         QVector4D color = material->color();
         if (m_useSRGB)
             color = qt_sRGB_to_linear_RGB(color);
-        state.context()->functions()->glBlendColor(color.x(), color.y(), color.z(), color.w());
+        QOpenGLContext::currentContext()->functions()->glBlendColor(color.x(), color.y(), color.z(), color.w());
         color = qsg_premultiply(color, state.opacity());
         program()->setUniformValue(m_color_id, color.w());
     }
 }
 
+class QSG32BitColorTextShader : public QSGTextMaskShader
+{
+public:
+    QSG32BitColorTextShader(QFontEngine::GlyphFormat glyphFormat)
+        : QSGTextMaskShader(glyphFormat)
+    {
+        setShaderSourceFile(QOpenGLShader::Fragment, QStringLiteral(":/scenegraph/shaders/32bitcolortext.frag"));
+    }
+
+    void updateState(const RenderState &state, QSGMaterial *newEffect, QSGMaterial *oldEffect) Q_DECL_OVERRIDE;
+};
+
+void QSG32BitColorTextShader::updateState(const RenderState &state, QSGMaterial *newEffect, QSGMaterial *oldEffect)
+{
+    QSGTextMaskShader::updateState(state, newEffect, oldEffect);
+    QSGTextMaskMaterial *material = static_cast<QSGTextMaskMaterial *>(newEffect);
+    QSGTextMaskMaterial *oldMaterial = static_cast<QSGTextMaskMaterial *>(oldEffect);
+
+    if (oldMaterial == Q_NULLPTR || material->color() != oldMaterial->color() || state.isOpacityDirty()) {
+        float opacity = material->color().w() * state.opacity();
+        program()->setUniformValue(m_color_id, opacity);
+    }
+}
+
 class QSGStyledTextShader : public QSG8BitTextMaskShader
 {
 public:
@@ -313,13 +345,14 @@ void QSGStyledTextShader::updateState(const RenderState &state,
             || oldMaterial->texture()->textureId() != material->texture()->textureId()) {
         program()->setUniformValue(m_textureScale_id, QVector2D(1.0 / material->cacheTextureWidth(),
                                                                 1.0 / material->cacheTextureHeight()));
-        glBindTexture(GL_TEXTURE_2D, material->texture()->textureId());
+        QOpenGLFunctions *funcs = QOpenGLContext::currentContext()->functions();
+        funcs->glBindTexture(GL_TEXTURE_2D, material->texture()->textureId());
 
         // Set the mag/min filters to be linear. We only need to do this when the texture
         // has been recreated.
         if (updated) {
-            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
-            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
+            funcs->glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
+            funcs->glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
         }
     }
 
@@ -374,8 +407,8 @@ void QSGTextMaskMaterial::init(QFontEngine::GlyphFormat glyphFormat)
                         : QFontEngine::Format_A32;
         }
 
-        qreal devicePixelRatio = ctx->surface()->surfaceClass() == QSurface::Window ?
-            static_cast<QWindow *>(ctx->surface())->devicePixelRatio() : ctx->screen()->devicePixelRatio();
+        qreal devicePixelRatio = qsg_device_pixel_ratio(ctx);
+
 
         QTransform glyphCacheTransform = QTransform::fromScale(devicePixelRatio, devicePixelRatio);
         if (!fontEngine->supportsTransformation(glyphCacheTransform))
@@ -483,8 +516,16 @@ void QSGTextMaskMaterial::populate(const QPointF &p,
 
 QSGMaterialType *QSGTextMaskMaterial::type() const
 {
-    static QSGMaterialType rgb, gray;
-    return glyphCache()->glyphFormat() == QFontEngine::Format_A32 ? &rgb : &gray;
+    static QSGMaterialType argb, rgb, gray;
+    switch (glyphCache()->glyphFormat()) {
+    case QFontEngine::Format_ARGB:
+        return &argb;
+    case QFontEngine::Format_A32:
+        return &rgb;
+    case QFontEngine::Format_A8:
+    default:
+        return &gray;
+    }
 }
 
 QOpenGLTextureGlyphCache *QSGTextMaskMaterial::glyphCache() const
@@ -494,10 +535,15 @@ QOpenGLTextureGlyphCache *QSGTextMaskMaterial::glyphCache() const
 
 QSGMaterialShader *QSGTextMaskMaterial::createShader() const
 {
-    QFontEngine::GlyphFormat glyphFormat = glyphCache()->glyphFormat();
-    return glyphFormat == QFontEngine::Format_A32
-           ? (QSGMaterialShader *) new QSG24BitTextMaskShader(glyphFormat)
-           : (QSGMaterialShader *) new QSG8BitTextMaskShader(glyphFormat);
+    switch (QFontEngine::GlyphFormat glyphFormat = glyphCache()->glyphFormat()) {
+    case QFontEngine::Format_ARGB:
+        return new QSG32BitColorTextShader(glyphFormat);
+    case QFontEngine::Format_A32:
+        return new QSG24BitTextMaskShader(glyphFormat);
+    case QFontEngine::Format_A8:
+    default:
+        return new QSG8BitTextMaskShader(glyphFormat);
+    }
 }
 
 static inline int qsg_colorDiff(const QVector4D &a, const QVector4D &b)
