@@ -1,41 +1,33 @@
 
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the QtQuick module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -289,10 +281,22 @@ void QSGDefaultRectangleNode::update()
     if (m_dirty_geometry) {
         updateGeometry();
         m_dirty_geometry = false;
+
+        QSGNode::DirtyState state = QSGNode::DirtyGeometry;
+        // smoothed material is always blended, so no change in material state
+        if (material() == &m_material) {
+            bool wasBlending = (m_material.flags() & QSGMaterial::Blending);
+            bool isBlending = (m_gradient_stops.size() > 0 && !m_gradient_is_opaque)
+                               || (m_color.alpha() < 255 && m_color.alpha() != 0)
+                               || (m_pen_width > 0 && m_border_color.alpha() < 255);
+            if (wasBlending != isBlending) {
+                m_material.setFlag(QSGMaterial::Blending, isBlending);
+                state |= QSGNode::DirtyMaterial;
+            }
+        }
+
+        markDirty(state);
     }
-    m_material.setFlag(QSGMaterial::Blending, (m_gradient_stops.size() > 0 && !m_gradient_is_opaque)
-                                               || (m_color.alpha() < 255 && m_color.alpha() != 0)
-                                               || (m_pen_width > 0 && m_border_color.alpha() < 255));
 }
 
 void QSGDefaultRectangleNode::updateGeometry()
@@ -319,11 +323,11 @@ void QSGDefaultRectangleNode::updateGeometry()
     const QGradientStops &stops = m_gradient_stops;
 
     int nextGradientStop = 0;
-    float gradientPos = 0.5f * penWidth / height;
+    float gradientPos = penWidth / height;
     while (nextGradientStop < stops.size() && stops.at(nextGradientStop).first <= gradientPos)
         ++nextGradientStop;
     int lastGradientStop = stops.size() - 1;
-    float lastGradientPos = 1.0f - 0.5f * penWidth / height;
+    float lastGradientPos = 1.0f - penWidth / height;
     while (lastGradientStop >= nextGradientStop && stops.at(lastGradientStop).first >= lastGradientPos)
         --lastGradientStop;
     int gradientIntersections = (lastGradientStop - nextGradientStop + 1);
@@ -385,7 +389,7 @@ void QSGDefaultRectangleNode::updateGeometry()
         int borderTail = 0;
         int outerAAHead = 0;
         int outerAATail = 0;
-        bool hasFill = m_color.rgba() != 0 || !stops.isEmpty();
+        bool hasFill = m_color.alpha() > 0 || !stops.isEmpty();
         if (hasFill)
             indexCount += fillIndexCount;
         if (m_antialiasing) {
@@ -609,7 +613,7 @@ void QSGDefaultRectangleNode::updateGeometry()
         int borderTail = 0;
         int outerAAHead = 0;
         int outerAATail = 0;
-        bool hasFill = m_color.rgba() != 0 || !stops.isEmpty();
+        bool hasFill = m_color.alpha() > 0 || !stops.isEmpty();
         if (hasFill)
             indexCount += fillIndexCount;
         if (m_antialiasing) {
@@ -639,11 +643,11 @@ void QSGDefaultRectangleNode::updateGeometry()
         for (int part = -1; part <= 1; part += 2) {
             float y = (part == 1 ? innerRect.bottom() : innerRect.top());
             float Y = (part == 1 ? outerRect.bottom() : outerRect.top());
-            gradientPos = (y - innerRect.top() + 0.5f * penWidth) / height;
+            gradientPos = (y - innerRect.top() + penWidth) / height;
 
             while (nextGradientStop <= lastGradientStop && stops.at(nextGradientStop).first <= gradientPos) {
                 // Insert vertices at gradient stops.
-                float gy = (innerRect.top() - 0.5f * penWidth) + stops.at(nextGradientStop).first * height;
+                float gy = (innerRect.top() - penWidth) + stops.at(nextGradientStop).first * height;
 
                 fillColor = colorToColor4ub(stops.at(nextGradientStop).second);
 
@@ -770,8 +774,6 @@ void QSGDefaultRectangleNode::updateGeometry()
             Q_ASSERT(outerAATail == indexCount);
         }
     }
-
-    markDirty(DirtyGeometry);
 }
 
 
