@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the QtQuick module of the Qt Toolkit.
 **
@@ -10,9 +10,9 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia. For licensing terms and
-** conditions see http://qt.digia.com/licensing. For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
@@ -23,8 +23,8 @@
 ** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights. These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** $QT_END_LICENSE$
@@ -49,6 +49,7 @@ class QSGThreadedRenderLoop : public QSGRenderLoop
     Q_OBJECT
 public:
     QSGThreadedRenderLoop();
+    ~QSGThreadedRenderLoop();
 
     void show(QQuickWindow *) {}
     void hide(QQuickWindow *);
@@ -60,6 +61,8 @@ public:
 
     void update(QQuickWindow *window);
     void maybeUpdate(QQuickWindow *window);
+    void handleUpdateRequest(QQuickWindow *window);
+
     QSGContext *sceneGraphContext() const;
     QSGRenderContext *createRenderContext(QSGContext *) const;
 
@@ -80,7 +83,6 @@ private:
         QQuickWindow *window;
         QSGRenderThread *thread;
         QSurfaceFormat actualWindowFormat;
-        int timerId;
         uint updateDuringSync : 1;
         uint forceRenderPass : 1;
     };
@@ -89,7 +91,6 @@ private:
 
     void releaseResources(Window *window, bool inDestructor);
     bool checkAndResetForceUpdate(QQuickWindow *window);
-    Window *windowForTimer(int timerId) const;
 
     bool anyoneShowing() const;
     void initialize();
@@ -109,7 +110,6 @@ private:
     QList<Window> m_windows;
 
     int m_animation_timer;
-    int m_exhaust_delay;
 
     bool m_lockedForSync;
 };
