@@ -1,40 +1,32 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the QtQuick module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -47,6 +39,7 @@
 #include <QtGui/QGuiApplication>
 #include <QtGui/qdesktopservices.h>
 #include <QtGui/qfontdatabase.h>
+#include <QtGui/qstylehints.h>
 
 #include <private/qv4engine_p.h>
 #include <private/qv4object_p.h>
@@ -61,23 +54,9 @@ QT_BEGIN_NAMESPACE
 class QQuickColorProvider : public QQmlColorProvider
 {
 public:
-    static inline QColor QColorFromString(const QString &s)
-    {
-        // Should we also handle #rrggbb here?
-        if (s.length() == 9 && s.startsWith(QLatin1Char('#'))) {
-            uchar a = fromHex(s, 1);
-            uchar r = fromHex(s, 3);
-            uchar g = fromHex(s, 5);
-            uchar b = fromHex(s, 7);
-            return QColor(r, g, b, a);
-        }
-
-        return QColor(s);
-    }
-
     QVariant colorFromString(const QString &s, bool *ok)
     {
-        QColor c(QColorFromString(s));
+        QColor c(s);
         if (c.isValid()) {
             if (ok) *ok = true;
             return QVariant(c);
@@ -89,7 +68,7 @@ public:
 
     unsigned rgbaFromString(const QString &s, bool *ok)
     {
-        QColor c(QColorFromString(s));
+        QColor c(s);
         if (c.isValid()) {
             if (ok) *ok = true;
             return c.rgba();
@@ -119,6 +98,11 @@ public:
         return QVariant(QColor::fromHslF(h, s, l, a));
     }
 
+    QVariant fromHsvF(double h, double s, double v, double a)
+    {
+        return QVariant(QColor::fromHsvF(h, s, v, a));
+    }
+
     QVariant lighter(const QVariant &var, qreal factor)
     {
         QColor color = var.value<QColor>();
@@ -155,34 +139,6 @@ public:
 
         return QVariant::fromValue(QColor::fromRgbF(r, g, b, a + inv_a * baseColor.alphaF()));
     }
-
-private:
-    static uchar fromHex(const uchar c, const uchar c2)
-    {
-        uchar rv = 0;
-        if (c >= '0' && c <= '9')
-            rv += (c - '0') * 16;
-        else if (c >= 'A' && c <= 'F')
-            rv += (c - 'A' + 10) * 16;
-        else if (c >= 'a' && c <= 'f')
-            rv += (c - 'a' + 10) * 16;
-
-        if (c2 >= '0' && c2 <= '9')
-            rv += (c2 - '0');
-        else if (c2 >= 'A' && c2 <= 'F')
-            rv += (c2 - 'A' + 10);
-        else if (c2 >= 'a' && c2 <= 'f')
-            rv += (c2 - 'a' + 10);
-
-        return rv;
-    }
-
-    static inline uchar fromHex(const QString &s, int idx)
-    {
-        uchar c = s.at(idx).toLatin1();
-        uchar c2 = s.at(idx + 1).toLatin1();
-        return fromHex(c, c2);
-    }
 };
 
 
@@ -311,13 +267,11 @@ public:
         return QMatrix4x4();
     }
 
-    static QFont fontFromObject(QQmlV4Handle object, QV8Engine *e, bool *ok)
+    static QFont fontFromObject(QQmlV4Handle object, QV4::ExecutionEngine *v4, bool *ok)
     {
-        Q_UNUSED(e);
-
-        if (ok) *ok = false;
+        if (ok)
+            *ok = false;
         QFont retn;
-        QV4::ExecutionEngine *v4 = QV8Engine::getV4(e);
         QV4::Scope scope(v4);
         QV4::ScopedObject obj(scope, object);
         if (!obj) {
@@ -389,16 +343,16 @@ public:
         return retn;
     }
 
-    static QMatrix4x4 matrix4x4FromObject(QQmlV4Handle object, QV8Engine *e, bool *ok)
+    static QMatrix4x4 matrix4x4FromObject(QQmlV4Handle object, QV4::ExecutionEngine *v4, bool *ok)
     {
-        if (ok) *ok = false;
-        QV4::ExecutionEngine *v4 = QV8Engine::getV4(e);
+        if (ok)
+            *ok = false;
         QV4::Scope scope(v4);
         QV4::ScopedArrayObject array(scope, object);
         if (!array)
             return QMatrix4x4();
 
-        if (array->arrayLength() != 16)
+        if (array->getLength() != 16)
             return QMatrix4x4();
 
         float matVals[16];
@@ -414,35 +368,28 @@ public:
         return QMatrix4x4(matVals);
     }
 
-    template<typename T>
-    bool typedCreate(QQmlValueType *&v)
-    {
-        v = new T;
-        return true;
-    }
-
-    bool create(int type, QQmlValueType *&v)
+    const QMetaObject *getMetaObjectForMetaType(int type)
     {
         switch (type) {
         case QMetaType::QColor:
-            return typedCreate<QQuickColorValueType>(v);
+            return &QQuickColorValueType::staticMetaObject;
         case QMetaType::QFont:
-            return typedCreate<QQuickFontValueType>(v);
+            return &QQuickFontValueType::staticMetaObject;
         case QMetaType::QVector2D:
-            return typedCreate<QQuickVector2DValueType>(v);
+            return &QQuickVector2DValueType::staticMetaObject;
         case QMetaType::QVector3D:
-            return typedCreate<QQuickVector3DValueType>(v);
+            return &QQuickVector3DValueType::staticMetaObject;
         case QMetaType::QVector4D:
-            return typedCreate<QQuickVector4DValueType>(v);
+            return &QQuickVector4DValueType::staticMetaObject;
         case QMetaType::QQuaternion:
-            return typedCreate<QQuickQuaternionValueType>(v);
+            return &QQuickQuaternionValueType::staticMetaObject;
         case QMetaType::QMatrix4x4:
-            return typedCreate<QQuickMatrix4x4ValueType>(v);
+            return &QQuickMatrix4x4ValueType::staticMetaObject;
         default:
             break;
         }
 
-        return false;
+        return 0;
     }
 
     template<typename T>
@@ -639,7 +586,7 @@ public:
 
         switch (type) {
         case QMetaType::QColor:
-            return createFromStringTyped<QColor>(data, dataSize, QQuickColorProvider::QColorFromString(s));
+            return createFromStringTyped<QColor>(data, dataSize, QColor(s));
         case QMetaType::QVector2D:
             return createFromStringTyped<QVector2D>(data, dataSize, vector2DFromString(s, &ok));
         case QMetaType::QVector3D:
@@ -677,7 +624,7 @@ public:
 
     bool variantFromString(const QString &s, QVariant *v)
     {
-        QColor c(QQuickColorProvider::QColorFromString(s));
+        QColor c(s);
         if (c.isValid()) {
             *v = QVariant::fromValue(c);
             return true;
@@ -725,7 +672,7 @@ public:
         switch (type) {
         case QMetaType::QColor:
             {
-            QColor c(QQuickColorProvider::QColorFromString(s));
+            QColor c(s);
             *v = QVariant::fromValue(c);
             return true;
             }
@@ -761,9 +708,8 @@ public:
         return false;
     }
 
-    bool variantFromJsObject(int type, QQmlV4Handle object, QV8Engine *e, QVariant *v)
+    bool variantFromJsObject(int type, QQmlV4Handle object, QV4::ExecutionEngine *v4, QVariant *v)
     {
-        QV4::ExecutionEngine *v4 = QV8Engine::getV4(e);
         QV4::Scope scope(v4);
 #ifndef QT_NO_DEBUG
         QV4::ScopedObject obj(scope, object);
@@ -772,10 +718,10 @@ public:
         bool ok = false;
         switch (type) {
         case QMetaType::QFont:
-            *v = QVariant::fromValue(fontFromObject(object, e, &ok));
+            *v = QVariant::fromValue(fontFromObject(object, v4, &ok));
             break;
         case QMetaType::QMatrix4x4:
-            *v = QVariant::fromValue(matrix4x4FromObject(object, e, &ok));
+            *v = QVariant::fromValue(matrix4x4FromObject(object, v4, &ok));
         default: break;
         }
 
@@ -986,10 +932,19 @@ public:
 #ifndef QT_NO_IM
     QInputMethod *inputMethod()
     {
-        return qGuiApp->inputMethod();
+        QInputMethod *im = qGuiApp->inputMethod();
+        QQmlEngine::setObjectOwnership(im, QQmlEngine::CppOwnership);
+        return im;
     }
 #endif
 
+    QStyleHints *styleHints()
+    {
+        QStyleHints *sh = qGuiApp->styleHints();
+        QQmlEngine::setObjectOwnership(sh, QQmlEngine::CppOwnership);
+        return sh;
+    }
+
     QStringList fontFamilies()
     {
         QFontDatabase database;
@@ -1032,4 +987,11 @@ void QQuick_initializeProviders()
     QQml_setGuiProvider(getGuiProvider());
 }
 
+void QQuick_deinitializeProviders()
+{
+    QQml_removeValueTypeProvider(getValueTypeProvider());
+    QQml_setColorProvider(0); // technically, another plugin may have overridden our providers
+    QQml_setGuiProvider(0);   // but we cannot handle that case in a sane way.
+}
+
 QT_END_NAMESPACE
