@@ -1,40 +1,32 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -187,7 +179,7 @@ private slots:
     void setBreakpointInScriptOnComment();
     void setBreakpointInScriptOnEmptyLine();
     void setBreakpointInScriptOnOptimizedBinding();
-//    void setBreakpointInScriptWithCondition(); // Not supported yet.
+    void setBreakpointInScriptWithCondition();
     void setBreakpointInScriptThatQuits();
     //void setBreakpointInFunction(); //NOT SUPPORTED
 //    void setBreakpointOnEvent();
@@ -942,7 +934,7 @@ void tst_QQmlDebugJS::setBreakpointInScriptOnCompleted()
 {
     //void setBreakpoint(QString type, QString target, int line = -1, int column = -1, bool enabled = false, QString condition = QString(), int ignoreCount = -1)
 
-    int sourceLine = 47;
+    int sourceLine = 39;
     QVERIFY(init(ONCOMPLETED_QMLFILE));
 
     client->setBreakpoint(QLatin1String(SCRIPTREGEXP), QLatin1String(ONCOMPLETED_QMLFILE), sourceLine, -1, true);
@@ -962,7 +954,7 @@ void tst_QQmlDebugJS::setBreakpointInScriptOnComponentCreated()
 {
     //void setBreakpoint(QString type, QString target, int line = -1, int column = -1, bool enabled = false, QString condition = QString(), int ignoreCount = -1)
 
-    int sourceLine = 47;
+    int sourceLine = 39;
     QVERIFY(init(CREATECOMPONENT_QMLFILE));
 
     client->setBreakpoint(QLatin1String(SCRIPTREGEXP), QLatin1String(ONCOMPLETED_QMLFILE), sourceLine, -1, true);
@@ -980,11 +972,12 @@ void tst_QQmlDebugJS::setBreakpointInScriptOnComponentCreated()
 
 void tst_QQmlDebugJS::setBreakpointInScriptOnTimerCallback()
 {
-    int sourceLine = 48;
+    int sourceLine = 40;
     QVERIFY(init(TIMER_QMLFILE));
 
     client->connect();
-
+    //We can set the breakpoint after connect() here because the timer is repeating and if we miss
+    //its first iteration we can still catch the second one.
     client->setBreakpoint(QLatin1String(SCRIPTREGEXP), QLatin1String(TIMER_QMLFILE), sourceLine, -1, true);
     QVERIFY(QQmlDebugTest::waitForSignal(client, SIGNAL(stopped())));
 
@@ -1001,11 +994,11 @@ void tst_QQmlDebugJS::setBreakpointInScriptInDifferentFile()
 {
     //void setBreakpoint(QString type, QString target, int line = -1, int column = -1, bool enabled = false, QString condition = QString(), int ignoreCount = -1)
 
-    int sourceLine = 43;
+    int sourceLine = 35;
     QVERIFY(init(LOADJSFILE_QMLFILE));
 
-    client->connect();
     client->setBreakpoint(QLatin1String(SCRIPTREGEXP), QLatin1String(TEST_JSFILE), sourceLine, -1, true);
+    client->connect();
     QVERIFY(QQmlDebugTest::waitForSignal(client, SIGNAL(stopped())));
 
     QString jsonString(client->response);
@@ -1021,12 +1014,12 @@ void tst_QQmlDebugJS::setBreakpointInScriptOnComment()
 {
     //void setBreakpoint(QString type, QString target, int line = -1, int column = -1, bool enabled = false, QString condition = QString(), int ignoreCount = -1)
 
-    int sourceLine = 47;
-    int actualLine = 49;
+    int sourceLine = 39;
+    int actualLine = 41;
     QVERIFY(init(BREAKPOINTRELOCATION_QMLFILE));
 
-    client->connect();
     client->setBreakpoint(QLatin1String(SCRIPTREGEXP), QLatin1String(BREAKPOINTRELOCATION_QMLFILE), sourceLine, -1, true);
+    client->connect();
     QEXPECT_FAIL("", "Relocation of breakpoints is disabled right now", Abort);
     QVERIFY(QQmlDebugTest::waitForSignal(client, SIGNAL(stopped()), 1));
 
@@ -1043,12 +1036,12 @@ void tst_QQmlDebugJS::setBreakpointInScriptOnEmptyLine()
 {
     //void setBreakpoint(QString type, QString target, int line = -1, int column = -1, bool enabled = false, QString condition = QString(), int ignoreCount = -1)
 
-    int sourceLine = 48;
-    int actualLine = 49;
+    int sourceLine = 40;
+    int actualLine = 41;
     QVERIFY(init(BREAKPOINTRELOCATION_QMLFILE));
 
-    client->connect();
     client->setBreakpoint(QLatin1String(SCRIPTREGEXP), QLatin1String(BREAKPOINTRELOCATION_QMLFILE), sourceLine, -1, true);
+    client->connect();
     QEXPECT_FAIL("", "Relocation of breakpoints is disabled right now", Abort);
     QVERIFY(QQmlDebugTest::waitForSignal(client, SIGNAL(stopped()), 1));
 
@@ -1065,7 +1058,7 @@ void tst_QQmlDebugJS::setBreakpointInScriptOnOptimizedBinding()
 {
     //void setBreakpoint(QString type, QString target, int line = -1, int column = -1, bool enabled = false, QString condition = QString(), int ignoreCount = -1)
 
-    int sourceLine = 52;
+    int sourceLine = 44;
     QVERIFY(init(BREAKPOINTRELOCATION_QMLFILE));
 
     client->setBreakpoint(QLatin1String(SCRIPTREGEXP), QLatin1String(BREAKPOINTRELOCATION_QMLFILE), sourceLine, -1, true);
@@ -1081,18 +1074,14 @@ void tst_QQmlDebugJS::setBreakpointInScriptOnOptimizedBinding()
     QCOMPARE(QFileInfo(body.value("script").toMap().value("name").toString()).fileName(), QLatin1String(BREAKPOINTRELOCATION_QMLFILE));
 }
 
-#if 0
 void tst_QQmlDebugJS::setBreakpointInScriptWithCondition()
 {
-    QFAIL("conditional breakpoints are not yet supported");
-
-    //void setBreakpoint(QString type, QString target, int line = -1, int column = -1, bool enabled = false, QString condition = QString(), int ignoreCount = -1)
-
     int out = 10;
-    int sourceLine = 50;
+    int sourceLine = 42;
     QVERIFY(init(CONDITION_QMLFILE));
 
     client->connect();
+    //The breakpoint is in a timer loop so we can set it after connect().
     client->setBreakpoint(QLatin1String(SCRIPTREGEXP), QLatin1String(CONDITION_QMLFILE), sourceLine, 1, true, QLatin1String("a > 10"));
     QVERIFY(QQmlDebugTest::waitForSignal(client, SIGNAL(stopped())));
 
@@ -1100,29 +1089,32 @@ void tst_QQmlDebugJS::setBreakpointInScriptWithCondition()
     QString jsonString = client->response;
     QVariantMap value = client->parser.call(QJSValueList() << QJSValue(jsonString)).toVariant().toMap();
 
-    QVariantMap body = value.value("body").toMap();
+    {
+        QVariantMap body = value.value("body").toMap();
+        int frameIndex = body.value("index").toInt();
 
-    int frameIndex = body.value("index").toInt();
-
-    //Verify the value of 'result'
-    client->evaluate(QLatin1String("a"),frameIndex);
-
-    QVERIFY(QQmlDebugTest::waitForSignal(client, SIGNAL(result())));
+        //Verify the value of 'result'
+        client->evaluate(QLatin1String("a"),frameIndex);
+        QVERIFY(QQmlDebugTest::waitForSignal(client, SIGNAL(result())));
+    }
 
     jsonString = client->response;
-    value = client->parser.call(QJSValueList() << QJSValue(jsonString)).toVariant().toMap();
+    QJSValue val = client->parser.call(QJSValueList() << QJSValue(jsonString));
+    QVERIFY(val.isObject());
+    QJSValue body = val.property(QStringLiteral("body"));
+    QVERIFY(body.isObject());
+    val = body.property("value");
+    QVERIFY(val.isNumber());
 
-    body = value.value("body").toMap();
-
-    QVERIFY(body.value("value").toInt() > out);
+    const int a = val.toInt();
+    QVERIFY(a > out);
 }
-#endif
 
 void tst_QQmlDebugJS::setBreakpointInScriptThatQuits()
 {
     QVERIFY(init(QUIT_QMLFILE));
 
-    int sourceLine = 49;
+    int sourceLine = 41;
 
     client->setBreakpoint(QLatin1String(SCRIPTREGEXP), QLatin1String(QUIT_QMLFILE), sourceLine, -1, true);
     client->connect();
@@ -1148,31 +1140,12 @@ void tst_QQmlDebugJS::setBreakpointWhenAttaching()
     QVERIFY(init(QLatin1String(TIMER_QMLFILE), false));
 
     client->connect();
+    //The breakpoint is in a timer loop so we can set it after connect().
     client->setBreakpoint(QLatin1String(SCRIPTREGEXP), QLatin1String(TIMER_QMLFILE), sourceLine);
     QVERIFY(QQmlDebugTest::waitForSignal(client, SIGNAL(stopped())));
 }
 */
 
-//void tst_QQmlDebugJS::setBreakpointInFunction()
-//{
-//    //void setBreakpoint(QString type, QString target, int line = -1, int column = -1, bool enabled = false, QString condition = QString(), int ignoreCount = -1)
-
-//    int actualLine = 31;
-
-//    client->connect();
-//    client->setBreakpoint(QLatin1String(FUNCTION), QLatin1String("doSomethingElse"), -1, -1, true);
-
-//    QVERIFY(QQmlDebugTest::waitForSignal(client, SIGNAL(stopped())));
-
-//    QString jsonString(client->response);
-//    QVariantMap value = client->parser.call(QJSValueList() << QJSValue(jsonString)).toVariant().toMap();
-
-//    QVariantMap body = value.value("body").toMap();
-
-//    QCOMPARE(body.value("sourceLine").toInt(), actualLine);
-//    QCOMPARE(QFileInfo(body.value("script").toMap().value("name").toString()).fileName(), QLatin1String(QMLFILE));
-//}
-
 #if 0
 void tst_QQmlDebugJS::setBreakpointOnEvent()
 {
@@ -1182,9 +1155,8 @@ void tst_QQmlDebugJS::setBreakpointOnEvent()
 
     QVERIFY(init(TIMER_QMLFILE));
 
-    client->connect();
-
     client->setBreakpoint(QLatin1String(EVENT), QLatin1String("triggered"), -1, -1, true);
+    client->connect();
     QVERIFY(QQmlDebugTest::waitForSignal(client, SIGNAL(stopped())));
 
     QString jsonString(client->response);
@@ -1200,11 +1172,14 @@ void tst_QQmlDebugJS::clearBreakpoint()
 {
     //void clearBreakpoint(int breakpoint);
 
-    int sourceLine1 = 50;
-    int sourceLine2 = 51;
+    int sourceLine1 = 42;
+    int sourceLine2 = 43;
     QVERIFY(init(CHANGEBREAKPOINT_QMLFILE));
 
     client->connect();
+    //The breakpoints are in a timer loop so we can set them after connect().
+    //Furthermore the breakpoints should be hit in the right order because setting of breakpoints
+    //can only occur in the QML event loop. (see QCOMPARE for sourceLine2 below)
     client->setBreakpoint(QLatin1String(SCRIPTREGEXP), QLatin1String(CHANGEBREAKPOINT_QMLFILE), sourceLine1, -1, true);
     client->setBreakpoint(QLatin1String(SCRIPTREGEXP), QLatin1String(CHANGEBREAKPOINT_QMLFILE), sourceLine2, -1, true);
 
@@ -1254,7 +1229,7 @@ void tst_QQmlDebugJS::stepNext()
 {
     //void continueDebugging(StepAction stepAction, int stepCount = 1);
 
-    int sourceLine = 50;
+    int sourceLine = 42;
     QVERIFY(init(STEPACTION_QMLFILE));
 
     client->setBreakpoint(QLatin1String(SCRIPTREGEXP), QLatin1String(STEPACTION_QMLFILE), sourceLine, -1, true);
@@ -1277,12 +1252,12 @@ void tst_QQmlDebugJS::stepIn()
 {
     //void continueDebugging(StepAction stepAction, int stepCount = 1);
 
-    int sourceLine = 54;
-    int actualLine = 50;
+    int sourceLine = 46;
+    int actualLine = 42;
     QVERIFY(init(STEPACTION_QMLFILE));
 
-    client->connect();
     client->setBreakpoint(QLatin1String(SCRIPTREGEXP), QLatin1String(STEPACTION_QMLFILE), sourceLine, 1, true);
+    client->connect();
     QVERIFY(QQmlDebugTest::waitForSignal(client, SIGNAL(stopped())));
 
     client->continueDebugging(QJSDebugClient::In);
@@ -1301,12 +1276,12 @@ void tst_QQmlDebugJS::stepOut()
 {
     //void continueDebugging(StepAction stepAction, int stepCount = 1);
 
-    int sourceLine = 50;
-    int actualLine = 54;
+    int sourceLine = 42;
+    int actualLine = 46;
     QVERIFY(init(STEPACTION_QMLFILE));
 
-    client->connect();
     client->setBreakpoint(QLatin1String(SCRIPTREGEXP), QLatin1String(STEPACTION_QMLFILE), sourceLine, -1, true);
+    client->connect();
     QVERIFY(QQmlDebugTest::waitForSignal(client, SIGNAL(stopped())));
 
     client->continueDebugging(QJSDebugClient::Out);
@@ -1325,13 +1300,13 @@ void tst_QQmlDebugJS::continueDebugging()
 {
     //void continueDebugging(StepAction stepAction, int stepCount = 1);
 
-    int sourceLine1 = 54;
-    int sourceLine2 = 51;
+    int sourceLine1 = 46;
+    int sourceLine2 = 43;
     QVERIFY(init(STEPACTION_QMLFILE));
 
-    client->connect();
     client->setBreakpoint(QLatin1String(SCRIPTREGEXP), QLatin1String(STEPACTION_QMLFILE), sourceLine1, -1, true);
     client->setBreakpoint(QLatin1String(SCRIPTREGEXP), QLatin1String(STEPACTION_QMLFILE), sourceLine2, -1, true);
+    client->connect();
     QVERIFY(QQmlDebugTest::waitForSignal(client, SIGNAL(stopped())));
 
     client->continueDebugging(QJSDebugClient::Continue);
@@ -1350,7 +1325,7 @@ void tst_QQmlDebugJS::backtrace()
 {
     //void backtrace(int fromFrame = -1, int toFrame = -1, bool bottom = false);
 
-    int sourceLine = 47;
+    int sourceLine = 39;
     QVERIFY(init(ONCOMPLETED_QMLFILE));
 
     client->setBreakpoint(QLatin1String(SCRIPTREGEXP), QLatin1String(ONCOMPLETED_QMLFILE), sourceLine, -1, true);
@@ -1365,7 +1340,7 @@ void tst_QQmlDebugJS::getFrameDetails()
 {
     //void frame(int number = -1);
 
-    int sourceLine = 47;
+    int sourceLine = 39;
     QVERIFY(init(ONCOMPLETED_QMLFILE));
 
     client->setBreakpoint(QLatin1String(SCRIPTREGEXP), QLatin1String(ONCOMPLETED_QMLFILE), sourceLine, -1, true);
@@ -1380,7 +1355,7 @@ void tst_QQmlDebugJS::getScopeDetails()
 {
     //void scope(int number = -1, int frameNumber = -1);
 
-    int sourceLine = 47;
+    int sourceLine = 39;
     QVERIFY(init(ONCOMPLETED_QMLFILE));
 
     client->setBreakpoint(QLatin1String(SCRIPTREGEXP), QLatin1String(ONCOMPLETED_QMLFILE), sourceLine, -1, true);
@@ -1454,7 +1429,7 @@ void tst_QQmlDebugJS::getScripts()
 
     QVERIFY(init());
 
-    client->setBreakpoint(QLatin1String(SCRIPTREGEXP), QString(TEST_QMLFILE), 48, -1, true);
+    client->setBreakpoint(QLatin1String(SCRIPTREGEXP), QString(TEST_QMLFILE), 40, -1, true);
     client->connect();
     QVERIFY(QQmlDebugTest::waitForSignal(client, SIGNAL(stopped())));
 
