@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
@@ -10,9 +10,9 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia. For licensing terms and
-** conditions see http://qt.digia.com/licensing. For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
@@ -23,8 +23,8 @@
 ** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights. These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** $QT_END_LICENSE$
@@ -84,6 +84,8 @@ QQmlDebugProcess::QQmlDebugProcess(const QString &executable, QObject *parent)
     , m_executable(executable)
     , m_started(false)
     , m_port(0)
+    , m_maximumBindErrors(0)
+    , m_receivedBindErrors(0)
 {
     m_process.setProcessChannelMode(QProcess::MergedChannels);
     m_timer.setSingleShot(true);
@@ -152,6 +154,11 @@ void QQmlDebugProcess::stop()
     }
 }
 
+void QQmlDebugProcess::setMaximumBindErrors(int ignore)
+{
+    m_maximumBindErrors = ignore;
+}
+
 void QQmlDebugProcess::timeout()
 {
     qWarning() << "Timeout while waiting for QML debugging messages "
@@ -222,10 +229,13 @@ void QQmlDebugProcess::processAppOutput()
                 continue;
             }
             if (line.contains("Unable to listen")) {
-                qWarning() << "App was unable to bind to port!";
-                m_timer.stop();
-                m_eventLoop.quit();
-                continue;
+                if (++m_receivedBindErrors >= m_maximumBindErrors) {
+                    if (m_maximumBindErrors == 0)
+                        qWarning() << "App was unable to bind to port!";
+                    m_timer.stop();
+                    m_eventLoop.quit();
+                 }
+                 continue;
             }
         } else {
             // set to true if there is output not coming from the debugger
