@@ -1,40 +1,32 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -55,6 +47,8 @@ public:
 private slots:
     void scriptString();
     void syntaxError();
+    void exception();
+    void expressionFromDataComponent();
 };
 
 class TestObject : public QObject
@@ -116,6 +110,40 @@ void tst_qqmlexpression::syntaxError()
     QCOMPARE(v, QVariant());
 }
 
+void tst_qqmlexpression::exception()
+{
+    QQmlEngine engine;
+    QQmlExpression expression(engine.rootContext(), 0, "abc=123");
+    QVariant v = expression.evaluate();
+    QCOMPARE(v, QVariant());
+    QVERIFY(expression.hasError());
+}
+
+void tst_qqmlexpression::expressionFromDataComponent()
+{
+    qmlRegisterType<TestObject>("Test", 1, 0, "TestObject");
+
+    QQmlEngine engine;
+    QQmlComponent c(&engine);
+
+    QUrl url = testFileUrl("expressionFromDataComponent.qml");
+
+    {
+        QFile f(url.toLocalFile());
+        QVERIFY(f.open(QIODevice::ReadOnly));
+        c.setData(f.readAll(), url);
+    }
+
+    QScopedPointer<TestObject> object;
+    object.reset(qobject_cast<TestObject*>(c.create()));
+    Q_ASSERT(!object.isNull());
+
+    QQmlExpression expression(object->scriptString());
+    QVariant result = expression.evaluate();
+    QVERIFY(result.type() == QVariant::String);
+    QCOMPARE(result.toString(), QStringLiteral("success"));
+}
+
 QTEST_MAIN(tst_qqmlexpression)
 
 #include "tst_qqmlexpression.moc"
