@@ -1,40 +1,32 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -44,12 +36,14 @@
 #include <QtQml/qqmlcomponent.h>
 #include <QDebug>
 
+#if defined(Q_OS_MAC)
+// For _PC_CASE_SENSITIVE
+#include <unistd.h>
+#endif
+
 #include "../../shared/testhttpserver.h"
 #include "../../shared/util.h"
 
-#define SERVER_ADDR "http://127.0.0.1:14456"
-#define SERVER_PORT 14456
-
 // Note: this test does not use module identifier directives in the qmldir files, because
 // it would result in repeated attempts to insert types into the same namespace.
 // This occurs because type registration is process-global, while the test
@@ -145,7 +139,7 @@ void tst_qqmlmoduleplugin::importsPlugin()
     QTest::ignoreMessage(QtWarningMsg, "Module 'org.qtproject.AutoTestQmlPluginType' does not contain a module identifier directive - it cannot be protected from external registrations.");
     QQmlComponent component(&engine, testFileUrl(QStringLiteral("works.qml")));
     foreach (QQmlError err, component.errors())
-    	qWarning() << err;
+        qWarning() << err;
     VERIFY_ERRORS(0);
     QObject *object = component.create();
     QVERIFY(object != 0);
@@ -242,13 +236,14 @@ void tst_qqmlmoduleplugin::importPluginWithQmlFile()
 
 void tst_qqmlmoduleplugin::remoteImportWithQuotedUrl()
 {
-    TestHTTPServer server(SERVER_PORT);
-    QVERIFY(server.isValid());
+    TestHTTPServer server;
+    QVERIFY2(server.listen(), qPrintable(server.errorString()));
     server.serveDirectory(m_dataImportsDirectory);
 
     QQmlEngine engine;
     QQmlComponent component(&engine);
-    component.setData("import \"" SERVER_ADDR "/org/qtproject/PureQmlModule\" \nComponentA { width: 300; ComponentB{} }", QUrl());
+    const QString qml = "import \"" + server.urlString("/org/qtproject/PureQmlModule") + "\" \nComponentA { width: 300; ComponentB{} }";
+    component.setData(qml.toUtf8(), QUrl());
 
     QTRY_COMPARE(component.status(), QQmlComponent::Ready);
     QObject *object = component.create();
@@ -263,8 +258,8 @@ void tst_qqmlmoduleplugin::remoteImportWithQuotedUrl()
 
 void tst_qqmlmoduleplugin::remoteImportWithUnquotedUri()
 {
-    TestHTTPServer server(SERVER_PORT);
-    QVERIFY(server.isValid());
+    TestHTTPServer server;
+    QVERIFY2(server.listen(), qPrintable(server.errorString()));
     server.serveDirectory(m_dataImportsDirectory);
 
     QQmlEngine engine;
@@ -511,6 +506,27 @@ void tst_qqmlmoduleplugin::importStrictModule_data()
         << QString()
         << QString();
 
+    QTest::newRow("two strict modules with different major version")
+        << "import org.qtproject.StrictModule 1.0\n"
+           "import org.qtproject.StrictModule 2.0\n"
+           "MyPluginType {}"
+        << QString()
+        << QString();
+
+    QTest::newRow("old namespaced strict module")
+        << "import org.qtproject.StrictModule 1.0 as Old\n"
+           "import org.qtproject.StrictModule 2.0 as New\n"
+           "Old.MyPluginType {}"
+        << QString()
+        << QString();
+
+    QTest::newRow("new namespaced strict modules")
+        << "import org.qtproject.StrictModule 1.0 as Old\n"
+           "import org.qtproject.StrictModule 2.0 as New\n"
+           "New.MyPluginType {}"
+        << QString()
+        << QString();
+
     QTest::newRow("wrong target")
         << "import org.qtproject.InvalidStrictModule 1.0\n"
            "MyPluginType {}"
