@@ -1,40 +1,32 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -58,9 +50,6 @@
 #include "../../shared/testhttpserver.h"
 #include "../../shared/util.h"
 
-#define SERVER_PORT 14446
-#define SERVER_ADDR "http://127.0.0.1:14446"
-
 Q_DECLARE_METATYPE(QQuickImageBase::Status)
 
 class tst_qquickborderimage : public QQmlDataTest
@@ -131,10 +120,10 @@ void tst_qquickborderimage::imageSource_data()
 
     QTest::newRow("local") << testFileUrl("colors.png").toString() << false << "";
     QTest::newRow("local not found") << testFileUrl("no-such-file.png").toString() << false
-        << "file::2:1: QML BorderImage: Cannot open: " + testFileUrl("no-such-file.png").toString();
-    QTest::newRow("remote") << SERVER_ADDR "/colors.png" << true << "";
-    QTest::newRow("remote not found") << SERVER_ADDR "/no-such-file.png" << true
-        << "file::2:1: QML BorderImage: Error downloading " SERVER_ADDR "/no-such-file.png - server replied: Not found";
+        << "<Unknown File>:2:1: QML BorderImage: Cannot open: " + testFileUrl("no-such-file.png").toString();
+    QTest::newRow("remote") << "/colors.png" << true << "";
+    QTest::newRow("remote not found") << "/no-such-file.png" << true
+        << "<Unknown File>:2:1: QML BorderImage: Error downloading {{ServerBaseUrl}}/no-such-file.png - server replied: Not found";
 }
 
 void tst_qquickborderimage::imageSource()
@@ -143,11 +132,12 @@ void tst_qquickborderimage::imageSource()
     QFETCH(bool, remote);
     QFETCH(QString, error);
 
-    TestHTTPServer *server = 0;
+    TestHTTPServer server;
     if (remote) {
-        server = new TestHTTPServer(SERVER_PORT);
-        QVERIFY(server->isValid());
-        server->serveDirectory(dataDirectory());
+        QVERIFY2(server.listen(), qPrintable(server.errorString()));
+        server.serveDirectory(dataDirectory());
+        source = server.urlString(source);
+        error.replace(QStringLiteral("{{ServerBaseUrl}}"), server.baseUrl().toString());
     }
 
     if (!error.isEmpty())
@@ -177,7 +167,6 @@ void tst_qquickborderimage::imageSource()
     }
 
     delete obj;
-    delete server;
 }
 
 void tst_qquickborderimage::clearSource()
@@ -290,13 +279,14 @@ void tst_qquickborderimage::sciSource()
 {
     QFETCH(QString, source);
     QFETCH(bool, valid);
+    QFETCH(bool, remote);
 
-    bool remote = source.startsWith("http");
-    TestHTTPServer *server = 0;
+    TestHTTPServer server;
     if (remote) {
-        server = new TestHTTPServer(SERVER_PORT);
-        QVERIFY(server->isValid());
-        server->serveDirectory(dataDirectory());
+        QVERIFY2(server.listen(), qPrintable(server.errorString()));
+        server.serveDirectory(dataDirectory());
+        source = server.urlString(source);
+        server.registerFileNameForContentSubstitution(QUrl(source).path());
     }
 
     QString componentStr = "import QtQuick 2.0\nBorderImage { source: \"" + source + "\"; width: 300; height: 300 }";
@@ -325,21 +315,21 @@ void tst_qquickborderimage::sciSource()
     }
 
     delete obj;
-    delete server;
 }
 
 void tst_qquickborderimage::sciSource_data()
 {
     QTest::addColumn<QString>("source");
     QTest::addColumn<bool>("valid");
+    QTest::addColumn<bool>("remote");
 
-    QTest::newRow("local") << testFileUrl("colors-round.sci").toString() << true;
-    QTest::newRow("local quoted filename") << testFileUrl("colors-round-quotes.sci").toString() << true;
-    QTest::newRow("local not found") << testFileUrl("no-such-file.sci").toString() << false;
-    QTest::newRow("remote") << SERVER_ADDR "/colors-round.sci" << true;
-    QTest::newRow("remote filename quoted") << SERVER_ADDR "/colors-round-quotes.sci" << true;
-    QTest::newRow("remote image") << SERVER_ADDR "/colors-round-remote.sci" << true;
-    QTest::newRow("remote not found") << SERVER_ADDR "/no-such-file.sci" << false;
+    QTest::newRow("local") << testFileUrl("colors-round.sci").toString() << true << /*remote*/false;
+    QTest::newRow("local quoted filename") << testFileUrl("colors-round-quotes.sci").toString() << true << /*remote*/false;
+    QTest::newRow("local not found") << testFileUrl("no-such-file.sci").toString() << false << /*remote*/false;
+    QTest::newRow("remote") << "/colors-round.sci" << true << /*remote*/true;
+    QTest::newRow("remote filename quoted") << "/colors-round-quotes.sci" << true << /*remote*/true;
+    QTest::newRow("remote image") << "/colors-round-remote.sci" << true << /*remote*/true;
+    QTest::newRow("remote not found") << "/no-such-file.sci" << false << /*remote*/true;
 }
 
 void tst_qquickborderimage::invalidSciFile()
@@ -425,7 +415,7 @@ void tst_qquickborderimage::statusChanges_data()
     QTest::newRow("nofile") << "" << 0 << false << QQuickImageBase::Null;
     QTest::newRow("nonexistent") << testFileUrl("thisfiledoesnotexist.png").toString() << 1 << false << QQuickImageBase::Error;
     QTest::newRow("noprotocol") << QString("thisfiledoesnotexisteither.png") << 2 << false << QQuickImageBase::Error;
-    QTest::newRow("remote") << "http://localhost:14446/colors.png" << 2 << true << QQuickImageBase::Ready;
+    QTest::newRow("remote") << "/colors.png" << 2 << true << QQuickImageBase::Ready;
 }
 
 void tst_qquickborderimage::statusChanges()
@@ -435,11 +425,11 @@ void tst_qquickborderimage::statusChanges()
     QFETCH(bool, remote);
     QFETCH(QQuickImageBase::Status, finalStatus);
 
-    TestHTTPServer *server = 0;
+    TestHTTPServer server;
     if (remote) {
-        server = new TestHTTPServer(SERVER_PORT);
-        QVERIFY(server->isValid());
-        server->serveDirectory(dataDirectory(), TestHTTPServer::Delay);
+        QVERIFY2(server.listen(), qPrintable(server.errorString()));
+        server.serveDirectory(dataDirectory());
+        source = server.urlString(source);
     }
 
     QString componentStr = "import QtQuick 2.0\nBorderImage { width: 300; height: 300 }";
@@ -452,18 +442,17 @@ void tst_qquickborderimage::statusChanges()
     QVERIFY(obj != 0);
     obj->setSource(source);
     if (remote)
-        server->sendDelayedItem();
+        server.sendDelayedItem();
     QTRY_VERIFY(obj->status() == finalStatus);
     QCOMPARE(spy.count(), emissions);
 
     delete obj;
-    delete server;
 }
 
 void tst_qquickborderimage::sourceSizeChanges()
 {
-    TestHTTPServer server(14449);
-    QVERIFY(server.isValid());
+    TestHTTPServer server;
+    QVERIFY2(server.listen(), qPrintable(server.errorString()));
     server.serveDirectory(dataDirectory());
 
     QQmlEngine engine;
@@ -503,19 +492,19 @@ void tst_qquickborderimage::sourceSizeChanges()
     QTRY_COMPARE(sourceSizeSpy.count(), 3);
 
     // Remote
-    ctxt->setContextProperty("srcImage", QUrl("http://127.0.0.1:14449/heart200.png"));
+    ctxt->setContextProperty("srcImage", server.url("/heart200.png"));
     QTRY_COMPARE(obj->status(), QQuickBorderImage::Ready);
     QTRY_COMPARE(sourceSizeSpy.count(), 4);
 
-    ctxt->setContextProperty("srcImage", QUrl("http://127.0.0.1:14449/heart200.png"));
+    ctxt->setContextProperty("srcImage", server.url("/heart200.png"));
     QTRY_COMPARE(obj->status(), QQuickBorderImage::Ready);
     QTRY_COMPARE(sourceSizeSpy.count(), 4);
 
-    ctxt->setContextProperty("srcImage", QUrl("http://127.0.0.1:14449/heart200_copy.png"));
+    ctxt->setContextProperty("srcImage", server.url("/heart200_copy.png"));
     QTRY_COMPARE(obj->status(), QQuickBorderImage::Ready);
     QTRY_COMPARE(sourceSizeSpy.count(), 4);
 
-    ctxt->setContextProperty("srcImage", QUrl("http://127.0.0.1:14449/colors.png"));
+    ctxt->setContextProperty("srcImage", server.url("/colors.png"));
     QTRY_COMPARE(obj->status(), QQuickBorderImage::Ready);
     QTRY_COMPARE(sourceSizeSpy.count(), 5);
 
@@ -528,8 +517,8 @@ void tst_qquickborderimage::sourceSizeChanges()
 
 void tst_qquickborderimage::progressAndStatusChanges()
 {
-    TestHTTPServer server(14449);
-    QVERIFY(server.isValid());
+    TestHTTPServer server;
+    QVERIFY2(server.listen(), qPrintable(server.errorString()));
     server.serveDirectory(dataDirectory());
 
     QQmlEngine engine;
@@ -544,7 +533,7 @@ void tst_qquickborderimage::progressAndStatusChanges()
     QTRY_VERIFY(obj->progress() == 1.0);
 
     qRegisterMetaType<QQuickBorderImage::Status>();
-    QSignalSpy sourceSpy(obj, SIGNAL(sourceChanged(const QUrl &)));
+    QSignalSpy sourceSpy(obj, SIGNAL(sourceChanged(QUrl)));
     QSignalSpy progressSpy(obj, SIGNAL(progressChanged(qreal)));
     QSignalSpy statusSpy(obj, SIGNAL(statusChanged(QQuickImageBase::Status)));
 
@@ -565,7 +554,7 @@ void tst_qquickborderimage::progressAndStatusChanges()
     QTRY_COMPARE(statusSpy.count(), 1);
 
     // Loading remote file
-    ctxt->setContextProperty("srcImage", "http://127.0.0.1:14449/heart200.png");
+    ctxt->setContextProperty("srcImage", server.url("/heart200.png"));
     QTRY_VERIFY(obj->status() == QQuickBorderImage::Loading);
     QTRY_VERIFY(obj->progress() == 0.0);
     QTRY_VERIFY(obj->status() == QQuickBorderImage::Ready);
